@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -79,6 +79,7 @@ public abstract class OperationSupportImpl {
     private static final OperationSupportImpl FOR_DIRECT_UNINSTALL = new ForDirectUninstall();
     private static final OperationSupportImpl FOR_CUSTOM_INSTALL = new ForCustomInstall ();
     private static final OperationSupportImpl FOR_CUSTOM_UNINSTALL = new ForCustomUninstall ();
+    private static final OperationSupportImpl FOR_INTERNAL_UPDATE = new ForInternalUpdate();
     
     private static final Logger LOGGER = Logger.getLogger ("org.netbeans.modules.autoupdate.services.OperationSupportImpl");
     
@@ -109,6 +110,9 @@ public abstract class OperationSupportImpl {
     public static OperationSupportImpl forCustomUninstall () {
         return FOR_CUSTOM_UNINSTALL;
     }
+    public static OperationSupportImpl forInternalUpdate () {
+        return FOR_INTERNAL_UPDATE;
+    }
     
     public abstract Boolean doOperation(ProgressHandle progress/*or null*/, OperationContainer<?> container) throws OperationException;
     public abstract void doCancel () throws OperationException;
@@ -551,6 +555,25 @@ public abstract class OperationSupportImpl {
         }
         
     }
+
+    private static class ForInternalUpdate extends OperationSupportImpl {
+        public synchronized Boolean doOperation(ProgressHandle progress,
+                OperationContainer container) throws OperationException {
+            throw new UnsupportedOperationException("Not supported yet.");
+        }
+        public void doCancel () throws OperationException {
+            assert false : "Not supported yet";
+        }
+
+        public void doRestart (Restarter restarter, ProgressHandle progress) throws OperationException {
+            throw new UnsupportedOperationException ("Not supported yet.");
+        }
+
+        public void doRestartLater (Restarter restarter) {
+            throw new UnsupportedOperationException ("Not supported yet.");
+        }
+
+    }
     
     private static class ForCustomInstall extends OperationSupportImpl {
         private Collection<UpdateElement> affectedModules = null;
@@ -612,7 +635,7 @@ public abstract class OperationSupportImpl {
         }
 
         public void doRestart (Restarter restarter, ProgressHandle progress) throws OperationException {
-            createRestartMarker();
+            markForRestart();
             LifecycleManager.getDefault ().exit ();
             // if exit&restart fails => use restart later as fallback
             doRestartLater (restarter);
@@ -620,24 +643,13 @@ public abstract class OperationSupportImpl {
 
         public void doRestartLater (Restarter restarter) {
             // shedule module for restart
-            createRestartMarker();
+            markForRestart();
             if(affectedModules!=null) {
             for (UpdateElement el : affectedModules) {
                 UpdateUnitFactory.getDefault().scheduleForRestart (el);
             }
             }
         }
-        private void createRestartMarker() {
-            try {
-                File targetUserdir = new File(System.getProperty("netbeans.user")); // NOI18N
-                File restartFile = new File(targetUserdir, "var/restart");//NOI18N
-                if(!restartFile.exists()) {
-                    restartFile.createNewFile();
-                }
-            } catch (IOException ex) {
-                LOGGER.log(Level.INFO, "Can`t create restart file marker", ex);
-            }
-        }
     }
 
     private static class ForCustomUninstall extends OperationSupportImpl {
@@ -691,7 +703,7 @@ public abstract class OperationSupportImpl {
         }
 
         public void doRestart (Restarter restarter, ProgressHandle progress) throws OperationException {
-            createRestartMarker();
+            markForRestart();
             LifecycleManager.getDefault ().exit ();
             // if exit&restart fails => use restart later as fallback
             doRestartLater (restarter);
@@ -699,24 +711,21 @@ public abstract class OperationSupportImpl {
 
         public void doRestartLater (Restarter restarter) {
             // shedule module for restart
-            createRestartMarker();
+            markForRestart();
             if(affectedModules!=null) {
             for (UpdateElement el : affectedModules) {
                 UpdateUnitFactory.getDefault().scheduleForRestart (el);
             }
             }
         }
-        private void createRestartMarker() {
-            try {
-                File targetUserdir = new File(System.getProperty("netbeans.user")); // NOI18N
-                File restartFile = new File(targetUserdir, "var/restart");//NOI18N
-                if(!restartFile.exists()) {
-                    restartFile.createNewFile();
-                }
-            } catch (IOException ex) {
-                LOGGER.log(Level.INFO, "Can`t create restart file marker", ex);
-            }
+    }
 
+    private static void markForRestart() {
+        try {
+            LifecycleManager.getDefault().markForRestart();
+        } catch (UnsupportedOperationException x) {
+            LOGGER.log(Level.INFO, null, x);
         }
     }
+
 }
