@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -66,6 +66,7 @@ import org.openide.modules.ModuleInfo;
  */
 abstract class OperationValidator {
     private final static OperationValidator FOR_INSTALL = new InstallValidator();
+    private final static OperationValidator FOR_INTERNAL_UPDATE = new InternalUpdateValidator();
     private final static OperationValidator FOR_UNINSTALL = new UninstallValidator();
     private final static OperationValidator FOR_UPDATE = new UpdateValidator();
     private final static OperationValidator FOR_ENABLE = new EnableValidator();
@@ -86,6 +87,9 @@ abstract class OperationValidator {
         case INSTALL:
             isValid = FOR_INSTALL.isValidOperationImpl(updateUnit, updateElement);
             break;
+        case INTERNAL_UPDATE:
+            isValid = FOR_INTERNAL_UPDATE.isValidOperationImpl(updateUnit, updateElement);
+            break;
         case DIRECT_UNINSTALL:
         case UNINSTALL:
             isValid = FOR_UNINSTALL.isValidOperationImpl(updateUnit, updateElement);
@@ -141,6 +145,9 @@ abstract class OperationValidator {
         case CUSTOM_UNINSTALL:
             retval = FOR_CUSTOM_UNINSTALL.getRequiredElementsImpl(updateElement, moduleInfos, brokenDependencies);
             break;
+        case INTERNAL_UPDATE:
+            retval = FOR_INTERNAL_UPDATE.getRequiredElementsImpl(updateElement, moduleInfos, brokenDependencies);
+            break;
         default:
             assert false;
         }
@@ -164,6 +171,7 @@ abstract class OperationValidator {
                 break;
             case INSTALL :
             case UPDATE :
+            case INTERNAL_UPDATE:
                 getRequiredElements (type, updateElement, moduleInfos, broken);
                 break;
             case UNINSTALL :
@@ -185,6 +193,12 @@ abstract class OperationValidator {
             List<ModuleInfo> moduleInfos,
             Collection<String> brokenDependencies);
     
+    private static class InternalUpdateValidator extends UpdateValidator {
+        @Override
+        boolean isValidOperationImpl(UpdateUnit unit, UpdateElement uElement) {
+            return uElement.equals(unit.getInstalled()) || containsElement (uElement, unit);
+        }
+    }
     private static class InstallValidator extends OperationValidator {
         boolean isValidOperationImpl(UpdateUnit unit, UpdateElement uElement) {
             return unit.getInstalled() == null && containsElement (uElement, unit);
@@ -192,7 +206,7 @@ abstract class OperationValidator {
         
         List<UpdateElement> getRequiredElementsImpl (UpdateElement uElement, List<ModuleInfo> moduleInfos, Collection<String> brokenDependencies) {
             Set<Dependency> brokenDeps = new HashSet<Dependency> ();
-            List<UpdateElement> res = new LinkedList<UpdateElement> (Utilities.findRequiredUpdateElements (uElement, moduleInfos, brokenDeps));
+            List<UpdateElement> res = new LinkedList<UpdateElement> (Utilities.findRequiredUpdateElements (uElement, moduleInfos, brokenDeps, false));
             if (brokenDependencies != null) {
                 for (Dependency dep : brokenDeps) {
                     brokenDependencies.add (dep.toString ());
@@ -286,7 +300,14 @@ abstract class OperationValidator {
         }
         
         List<UpdateElement> getRequiredElementsImpl (UpdateElement uElement, List<ModuleInfo> moduleInfos, Collection<String> brokenDependencies) {
-             return FOR_INSTALL.getRequiredElementsImpl(uElement, moduleInfos, brokenDependencies);
+            Set<Dependency> brokenDeps = new HashSet<Dependency> ();
+            List<UpdateElement> res = new LinkedList<UpdateElement> (Utilities.findRequiredUpdateElements (uElement, moduleInfos, brokenDeps, true));
+            if (brokenDependencies != null) {
+                for (Dependency dep : brokenDeps) {
+                    brokenDependencies.add (dep.toString ());
+                }
+            }
+            return res;
         }
     }
     
@@ -524,9 +545,18 @@ abstract class OperationValidator {
         for (UpdateElement eagerEl : UpdateManagerImpl.getInstance ().getInstalledEagers ()) {
             // take a module
             UpdateElementImpl impl = Trampoline.API.impl (eagerEl);
-            assert impl instanceof ModuleUpdateElementImpl : eagerEl + " is instanceof ModuleUpdateElementImpl";
-            ModuleInfo mi = ((ModuleUpdateElementImpl) impl).getModuleInfo ();
-            installedEagers.add (Utilities.toModule (mi));
+            if(impl instanceof ModuleUpdateElementImpl) {
+                ModuleInfo mi = ((ModuleUpdateElementImpl) impl).getModuleInfo ();
+                installedEagers.add (Utilities.toModule (mi));
+            } else if(impl instanceof FeatureUpdateElementImpl) {
+                List <ModuleInfo> infos = ((FeatureUpdateElementImpl) impl).getModuleInfos();
+                for(ModuleInfo mi : infos) {
+                    installedEagers.add (Utilities.toModule (mi));
+                }
+            } else {
+                assert false : eagerEl + " is instanceof neither ModuleUpdateElementImpl nor FeatureUpdateElementImpl";
+            }
+            
         }
         // add installedEagers into affected modules to don't break uninstall of candidates
         compactSet.addAll (installedEagers);
