@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -24,7 +24,7 @@
  * Contributor(s):
  *
  * The Original Software is NetBeans. The Initial Developer of the Original
- * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
+ * Software is Sun Microsystems, Inc. Portions Copyright 1997-2009 Sun
  * Microsystems, Inc. All Rights Reserved.
  *
  * If you wish your version of this file to be governed by only the CDDL
@@ -43,21 +43,22 @@ package org.netbeans.core.multiview;
 
 import java.awt.*;
 import java.awt.event.*;
-import java.awt.event.KeyEvent;
 import java.util.Enumeration;
 import java.util.HashSet;
 import java.util.Set;
+import java.util.logging.Level;
+import java.util.logging.Logger;
 import javax.swing.*;
 import javax.swing.JToggleButton.ToggleButtonModel;
 import javax.swing.KeyStroke;
 import javax.swing.border.Border;
-import javax.swing.text.Keymap;
 import org.netbeans.core.spi.multiview.MultiViewDescription;
 import org.netbeans.core.spi.multiview.MultiViewElement;
+import org.openide.awt.Actions;
 import org.openide.awt.Mnemonics;
-import org.openide.util.Lookup;
+import org.openide.text.CloneableEditorSupport;
+import org.openide.text.CloneableEditorSupport.Pane;
 import org.openide.util.NbBundle;
-import org.openide.util.actions.CallbackSystemAction;
 
 
 /**
@@ -76,6 +77,7 @@ class TabsComponent extends JPanel {
     private CardLayout cardLayout;
     private Set<MultiViewElement> alreadyAddedElements;
     private JToolBar bar;
+    private JLabel spacer = new JLabel();
     
     private static final boolean AQUA = "Aqua".equals(UIManager.getLookAndFeel().getID()); //NOI18N
 
@@ -85,6 +87,27 @@ class TabsComponent extends JPanel {
     public TabsComponent(boolean toolVis) {
         super();
         bar = new JToolBar();
+
+        setFocusTraversalPolicyProvider(true);
+        setFocusTraversalPolicy(new DefaultFocusTraversalPolicy() {
+            @Override
+            public Component getDefaultComponent(Container aContainer) {
+                final MultiViewElement elem = model.getActiveElement();
+                final JComponent vr = elem.getVisualRepresentation();
+                if (vr instanceof CloneableEditorSupport.Pane) {
+                   Pane pane = (Pane)vr;
+                   JEditorPane editorPane = pane.getEditorPane();
+                   if (null != editorPane && editorPane.isShowing()) {
+                     return editorPane;
+                   } else if( null == editorPane ) {
+                       Logger.getLogger(TabsComponent.class.getName()).log(Level.INFO,
+                               "No editor pane in " + model.getActiveElement().getVisualRepresentation() ); //NOI18N
+                   }
+               }
+               return super.getDefaultComponent(aContainer);
+            }
+        });
+
         Border b = (Border)UIManager.get("Nb.Editor.Toolbar.border"); //NOI18N
         bar.setBorder(b);
         bar.setFloatable(false);
@@ -152,10 +175,16 @@ class TabsComponent extends JPanel {
         GridBagConstraints cons = new GridBagConstraints();
         cons.anchor = GridBagConstraints.EAST;
         cons.fill = GridBagConstraints.BOTH;
-        cons.gridwidth = GridBagConstraints.REMAINDER;
         cons.weightx = 1;
 
         bar.add(toolbarPanel, cons);
+
+        cons = new GridBagConstraints();
+        cons.anchor = GridBagConstraints.EAST;
+        cons.fill = GridBagConstraints.BOTH;
+        cons.weightx = 1;
+
+        bar.add(spacer, cons);
     }
 
     
@@ -211,7 +240,7 @@ class TabsComponent extends JPanel {
     
     private JToggleButton createButton(MultiViewDescription description) {
         final JToggleButton button = new JToggleButton();
-        Mnemonics.setLocalizedText(button, description.getDisplayName());
+        Mnemonics.setLocalizedText(button, Actions.cutAmpersand(description.getDisplayName()));
         button.setModel(new TabsButtonModel(description));
         button.setRolloverEnabled(true);
         Border b = (getButtonBorder());
@@ -231,40 +260,7 @@ class TabsComponent extends JPanel {
             buttonMouseListener = new ButtonMouseListener();
         }
         button.addMouseListener (buttonMouseListener);
-
-        //HACK start - now find the global action shortcut
-        Keymap map = Lookup.getDefault().lookup(Keymap.class);
-        KeyStroke stroke = null;
-        KeyStroke stroke2 = null;
-//in tests map can be null, that's why the check..
-        if (map != null) {
-            // map is null in tests..
-            Action[] acts = map.getBoundActions();
-            for (int i = 0; i < acts.length;i++) {
-                if (acts[i] instanceof CallbackSystemAction) {
-                    CallbackSystemAction sa = (CallbackSystemAction)acts[i];
-                    if ("NextViewAction".equals(sa.getActionMapKey())) { //NOI18N
-                        KeyStroke[] strokes = map.getKeyStrokesForAction(acts[i]);
-                        if (strokes != null && strokes.length > 0) {
-                            stroke = strokes[0];
-                        }
-                    }
-                    if ("PreviousViewAction".equals(sa.getActionMapKey())) { //NOI18N
-                        KeyStroke[] strokes = map.getKeyStrokesForAction(acts[i]);
-                        if (strokes != null && strokes.length > 0) {
-                            stroke2 = strokes[0];
-                        }
-                    }
-                }
-            }
-        }
-        //HACK end
-        String key1 = stroke == null ? "" : KeyEvent.getKeyModifiersText(stroke.getModifiers()) + "+" + KeyEvent.getKeyText(stroke.getKeyCode());//NOI18N
-        String key2 = stroke2 == null ? "" : KeyEvent.getKeyModifiersText(stroke2.getModifiers()) + "+" + KeyEvent.getKeyText(stroke2.getKeyCode());//NOI18N
-        button.setToolTipText(NbBundle.getMessage(TabsComponent.class, "TabButton.tooltip",//NOI18N
-                              button.getText(), 
-                              key1,
-                              key2));
+        button.setToolTipText(NbBundle.getMessage(TabsComponent.class, "TabButton.tool_tip", button.getText()));
         button.setFocusable(true);
         button.setFocusPainted(true);
         return button;
@@ -275,6 +271,7 @@ class TabsComponent extends JPanel {
             if (toolbarPanel != null) {
                 bar.remove(toolbarPanel);
             }
+            bar.remove(spacer);
             if (innerbar == null) {
                 innerbar = getEmptyInnerToolBar();
             }
@@ -286,16 +283,20 @@ class TabsComponent extends JPanel {
                 innerbar.setBorder (BorderFactory.createEmptyBorder(2, 0, 2, 0));
             }
             toolbarPanel = innerbar;
-            if (toolbarPanel != null) {
-                GridBagConstraints cons = new GridBagConstraints();
-                cons.anchor = GridBagConstraints.EAST;
-                cons.fill = GridBagConstraints.BOTH;
-                cons.weightx = 1;
-                toolbarPanel.setMinimumSize(new Dimension(10, 10));
-                cons.gridwidth = GridBagConstraints.REMAINDER;
-                
-                bar.add(toolbarPanel, cons);
-            }
+            GridBagConstraints cons = new GridBagConstraints();
+            cons.anchor = GridBagConstraints.EAST;
+            cons.fill = GridBagConstraints.BOTH;
+            cons.weightx = 1;
+            toolbarPanel.setMinimumSize(new Dimension(10, 10));
+//            cons.gridwidth = GridBagConstraints.REMAINDER;
+
+            bar.add(toolbarPanel, cons);
+
+            cons = new GridBagConstraints();
+            cons.anchor = GridBagConstraints.EAST;
+            cons.fill = GridBagConstraints.BOTH;
+            cons.weightx = 1;
+            bar.add(spacer, cons);
             // rootcycle is the tabscomponent..
 //            toolbarPanel.setFocusCycleRoot(false);
             bar.revalidate();
