@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -118,6 +118,7 @@ public class Controller {
         if (io.getToolTipText() != null) {
             ioContainer.setToolTipText(result, io.getToolTipText());
         }
+        io.setClosed(false);
 
         //Make sure names are boldfaced for all open streams - if the tabbed
         //pane was just added in, it will just have used the name of the
@@ -271,6 +272,19 @@ public class Controller {
         nameUpdater.add(tab);
     }
 
+    private static boolean htmlTabsBroken() {
+        String version = System.getProperty("java.version");
+        for (int i = 14; i < 18; i++) {
+            if (version.startsWith("1.6.0_" + i)) {
+                return true;
+            }
+        }
+        return false;
+    }
+
+    // workaround for JDK bug (http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6670274)
+    // NB issue #113388
+    private static final boolean DONT_USE_HTML = htmlTabsBroken();
     private CoalescedNameUpdater nameUpdater = null;
     /**
      * Calls to methods invoked on NbIO done on the EQ are invoked synchronously
@@ -298,10 +312,6 @@ public class Controller {
             components.remove(tab);
         }
 
-        // workaround for JDK bug (http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6670274)
-        // NB issue #113388
-        private final boolean dontUseHtml = System.getProperty("java.version").startsWith("1.6.0_14");
-
         public void run() {
             for (OutputTab t : components) {
                 NbIO io = t.getIO();
@@ -319,7 +329,7 @@ public class Controller {
                     escaped = io.getName();
                 }
                 String name = io.isStreamClosed() ?  io.getName() + " " : //NOI18N
-                    (dontUseHtml ? io.getName() + " * " : "<html><b>" + escaped + " </b>&nbsp;</html>"); //NOI18N
+                    (DONT_USE_HTML ? io.getName() + " * " : "<html><b>" + escaped + " </b>&nbsp;</html>"); //NOI18N
 
                 if (LOG) {
                     log("  set name to " + name);
@@ -366,11 +376,8 @@ public class Controller {
                 if (tab == null) {
                     tab = createOutputTab(io);
                 }
-                ioContainer.open();
                 if (io.isFocusTaken()) {
                     ioContainer.requestActive();
-                } else {
-                    ioContainer.requestVisible();
                 }
                 ioContainer.select(tab);
                 break;
