@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -46,12 +46,13 @@
 
 package org.netbeans.core.output2;
 
-import java.util.logging.Logger;
+import java.awt.Color;
 import org.openide.util.NbBundle;
 import org.openide.windows.OutputListener;
 import java.io.*;
 import java.nio.ByteBuffer;
 import java.nio.CharBuffer;
+import java.util.Collection;
 import org.openide.util.Exceptions;
 import org.openide.util.Utilities;
 
@@ -77,7 +78,7 @@ class OutWriter extends PrintWriter {
     /**
      * Byte array used to write the line separator after line writes.
      */
-    static String lineSeparator = System.getProperty ("line.separator" );
+    static final String LINE_SEPARATOR = "\n";
 
     /** The read-write backing storage.  May be heap or */
     private Storage storage;
@@ -86,11 +87,6 @@ class OutWriter extends PrintWriter {
      * storage */
     private AbstractLines lines = new LinesImpl();
     
-    /** Flag set if one of several exceptions occurred while writing which
-     * mean the process doing the writing was brutally terminated.  Data will
-     * be readable but not writable if set to true */
-    private boolean terminated = false;
-    
     /** Flag set if a write failed due to disk space limits.  Subsequent
      * instances will use HeapStorage in this case */
     static boolean lowDiskSpace = false;
@@ -138,8 +134,7 @@ class OutWriter extends PrintWriter {
     }
     
     boolean isEmpty() {
-        return storage == null ? true : lines == null ? true : 
-            storage.size() == 0;
+        return storage == null ? true : storage.size() == 0;
     }
 
     @Override
@@ -147,23 +142,21 @@ class OutWriter extends PrintWriter {
         return "OutWriter@" + System.identityHashCode(this) + " for " + owner + " closed ";
     }
 
+    private int errorCount = 0;
+
     /** Generic exception handling, marking the error flag and notifying it with ErrorManager */
-    private void handleException (Exception e) {
+    private void handleException(Exception e) {
         setError();
-        String msg = Exceptions.findLocalizedMessage(e);
-        if (msg == null) {
-            Exceptions.attachLocalizedMessage(e,
-                                              NbBundle.getMessage(OutWriter.class,
-                                                                  "MSG_GenericError")); //NOI18N
-        }
         if (Controller.LOG) {
             StackTraceElement[] el = e.getStackTrace();
-            Controller.log ("EXCEPTION: " + e.getClass() + e.getMessage());
-            for (int i=1; i < el.length; i++) {
-                Controller.log (el[i].toString());
+            Controller.log("EXCEPTION: " + e.getClass() + e.getMessage());
+            for (int i = 1; i < el.length; i++) {
+                Controller.log(el[i].toString());
             }
         }
-        Exceptions.printStackTrace(e);
+        if (errorCount++ < 3) {
+            Exceptions.printStackTrace(e);
+        }
     }
 
     /**
@@ -173,22 +166,18 @@ class OutWriter extends PrintWriter {
      * @param bb
      * @throws IOException
      */
-    public synchronized void write(ByteBuffer bb, boolean completeLine) throws IOException {
-        if (checkError() || terminated) {
+    public synchronized void write(ByteBuffer bb, boolean completeLine) {
+        if (checkError()) {
             return;
         }
         closed = false;
         int start = -1;
         try {
             start = getStorage().write(bb);
-        } catch (java.nio.channels.ClosedByInterruptException cbie) {
-            //Execution termination has sent ThreadDeath to the process in the
-            //middle of a write
-            threadDeathClose();
         } catch (java.nio.channels.AsynchronousCloseException ace) {
             //Execution termination has sent ThreadDeath to the process in the
             //middle of a write
-            threadDeathClose();
+            onWriteException();
         } catch (IOException ioe) {
             //Out of disk space
             if (ioe.getMessage().indexOf("There is not enough space on the disk") != -1) { //NOI18N
@@ -204,23 +193,23 @@ class OutWriter extends PrintWriter {
                 //open for reads - if there's a problem there too, the error
                 //flag will be set when a read is attempted
                 Exceptions.printStackTrace(ioe);
-                threadDeathClose();
+                onWriteException();
             }
         }
+        if (checkError()) {
+            return;
+        }
         int length = bb.limit();
         lineLength = lineLength + length;
         if (start >= 0 && lineStart == -1) {
             lineStart = start;
         }
-        if (terminated) {
-            return;
-        }
         lines.lineUpdated(lineStart, lineLength, completeLine);
         if (completeLine) {
             lineStart = -1;
             lineLength = 0;
         }
-        if (owner != null && owner.hasStreamClosed()) {
+        if (owner != null && owner.isStreamClosed()) {
             owner.setStreamClosed(false);
             lines.fire();
         }
@@ -231,8 +220,8 @@ class OutWriter extends PrintWriter {
      * a writable one.  Typically this happens when an executing process was sent Thread.stop()
      * in the middle of a write.
      */
-    void threadDeathClose() {
-        terminated = true;
+    void onWriteException() {
+        trouble = true;
         if (Controller.LOG) Controller.log (this + " Close due to termination");
         ErrWriter err = owner.writer().err();
         if (err != null) {
@@ -256,13 +245,10 @@ class OutWriter extends PrintWriter {
         if (Controller.LOG) Controller.log (this + ": OutWriter.dispose - owner is " + (owner == null ? "null" : owner.getName()));
         clearListeners();
         if (storage != null) {
+            lines.onDispose(storage.size());
             storage.dispose();
             storage = null;
         }
-        if (lines != null) {
-            lines.clear();
-        }
-        trouble = true;
         if (Controller.LOG) Controller.log (this + ": Setting owner to null, trouble to true, dirty to false.  This OutWriter is officially dead.");
         owner = null;
         disposed = true;
@@ -276,20 +262,14 @@ class OutWriter extends PrintWriter {
             return;
         }
         synchronized (this) {
-            if (lines != null && lines.hasHyperlinks()) {
-                int[] listenerLines = lines.allListenerLines();
+            if (lines.hasListeners()) {
+                int[] listenerLines = lines.getLinesWithListeners();
                 Controller.ControllerOutputEvent e = new Controller.ControllerOutputEvent(owner, 0);
                 for (int i=0; i < listenerLines.length; i++) {
-                    OutputListener ol = (OutputListener) lines.getListenerForLine(listenerLines[i]);
-                    if (Controller.LOG) {
-                        Controller.log("Clearing listener " + ol);
-                    }
+                    Collection<OutputListener> ols = lines.getListenersForLine(listenerLines[i]);
                     e.setLine(listenerLines[i]);
-                    if (ol != null) {
+                    for (OutputListener ol : ols) {
                         ol.outputLineCleared(e);
-                    } else {
-                        //#56826 - debug messaging
-                        Logger.getAnonymousLogger().warning("issue #56826 - There was a null OutputListener on line:" + listenerLines[i]);
                     }
                 }
             } else {
@@ -320,11 +300,9 @@ class OutWriter extends PrintWriter {
             if (storage != null) {
                 storage.close();
             }
-            if (lines != null) {
-                lines.fire();
-            }
+            lines.fire();
         } catch (IOException ioe) {
-            handleException (ioe);
+            onWriteException();
         }
     }
 
@@ -341,11 +319,9 @@ class OutWriter extends PrintWriter {
         }
         try {
             getStorage().flush();
-            if (lines != null) {
-                lines.fire();
-            }
+            lines.fire();
         } catch (IOException e) {
-            handleException(e);
+            onWriteException();
         }
     }
 
@@ -354,7 +330,7 @@ class OutWriter extends PrintWriter {
         return disposed || trouble;
     }
 
-    class CharArrayWrapper implements CharSequence {
+    static class CharArrayWrapper implements CharSequence {
 
         private char[] arr;
         private int off;
@@ -399,31 +375,33 @@ class OutWriter extends PrintWriter {
     }
     
     /** write buffer size in chars */
-    static private final int writeBuffSize = 16*1024;
-    private final String tabReplacement = "        ";
+    private static final int WRITE_BUFF_SIZE = 16*1024;
+    static final String TAB_REPLACEMENT = "        ";
     public synchronized int doWrite(CharSequence s, int off, int len) {
-        int lineCount = 0;
         if (checkError() || len == 0) {
-            return lineCount;
+            return 0;
         }
 
+        int lineCount = 0;
         try {
             boolean written = false;
-            ByteBuffer byteBuff = getStorage().getWriteBuffer(writeBuffSize * 2);
+            ByteBuffer byteBuff = getStorage().getWriteBuffer(WRITE_BUFF_SIZE * 2);
             CharBuffer charBuff = byteBuff.asCharBuffer();
             for (int i = off; i < off + len; i++) {
-                if (charBuff.position() + tabReplacement.length() >= writeBuffSize) {
+                if (charBuff.position() + TAB_REPLACEMENT.length() >= WRITE_BUFF_SIZE) {
                     write((ByteBuffer) byteBuff.position(charBuff.position() * 2), false);
                     written = true;
                 }
                 if (written) {
-                    byteBuff = getStorage().getWriteBuffer(writeBuffSize * 2);
+                    byteBuff = getStorage().getWriteBuffer(WRITE_BUFF_SIZE * 2);
                     charBuff = byteBuff.asCharBuffer();
                     written = false;
                 }
                 char c = s.charAt(i);
                 if (c == '\t') {
-                    charBuff.put(tabReplacement);
+                    charBuff.put(TAB_REPLACEMENT);
+                } else if (c == '\r') {
+                    // skip
                 } else if (c == '\n') {
                     charBuff.put(c);
                     write((ByteBuffer) byteBuff.position(charBuff.position() * 2), true);
@@ -437,7 +415,7 @@ class OutWriter extends PrintWriter {
                 write((ByteBuffer) byteBuff.position(charBuff.position() * 2), false);
             }
         } catch (IOException ioe) {
-            handleException(ioe);
+            onWriteException();
         }
         lines.delayedFire();
         return lineCount;
@@ -450,7 +428,7 @@ class OutWriter extends PrintWriter {
 
     @Override
     public synchronized void println() {
-        doWrite(lineSeparator, 0, lineSeparator.length());
+        doWrite("\n", 0, 1);
     }
 
     /**
@@ -469,26 +447,32 @@ class OutWriter extends PrintWriter {
         doWrite(s, 0, s.length());
     }
 
-    public synchronized void println(String s, OutputListener l) throws IOException {
+    public synchronized void println(String s, OutputListener l) {
         println(s, l, false);
     }
 
-    public synchronized void println(String s, OutputListener l, boolean important) throws IOException {
-        if (checkError()) {
-            return;
+    public synchronized void println(String s, OutputListener l, boolean important) {
+        print(s, l, important, null, false, true);
+    }
+
+    synchronized void print(CharSequence s, OutputListener l, boolean important, Color c, boolean err, boolean addLS) {
+        int lastLine = lines.getLineCount() - 1;
+        int lastPos = lines.getCharCount();
+        doWrite(s, 0, s.length());
+        if (addLS) {
+            println();
         }
-        int addedCount = doWrite(s, 0, s.length());
-        println();
-        addedCount++;
-        int newCount = lines.getLineCount()-1;
-        for (int i = newCount - addedCount; i < newCount; i++) {
-            lines.addListener(i, l, important);
-            //#48485 we should update the UI, since the lines are in the model
-            // and jump next/previous can't jump to appropriate place.
-            lines.fire();
+        lines.updateLinesInfo(s, lastLine, lastPos, l, important, err, c);
+    }
+
+    synchronized void print(CharSequence s, LineInfo info, boolean important) {
+        int line = lines.getLineCount() - 1;
+        doWrite(s, 0, s.length());
+        if (info != null) {
+            lines.addLineInfo(line, info, important);
         }
     }
-        
+
     /**
      * A useless writer object to pass to the superclass constructor.  We override all methods
      * of it anyway.
@@ -522,10 +506,6 @@ class OutWriter extends PrintWriter {
              return OutWriter.this.disposed;
         }
 
-        protected boolean isTrouble() {
-            return OutWriter.this.trouble;
-        }
-
         public Object readLock() {
             return OutWriter.this;
         }
