@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -43,11 +43,13 @@ package org.netbeans.core.output2;
 
 import java.io.IOException;
 import java.io.UnsupportedEncodingException;
+import java.lang.reflect.InvocationTargetException;
 import javax.swing.SwingUtilities;
 import javax.swing.event.ChangeEvent;
 import javax.swing.event.ChangeListener;
 import org.netbeans.junit.NbTestCase;
 import org.netbeans.junit.RandomlyFails;
+import org.openide.util.Exceptions;
 
 /** Tests the OutWriter class
  *
@@ -78,17 +80,17 @@ public class OutWriterTest extends NbTestCase {
         
         assertTrue ("First line position should be 0 but is " + pos, pos == 0);
         
-        int expectedPosition = first.length() + OutWriter.lineSeparator.length();
+        int expectedPosition = first.length() + OutWriter.LINE_SEPARATOR.length();
         pos = ow.getLines().getLineStart(1);
         
         assertTrue ("Second line position should be length of first (" + first.length() + ") + line " +
-            "separator length (" + OutWriter.lineSeparator.length() + "), which should be " + 
+            "separator length (" + OutWriter.LINE_SEPARATOR.length() + "), which should be " +
             expectedPosition + " but is " + pos, 
             pos == expectedPosition);
          
         
         pos = ow.getLines().getLineStart (2);
-        int targetPos = first.length() + second.length() + 2 * OutWriter.lineSeparator.length();
+        int targetPos = first.length() + second.length() + 2 * OutWriter.LINE_SEPARATOR.length();
         
         assertTrue ("Third line position should be " + targetPos + " but is " + pos, pos == targetPos);
     }
@@ -114,7 +116,7 @@ public class OutWriterTest extends NbTestCase {
         
         assertTrue (ow.getLines().getLineCount() == 3);
 
-        int targetLength = first.length() + second.length() + 2 * OutWriter.lineSeparator.length();
+        int targetLength = first.length() + second.length() + 2 * OutWriter.LINE_SEPARATOR.length();
 
         assertTrue ( 
             "After printing strings with length " + first.length() + " and " + 
@@ -125,7 +127,7 @@ public class OutWriterTest extends NbTestCase {
         ow.println (third);
         
         targetLength = first.length() + second.length() + third.length() + 
-            (3 *  OutWriter.lineSeparator.length());
+            (3 *  OutWriter.LINE_SEPARATOR.length());
         
         assertTrue ("Length should be " + targetLength + " but position is "
             + ow.getLines().getCharCount(), targetLength == ow.getLines().getCharCount());
@@ -161,9 +163,9 @@ public class OutWriterTest extends NbTestCase {
             ioe.printStackTrace();
             fail (ioe.getMessage());
         }
-        String firstExpected = first + OutWriter.lineSeparator;
-        String secondExpected = second + OutWriter.lineSeparator;
-        String thirdExpected = third + OutWriter.lineSeparator;
+        String firstExpected = first + OutWriter.LINE_SEPARATOR;
+        String secondExpected = second + OutWriter.LINE_SEPARATOR;
+        String thirdExpected = third + OutWriter.LINE_SEPARATOR;
         
         assertEquals("First string should be \"" + firstExpected + "\" but was \"" + firstBack + "\"",
             firstBack, firstExpected);
@@ -198,7 +200,7 @@ public class OutWriterTest extends NbTestCase {
             " not " + line,
             line == 0);
         
-        line = ow.getLines().getLineAt (first.length() + OutWriter.lineSeparator.length() +
+        line = ow.getLines().getLineAt (first.length() + OutWriter.LINE_SEPARATOR.length() +
             (second.length() / 2));
         
         assertTrue ("Position halfway through line 1 should map to line 1, not " +
@@ -227,13 +229,7 @@ public class OutWriterTest extends NbTestCase {
         ow.println (third);
         
         ow.flush();
-        try {
-        SwingUtilities.invokeAndWait (new Runnable() {
-            public void run() {
-                System.currentTimeMillis();
-             }
-        });
-        } catch (Exception e) {}
+        processEQ();
         Thread.yield();
         
         assertTrue ("Linecount should be 4 after printing 3 lines, not " +
@@ -247,7 +243,7 @@ public class OutWriterTest extends NbTestCase {
                 ow.getLines().getLineCount(), ow.getLines().getLineCount() == 4);
 
     }
-
+    
     public void testAddChangeListener() {
         System.out.println("testAddChangeListener");
         OutWriter ow = new OutWriter ();
@@ -267,8 +263,8 @@ public class OutWriterTest extends NbTestCase {
         ow.println(first);
         ow.println (second);
         ow.println (third);
-        ow.flush();        
-
+        ow.flush();
+        processEQ();
         cl.assertChanged();
     }
 
@@ -370,7 +366,7 @@ public class OutWriterTest extends NbTestCase {
             ow.println(first);
             ow.flush();
             
-            String firstExpected = first + OutWriter.lineSeparator;
+            String firstExpected = first + OutWriter.LINE_SEPARATOR;
             String firstReceived = ow.getLines().getLine(0);
             
             assertEquals ("First line should be \"" + firstExpected + "\" but was \"" + firstReceived + "\"", firstExpected, firstReceived);
@@ -419,7 +415,7 @@ public class OutWriterTest extends NbTestCase {
             ow.flush();
             assertEquals(2, ow.getLines().getLineCount());
             String firstReceived = ow.getLines().getLine(0);
-            assertEquals ("xyz" + OutWriter.lineSeparator, firstReceived);
+            assertEquals ("xyz" + OutWriter.LINE_SEPARATOR, firstReceived);
         
             ow = new OutWriter();
             ow.println("firstline");
@@ -428,7 +424,7 @@ public class OutWriterTest extends NbTestCase {
             ow.flush();
             assertEquals(3, ow.getLines().getLineCount());
             firstReceived = ow.getLines().getLine(1);
-            assertEquals ("xyz" + OutWriter.lineSeparator, firstReceived);
+            assertEquals ("xyz" + OutWriter.LINE_SEPARATOR, firstReceived);
             
             ow = new OutWriter();
             ow.println("firstline");
@@ -438,31 +434,31 @@ public class OutWriterTest extends NbTestCase {
             ow.flush();
             assertEquals(3, ow.getLines().getLineCount());
             firstReceived = ow.getLines().getLine(1);
-            assertEquals ("xyzxyz-end" + OutWriter.lineSeparator, firstReceived);
+            assertEquals ("xyzxyz-end" + OutWriter.LINE_SEPARATOR, firstReceived);
             
             ow = new OutWriter();
             ow.write(new char[] {'x', 'y', '\n', 'z', 'z', 'z', '\n', 'A'});
             ow.flush();
             assertEquals(3, ow.getLines().getLineCount());
-            assertEquals("xy" + '\n', ow.getLines().getLine(0));
-            assertEquals("zzz" + '\n', ow.getLines().getLine(1));
+            assertEquals("xy" + OutWriter.LINE_SEPARATOR, ow.getLines().getLine(0));
+            assertEquals("zzz" + OutWriter.LINE_SEPARATOR, ow.getLines().getLine(1));
             assertEquals("A", ow.getLines().getLine(2));
             
             ow = new OutWriter();
             ow.write(new char[] {'x', 'y', '\n', 'z', 'z', 'z', '\n'});
             ow.flush();
             assertEquals(3, ow.getLines().getLineCount());
-            assertEquals("xy" + '\n', ow.getLines().getLine(0));
-            assertEquals("zzz" + '\n', ow.getLines().getLine(1));
+            assertEquals("xy" + OutWriter.LINE_SEPARATOR, ow.getLines().getLine(0));
+            assertEquals("zzz" + OutWriter.LINE_SEPARATOR, ow.getLines().getLine(1));
             
             ow = new OutWriter();
             ow.write(new char[] {'\n', '\n', '\n', 'z', 'z', 'z', '\n'});
             ow.flush();
             assertEquals(5, ow.getLines().getLineCount());
-            assertEquals("\n", ow.getLines().getLine(0));
-            assertEquals("\n", ow.getLines().getLine(1));
-            assertEquals("\n", ow.getLines().getLine(2));
-            assertEquals("zzz" + "\n", ow.getLines().getLine(3));
+            assertEquals(OutWriter.LINE_SEPARATOR, ow.getLines().getLine(0));
+            assertEquals(OutWriter.LINE_SEPARATOR, ow.getLines().getLine(1));
+            assertEquals(OutWriter.LINE_SEPARATOR, ow.getLines().getLine(2));
+            assertEquals("zzz" + OutWriter.LINE_SEPARATOR, ow.getLines().getLine(3));
             
             
         } catch (Exception e) {
@@ -484,7 +480,7 @@ public class OutWriterTest extends NbTestCase {
             ow.write('z');
             assertEquals ("xyz", ow.getLines().getLine(0));
             ow.println();
-            assertEquals ("xyz" + OutWriter.lineSeparator, ow.getLines().getLine(0));
+            assertEquals ("xyz" + OutWriter.LINE_SEPARATOR, ow.getLines().getLine(0));
             ow.write('a');
             assertEquals(2, ow.getLines().getLineCount());
             assertEquals ("a", ow.getLines().getLine(1));
@@ -493,14 +489,14 @@ public class OutWriterTest extends NbTestCase {
             ow = new OutWriter();
             ow.write(new char[] { 'x', 'y', 'z', '\n', 'A'});
             assertEquals(2, ow.getLines().getLineCount());
-            assertEquals ("xyz" + '\n', ow.getLines().getLine(0));
+            assertEquals ("xyz" + OutWriter.LINE_SEPARATOR, ow.getLines().getLine(0));
             assertEquals ("A", ow.getLines().getLine(1));
             ow.write('B');
             assertEquals(2, ow.getLines().getLineCount());
             assertEquals ("AB", ow.getLines().getLine(1));
             ow.println("CD");
             assertEquals(3, ow.getLines().getLineCount());
-            assertEquals ("ABCD" + OutWriter.lineSeparator, ow.getLines().getLine(1));
+            assertEquals ("ABCD" + OutWriter.LINE_SEPARATOR, ow.getLines().getLine(1));
             
         } catch (Exception e) {
             e.printStackTrace();
@@ -509,15 +505,21 @@ public class OutWriterTest extends NbTestCase {
         
     }
     
+    void processEQ() {
+        try {
+            SwingUtilities.invokeAndWait(new Runnable() {
 
+                public void run() {
+                    System.currentTimeMillis();
+                }
+            });
+        } catch (InterruptedException ex) {
+            Exceptions.printStackTrace(ex);
+        } catch (InvocationTargetException ex) {
+            Exceptions.printStackTrace(ex);
+        }
+    }
 
-   
-    
-    // TODO add test methods here, they have to start with 'test' name.
-    // for example:
-    // public void testHello() {}
-    
-    
     private class CL implements ChangeListener {
         
         public void assertChanged () {
@@ -541,6 +543,4 @@ public class OutWriterTest extends NbTestCase {
             ce = null;
         }
     }
-     
-    
 }
