@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -108,16 +108,17 @@ public class OutputDocumentTest extends NbTestCase {
 //                System.currentTimeMillis();
 //            }
 //        });
-        Thread.currentThread().sleep(1500);
+        ow.flush();
+        //Thread.currentThread().sleep(1500);
         
         DocumentEvent de = od.getEvent();
         assertTrue ("Event should have been fired", de != null);
         
         int elCount = doc.getElementCount();
         
-        assertTrue ("Element count should be 100 after printing 100 lines, " +
+        assertTrue ("Element count should be 101 after printing 100 lines, " +
             "not " + elCount, 
-            elCount == 100);
+            elCount == 101);
         
         DocumentEvent.ElementChange ec = de.getChange(doc);
         
@@ -184,20 +185,9 @@ public class OutputDocumentTest extends NbTestCase {
         assertFalse ("After closing the writer, the document should not expect itself to grow", doc.getLines().isGrowing());
     }
     
-    private static String STATE = "(STATE NOT SET!)";
     public void testWordWrap() {
-        System.out.println("testWordWrap - line data caching ON");
-        STATE = "(CACHED MODE)";
-        AbstractLines.unitTestUseCache(Boolean.TRUE);
+        System.out.println("testWordWrap");
         doTestWordWrap();
-        
-        System.out.println("testWordWrap - line data caching OFF");
-        STATE = "(DYNAMIC MODE)";
-        AbstractLines.unitTestUseCache(Boolean.FALSE);
-//        doTestWordWrap();
-        
-        AbstractLines.unitTestUseCache(null);
-        STATE = "(STATE NOT SET!)";
     }
         
     private void doTestWordWrap() {
@@ -219,27 +209,27 @@ public class OutputDocumentTest extends NbTestCase {
         ow.println (c80);
         
         int val = doc.getLines().getLogicalLineCountAbove(2, 90);
-        assertTrue (STATE + "With three 80 character lines of data, wrapped at 90 characters, there should be 2 lines above line 2, not " + val, val == 2);
+        assertTrue ("With three 80 character lines of data, wrapped at 90 characters, there should be 2 lines above line 2, not " + val, val == 2);
         
         val = doc.getLines().getLogicalLineCountIfWrappedAt(90);
-        assertTrue (STATE + "With three 80 character lines of data, wrapped at 90 characters, the line count should be 3, not " + val, val == 3);
+        assertTrue ("With three 80 character lines of data, wrapped at 90 characters, the line count should be 4, not " + val, val == 4);
         assertTrue (val == ow.getLines().getLineCount());
         
         val = doc.getLines().getLogicalLineCountIfWrappedAt(50);
-        assertTrue (STATE + "With three 80 character lines of data, wrapped at 50 characters, the line count should be 6, not " + val, val == 6);
+        assertTrue ("With three 80 character lines of data, wrapped at 50 characters, the line count should be 7, not " + val, val == 7);
         
         val = doc.getLines().getLogicalLineCountAbove(2, 50);
-        assertTrue (STATE + "With three 80 character lines of data, wrapped at 50 characters, there should be 4 logical lines above 2, not " + val, val == 4);
+        assertTrue ("With th0ree 80 character lines of data, wrapped at 50 characters, there should be 4 logical lines above 2, not " + val, val == 4);
         
         int[] wrapData = new int[] {5, 0, 0};
-        doc.getLines().toLogicalLineIndex(wrapData, 50);
-        assertTrue("The logical line index of the 5th phys line with three 80 char lines wrapped at 50 chars should be 2 in the document, not " + wrapData[0], wrapData[0] == 2);
+        doc.getLines().toPhysicalLineIndex(wrapData, 50);
+        assertTrue("The physical line index of the 5th logical line with three 80 char lines wrapped at 50 chars should be 2 in the document, not " + wrapData[0], wrapData[0] == 2);
         assertTrue("An 80 char line should wrap twice, not " + wrapData[2], wrapData[2] == 2);
-        assertTrue("On the 5th physical line with three 80 char lines wrapped at 50 chars should be the 1st line of actual line 3, not " + wrapData[1], wrapData[1] == 1);
+        assertTrue("On the 5th logical line with three 80 char lines wrapped at 50 chars should be the 1st line of actual line 3, not " + wrapData[1], wrapData[1] == 1);
         
         wrapData[0] = 6;
-        doc.getLines().toLogicalLineIndex(wrapData, 50);
-        assertTrue("On the 5th physical line with three 80 char lines wrapped at 50 chars should be the 2nd line of actual line 3, not " + wrapData[1], wrapData[1] == 2);
+        doc.getLines().toPhysicalLineIndex(wrapData, 50);
+        assertTrue("On the 6th logical line with three 80 char lines wrapped at 50 chars should be empty line, not " + wrapData[1], wrapData[1] == 0);
         
         ow.println(c20);
         ow.println(c80);
@@ -248,7 +238,7 @@ public class OutputDocumentTest extends NbTestCase {
         assertTrue ("There should be 6 logical lines above a 20 char line following three 80 char lines when wrapped at 50 chars", val == 6);
         
         wrapData[0] = 6;
-        doc.getLines().toLogicalLineIndex(wrapData, 50);
+        doc.getLines().toPhysicalLineIndex(wrapData, 50);
         assertTrue ("20 char line should not be wrapped, but shows " + wrapData[2] + " wraps", wrapData[2] == 1);
     }
     
@@ -269,7 +259,7 @@ public class OutputDocumentTest extends NbTestCase {
         assertNotNull ("Root element should not be null", el);
         
         assertTrue ("Root offset should be 0", el.getStartOffset() == 0);
-        assertTrue ("Root ending char should be count of written chars", el.getEndOffset() == ow.getLines().getCharCount());
+        assertTrue ("Root ending char should be count of written chars + 1", el.getEndOffset() == ow.getLines().getCharCount() + 1);
         assertTrue ("Wrong document object from default root element's getDocument method", el.getDocument() == doc);
         assertTrue ("Element count of the root element should be the line count", el.getElementCount() == ow.getLines().getLineCount());
         
@@ -315,9 +305,7 @@ public class OutputDocumentTest extends NbTestCase {
         ow.println (third);
         ow.flush();
         
-        String lineSeparator = new String(OutWriter.lineSepBytes, "UTF-16");
-        
-        int expectedLength = first.length() + second.length() + third.length() + (3 * lineSeparator.length()) ;
+        int expectedLength = first.length() + second.length() + third.length() + (3 * OutWriter.LINE_SEPARATOR.length()) ;
         int receivedLength = doc.getLength();
         
         assertTrue ("Number of characters counting carriage returns should be " 
@@ -399,8 +387,7 @@ public class OutputDocumentTest extends NbTestCase {
             ble.printStackTrace();
             fail ("Unexpected BadLocationException: " + ble.getMessage());
         }
-        String lineSeparator = new String(OutWriter.lineSepBytes, "UTF-16");
-        expected = first + lineSeparator + second + lineSeparator;
+        expected = first + OutWriter.LINE_SEPARATOR + second + OutWriter.LINE_SEPARATOR;
         
         assertEquals ("getText for first two strings should be \"" + expected + "\" but was \"" + received + "\"", expected, received);
     }
@@ -458,13 +445,12 @@ public class OutputDocumentTest extends NbTestCase {
         ow.println (second);
         ow.println (third);
         ow.flush();        
-        String lineSeparator = new String(OutWriter.lineSepBytes, "UTF-16");
         
         Element el = doc.getElement(0);
         assertTrue (el.getStartOffset() == 0);
         assertTrue ("End offset should be length of string + separator length (" 
-            + (first.length() + lineSeparator.length()) + " but was " + el.getEndOffset(), 
-            el.getEndOffset() == first.length() + lineSeparator.length());
+            + (first.length() + OutWriter.LINE_SEPARATOR.length()) + " but was " + el.getEndOffset(),
+            el.getEndOffset() == first.length() + OutWriter.LINE_SEPARATOR.length());
         
         el = doc.getElement(1);
     }
@@ -521,7 +507,7 @@ public class OutputDocumentTest extends NbTestCase {
         ow.println (third);
         ow.flush();    
         
-        assertTrue ("End offset should be chars printed", doc.getEndOffset() == ow.getLines().getCharCount());
+        assertTrue ("End offset should be chars printed + 1", doc.getEndOffset() == ow.getLines().getCharCount() + 1);
     }
     
     public void testGetParentElement() {
@@ -556,7 +542,7 @@ public class OutputDocumentTest extends NbTestCase {
         OutputDocument doc = new OutputDocument (ow);
         
         
-        assertTrue ("Document should be leaf if no text has been written", doc.isLeaf());
+        assertFalse("Document should not be leaf if no text has been written", doc.isLeaf());
         
         String first = "This is the first string";
         String second = "This is the second string, ain't it?";
@@ -579,19 +565,15 @@ public class OutputDocumentTest extends NbTestCase {
         ODListener docListener = new ODListener(doc);
         ODListener styListener = new ODListener(styled);
         
-        String lineSeparator = new String(OutWriter.lineSepBytes, "UTF-16");
-
-        
-        
         String s = "This is a string I will append";
         
-        styled.insertString(styled.getLength(), s + lineSeparator, SimpleAttributeSet.EMPTY);
+        styled.insertString(styled.getLength(), s + OutWriter.LINE_SEPARATOR, SimpleAttributeSet.EMPTY);
         ow.println (s);
         ow.flush();
         docListener.assertChanged();
         styListener.assertChanged();
         
-        styled.insertString(styled.getLength(), s + lineSeparator, SimpleAttributeSet.EMPTY);
+        styled.insertString(styled.getLength(), s + OutWriter.LINE_SEPARATOR, SimpleAttributeSet.EMPTY);
         ow.println (s);
         ow.flush();
         
@@ -621,7 +603,7 @@ public class OutputDocumentTest extends NbTestCase {
         //Stress test it to ensure no off-by-ones that show up only when the file is large
         for (int i = 0; i < 10; i++) {
             for (int j=0; j < STRINGS.length; j++) {
-                styled.insertString(styled.getLength(), s + lineSeparator, SimpleAttributeSet.EMPTY);
+                styled.insertString(styled.getLength(), s + OutWriter.LINE_SEPARATOR, SimpleAttributeSet.EMPTY);
                 ow.println (s);
                 ow.flush();
 
@@ -683,12 +665,12 @@ public class OutputDocumentTest extends NbTestCase {
         " an identical change on a StyledDocument returns " + styIndex,
         styIndex == docIndex);
         
-        assertTrue ("OutputDocument returned an array of " + docAdded.length +
+        /*assertTrue ("OutputDocument returned an array of " + docAdded.length +
             " affected elements, but an identical change on a StyledDocument " +
             "produces an array of " + styAdded.length, styAdded.length == 
-            docAdded.length);
+            docAdded.length);*/
 
-        for (int i=0; i < docAdded.length; i++) {
+        for (int i=0; i < styAdded.length; i++) {
             int docStartOffset = docAdded[i].getStartOffset();
             int styStartOffset = styAdded[i].getStartOffset();
             assertTrue ("Start offset of element " + i + " from " +
