@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -41,29 +41,36 @@
 
 package org.netbeans.core.startup;
 
+import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
+import java.io.FileReader;
 import java.io.IOException;
 import java.io.PrintStream;
 import java.io.PrintWriter;
+import java.io.Reader;
 import java.io.StringWriter;
 import java.io.UnsupportedEncodingException;
 import java.net.URL;
 import java.net.URLClassLoader;
 import java.text.DateFormat;
-import java.util.Arrays;
+import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Date;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.MissingResourceException;
+import java.util.Queue;
 import java.util.Set;
 import java.util.StringTokenizer;
+import java.util.concurrent.BlockingQueue;
 import java.util.concurrent.Callable;
+import java.util.concurrent.LinkedBlockingQueue;
 import java.util.logging.Formatter;
 import java.util.logging.Handler;
 import java.util.logging.Level;
@@ -71,10 +78,12 @@ import java.util.logging.LogManager;
 import java.util.logging.LogRecord;
 import java.util.logging.Logger;
 import java.util.logging.StreamHandler;
+import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import java.util.regex.PatternSyntaxException;
 import org.netbeans.TopSecurityManager;
 import org.openide.filesystems.FileUtil;
+import org.openide.util.Exceptions;
 import org.openide.util.Lookup;
 import org.openide.util.LookupEvent;
 import org.openide.util.LookupListener;
@@ -221,7 +230,7 @@ public final class TopLogging {
             if (DEBUG != null) DEBUG.println("initializing stdout"); // NOI18N
         }
     }
-    
+
 
     private static void printSystemInfo(PrintStream ps) {
         DateFormat df = DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.FULL, Locale.US);
@@ -230,10 +239,48 @@ public final class TopLogging {
         ps.println("-------------------------------------------------------------------------------"); // NOI18N
         ps.println(">Log Session: "+df.format (date)); // NOI18N
         ps.println(">System Info: "); // NOI18N
-        
+
+        List<File> clusters = new ArrayList<File>();
+        String nbdirs = System.getProperty("netbeans.dirs");
+        if (nbdirs != null) { // noted in #67862: should show all clusters here.
+            StringTokenizer tok = new StringTokenizer(nbdirs, File.pathSeparator);
+            while (tok.hasMoreTokens()) {
+                File dir = FileUtil.normalizeFile(new File(tok.nextToken()));
+                if (dir.isDirectory()) {
+                    clusters.add(dir);
+                }
+            }
+        }
+
         String buildNumber = System.getProperty ("netbeans.buildnumber"); // NOI18N
         String currentVersion = NbBundle.getMessage(TopLogging.class, "currentVersion", buildNumber );
-        ps.println("  Product Version         = " + currentVersion); // NOI18N
+        ps.print("  Product Version         = " + currentVersion); // NOI18N
+        for (File cluster : clusters) { // also print Hg ID if available; more precise
+            File buildInfo = new File(cluster, "build_info"); // NOI18N
+            if (buildInfo.isFile()) {
+                try {
+                    Reader r = new FileReader(buildInfo);
+                    try {
+                        BufferedReader b = new BufferedReader(r);
+                        String line;
+                        Pattern p = Pattern.compile("Hg ID:    ([0-9a-f]{12})"); // NOI18N
+                        while ((line = b.readLine()) != null) {
+                            Matcher m = p.matcher(line);
+                            if (m.matches()) {
+                                ps.print(" (#" + m.group(1) + ")"); // NOI18N
+                                break;
+                            }
+                        }
+                    } finally {
+                        r.close();
+                    }
+                } catch (IOException x) {
+                    x.printStackTrace(ps);
+                }
+                break;
+            }
+        }
+        ps.println();
         ps.println("  Operating System        = " + System.getProperty("os.name", "unknown")
                    + " version " + System.getProperty("os.version", "unknown")
                    + " running on " +  System.getProperty("os.arch", "unknown"));
@@ -257,18 +304,10 @@ public final class TopLogging {
         ps.print(  "  User Directory          = "); // NOI18N
         ps.println(CLIOptions.getUserDir()); // NOI18N
         ps.print(  "  Installation            = "); // NOI18N
-        String nbdirs = System.getProperty("netbeans.dirs");
-        if (nbdirs != null) { // noted in #67862: should show all clusters here.
-            StringTokenizer tok = new StringTokenizer(nbdirs, File.pathSeparator);
-            while (tok.hasMoreTokens()) {
-                File dir = FileUtil.normalizeFile(new File(tok.nextToken()));
-                if (dir.isDirectory()) {
-                    ps.print(dir);
-                    ps.print("\n                            "); //NOI18N
-                }
-            }
+        for (File cluster : clusters) {
+            ps.print(cluster + "\n                            "); // NOI18N
         }
-        ps.println(CLIOptions.getHomeDir());
+        ps.println(CLIOptions.getHomeDir()); // platform cluster is separate
         ps.println("  Boot & Ext. Classpath   = " + createBootClassPath()); // NOI18N
         String cp;
         ClassLoader l = Lookup.class.getClassLoader();
@@ -299,7 +338,7 @@ public final class TopLogging {
         // boot
         String boot = System.getProperty("sun.boot.class.path"); // NOI18N
         StringBuffer sb = (boot != null ? new StringBuffer(boot) : new StringBuffer());
-        
+
         // std extensions
         findBootJars(System.getProperty("java.ext.dirs"), sb);
         findBootJars(System.getProperty("java.endorsed.dirs"), sb);
@@ -336,11 +375,11 @@ public final class TopLogging {
         StreamHandler s = new StreamHandler (
             pw, NbFormatter.FORMATTER
         );
-        return s;
+        return new NonClose(s, 50);
     }
-    
-    private static java.util.logging.Handler streamHandler;
-    private static synchronized java.util.logging.Handler streamHandler () {
+
+    private static NonClose streamHandler;
+    private static synchronized NonClose streamHandler() {
         if (streamHandler == null) {
             StreamHandler sth = new StreamHandler (OLD_ERR, NbFormatter.FORMATTER);
             sth.setLevel(Level.ALL);
@@ -348,9 +387,9 @@ public final class TopLogging {
         }
         return streamHandler;
     }
-    
-    private static java.util.logging.Handler defaultHandler;
-    private static synchronized java.util.logging.Handler defaultHandler () {
+
+    private static NonClose defaultHandler;
+    private static synchronized NonClose defaultHandler() {
         if (defaultHandler != null) return defaultHandler;
 
         String home = System.getProperty("netbeans.user");
@@ -371,7 +410,7 @@ public final class TopLogging {
                 if (f.exists()) {
                     f.renameTo(f1);
                 }
-                
+
                 FileOutputStream fout = new FileOutputStream(f, false);
                 Handler h = new StreamHandler(fout, NbFormatter.FORMATTER);
                 h.setLevel(Level.ALL);
@@ -381,7 +420,7 @@ public final class TopLogging {
                 ex.printStackTrace();
             }
         }
-        
+
         if (defaultHandler == null) {
             defaultHandler = streamHandler();
             disabledConsole = true;
@@ -392,8 +431,8 @@ public final class TopLogging {
     /** Allows tests to flush all standard handlers */
     static void flush(boolean clear) {
         System.err.flush();
-        
-        
+
+
         Handler s = streamHandler;
         if (s != null) {
             s.flush();
@@ -410,16 +449,14 @@ public final class TopLogging {
         }
     }
     static void close() {
-        Handler s = streamHandler;
-        if (s instanceof NonClose) {
-            NonClose ns = (NonClose)s;
-            ns.doClose();
+        NonClose s = streamHandler;
+        if (s != null) {
+            s.doClose();
         }
 
-        Handler d = defaultHandler;
+        NonClose d = defaultHandler;
         if (d != null) {
-            NonClose nd = (NonClose)d;
-            nd.doClose();
+            d.doClose();
         }
     }
     static void exit(int exit) {
@@ -431,10 +468,11 @@ public final class TopLogging {
      */
     private static final class NonClose extends Handler
     implements Runnable {
-        private static RequestProcessor RP = new RequestProcessor("Logging Flush"); // NOI18N
+        private static RequestProcessor RP = new RequestProcessor("Logging Flush", 1, false, false); // NOI18N
         private static ThreadLocal<Boolean> FLUSHING = new ThreadLocal<Boolean>();
 
         private final Handler delegate;
+        private final BlockingQueue<LogRecord> queue = new LinkedBlockingQueue<LogRecord>(1000);
         private RequestProcessor.Task flush;
         private int delay;
 
@@ -446,21 +484,45 @@ public final class TopLogging {
         }
 
         public void publish(LogRecord record) {
-            delegate.publish(record);
+            if (RP.isRequestProcessorThread()) {
+                return;
+            }
+            if (!queue.offer(record)) {
+                for (;;) {
+                    try {
+                        // queue is full, schedule its clearing
+                        if (!schedule(0)) {
+                            return;
+                        }
+                        queue.put(record);
+                        Thread.yield();
+                        break;
+                    } catch (InterruptedException ex) {
+                        // OK, ignore and try again
+                    }
+                }
+            }
+            schedule(delay);
+        }
+
+        private boolean schedule(int d) {
             if (!Boolean.TRUE.equals(FLUSHING.get())) {
                 try {
                     FLUSHING.set(true);
-                    flush.schedule(delay);
+                    flush.schedule(d);
                 } finally {
                     FLUSHING.set(false);
                 }
+                return true;
+            } else {
+                return false;
             }
         }
 
         public void flush() {
             flush.cancel();
             flush.waitFinished();
-            delegate.flush();
+            run();
         }
 
         public void close() throws SecurityException {
@@ -485,6 +547,13 @@ public final class TopLogging {
         }
 
         public void run() {
+            for (;;) {
+                LogRecord r = queue.poll();
+                if (r == null) {
+                    break;
+                }
+                delegate.publish(r);
+            }
             delegate.flush();
         }
     }
@@ -560,7 +629,7 @@ public final class TopLogging {
             pw.println(trace[i]);
         }
     }
-    
+
     /** Modified formater for use in NetBeans.
      */
     private static final class NbFormatter extends java.util.logging.Formatter {
@@ -584,6 +653,9 @@ public final class TopLogging {
             if (message != null && message.indexOf('\n') != -1 && record.getThrown() == null) {
                 // multi line messages print witout any wrappings
                 sb.append(message);
+                if (message.charAt(message.length() - 1) != '\n') {
+                    sb.append(lineSeparator);
+                }
                 return;
             }
             if ("stderr".equals(record.getLoggerName()) && record.getLevel() == Level.INFO) { // NOI18N
@@ -592,7 +664,7 @@ public final class TopLogging {
                 return;
             }
 
-            sb.append(record.getLevel().getLocalizedName());
+            sb.append(record.getLevel().getName());
             addLoggerName (sb, record);
             if (message != null) {
                 sb.append(": ");
@@ -620,7 +692,7 @@ public final class TopLogging {
                 specialProcessing(sb, record.getThrown(), beenThere);
             }
         }
-        
+
         private static void addLoggerName (StringBuilder sb, java.util.logging.LogRecord record) {
             String name = record.getLoggerName ();
             if (!"".equals (name)) {
@@ -687,7 +759,7 @@ public final class TopLogging {
                     Object[] param;
                     if (col != -1 || line != -1) {
                         msg = "EXC_sax_parse_col_line"; // NOI18N
-                        param = new Object[] {String.valueOf(pubid), String.valueOf(sysid), new Integer(col), new Integer(line)};
+                        param = new Object[] {String.valueOf(pubid), String.valueOf(sysid), col, line};
                     } else {
                         msg = "EXC_sax_parse"; // NOI18N
                         param = new Object[] { String.valueOf(pubid), String.valueOf(sysid) };
@@ -706,7 +778,7 @@ public final class TopLogging {
      */
     private static final class LgStream extends PrintStream implements Runnable {
         private Logger log;
-        private final StringBuffer sb = new StringBuffer();
+        private final StringBuilder sb = new StringBuilder();
         private static RequestProcessor RP = new RequestProcessor("StdErr Flush");
         private RequestProcessor.Task flush = RP.create(this, true);
 
@@ -723,7 +795,9 @@ public final class TopLogging {
             if (unwantedMessages != null && unwantedMessages.matcher(s).find()) {
                 new Exception().printStackTrace(DEBUG);
             }
-            sb.append(s);
+            synchronized (sb) {
+                sb.append(s);
+            }
             checkFlush();
         }
 
@@ -735,13 +809,16 @@ public final class TopLogging {
             if (RP.isRequestProcessorThread()) {
                 return;
             }
-            sb.append((char)b);
+            synchronized (sb) {
+                sb.append((char)b);
+            }
             checkFlush();
         }
 
         @Override
         public void flush() {
             try {
+                flush.schedule(0);
                 flush.waitFinished(500);
             } catch (InterruptedException ex) {
                 // ok, flush failed, do not even print
@@ -750,8 +827,8 @@ public final class TopLogging {
             super.flush();
         }
 
-        
-        
+
+
         private void checkFlush() {
             //if (DEBUG != null) DEBUG.println("checking flush; buffer: " + sb); // NOI18N
             try {
@@ -778,46 +855,54 @@ public final class TopLogging {
                             at org.netbeans.insane.impl.LiveEngine.trace(LiveEngine.java:180)
                             at org.netbeans.insane.live.LiveReferences.fromRoots(LiveReferences.java:110)
 
-                 * just ignore it, we cannot print it at this situation anyway...                
+                 * just ignore it, we cannot print it at this situation anyway...
                  */
             }
         }
-        
+
         public void run() {
             for (;;) {
                 String toLog;
                 synchronized (sb) {
-                    int first = sb.indexOf("\n"); // NOI18N
-                    if (first < 0) {
+                    if (sb.length() == 0) {
                         break;
                     }
-                    if (first == 0) {
-                        sb.delete(0, 1);
-                        toLog = "\n";
-                    } else {
-                        toLog = sb.substring(0, first + 1);
-                        sb.delete(0, first + 1);
+                    int last = -1;
+                    for (int i = sb.length() - 1; i >=0; i--) {
+                        if (sb.charAt(i) == '\n') {
+                            last = i;
+                            break;
+                        }
                     }
+                    if (last == -1) {
+                        break;
+                    }
+                    toLog = sb.substring(0, last + 1);
+                    sb.delete(0, last + 1);
+                }
+                int begLine = 0;
+                while (begLine < toLog.length()) {
+                    int endLine = toLog.indexOf('\n', begLine);
+                    log.log(Level.INFO, toLog.substring(begLine, endLine + 1));
+                    begLine = endLine + 1;
                 }
-                //if (DEBUG != null) DEBUG.println("delegating: " + toLog); // NOI18N
-                log.log(Level.INFO, toLog);
             }
         }
     } // end of LgStream
 
-    private static final class LookupDel extends Handler 
+    private static final class LookupDel extends Handler
     implements LookupListener {
         private Lookup.Result<Handler> handlers;
         private Collection<? extends Handler> instances;
-        
-        
+
+
         public LookupDel() {
             handlers = Lookup.getDefault().lookupResult(Handler.class);
             instances = handlers.allInstances();
             handlers.addLookupListener(this);
         }
-        
-        
+
+
         public void publish(LogRecord record) {
             for (Handler h : instances) {
                 h.publish(record);
@@ -843,7 +928,7 @@ public final class TopLogging {
 
     /** Instances are created in awt.EventDispatchThread */
     public static final class AWTHandler {
-        /** The name MUST be handle and MUST be public 
+        /** The name MUST be handle and MUST be public
          * @param t the throwable to print
          */
         public static void handle(Throwable t) {
