@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -284,8 +284,9 @@ final class FileAssociationsPanel extends javax.swing.JPanel {
         if(cbExtension.getItemAt(0).equals(chooseExtensionItem)) {
             // remove initial hint item
             cbExtension.removeItem(chooseExtensionItem);
-            cbType.setEnabled(true);
         }
+        // refuse to change xml extension MIME type (#171126)
+        cbType.setEnabled(!newExtension.equalsIgnoreCase("xml")); //NOI18N
         cbType.setSelectedItem(model.getMimeItem(newExtension));
         btnRemove.setEnabled(model.canBeRemoved(newExtension));
     }//GEN-LAST:event_cbExtensionActionPerformed
