@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -851,8 +851,6 @@ public final class WindowManagerImpl extends WindowManager implements Workspace
         return null;
     }
     
-    private static final boolean NAME_HACK = Boolean.getBoolean("nb.tabnames.html"); //NOI18N
-    
     /** Helper method to retrieve some form of display name of TopComponent.
      * First tries TopComponent's getHtmlDisplayName, if is it null then continues
      * with getDisplayName and getName in this order.
@@ -872,39 +870,6 @@ public final class WindowManagerImpl extends WindowManager implements Workspace
         if (displayName == null) {
             displayName = tc.getName();
         }
-        if (NAME_HACK && displayName != null) {
-            //THIS IS FOR DEMO PURPOSES ONLY!  A PROPER API IS NEEDED
-            //(TopComponent.getHtmlDisplayName()), OR
-            //HTML SHOULD BE PRE-SUPPLIED
-            if (displayName.endsWith("*")) { 
-                
-                if (displayName.startsWith("<html>")) {
-                    displayName = displayName.substring(6);
-                }
-                
-                displayName = "<html><b>" + 
-                    displayName.substring(0, displayName.length()-2);
-                
-            } else {
-                
-                int i = displayName.indexOf ("[r/o]"); 
-                if (i > 0) {
-                    
-                    if (displayName.startsWith("<html>")) {
-                        displayName = displayName.substring(6);
-                        i -= 6;
-                    }
-
-                    int roLength = "[r/o]".length();
-                    String nuName = "<html><font color='#555555'><i>" + //NOI18N
-                        displayName.substring (0, i-1);
-                    if (i + roLength < displayName.length()) {
-                        nuName += displayName.substring(i + roLength);
-                    }
-                    displayName = nuName;
-                }
-            }
-        }
         return displayName;
     }
     
@@ -1338,26 +1303,26 @@ public final class WindowManagerImpl extends WindowManager implements Workspace
                 return;
             }
             
-            ArrayList<Runnable> arrCopy = null;
             synchronized (this) {
                 if (arr.isEmpty()) {
                     return;
                 }
                 
-                arrCopy = arr;
-                arr = new ArrayList<Runnable>();
-            }
-
-            Logger perf = Logger.getLogger("org.netbeans.log.startup"); // NOI18N
-            for (Runnable r : arrCopy) {
-                try {
-                    perf.log(Level.FINE, "start", "invokeWhenUIReady: " + r.getClass().getName()); // NOI18N
-                    r.run();
-                    perf.log(Level.FINE, "end", "invokeWhenUIReady: " + r.getClass().getName()); // NOI18N
-                } catch (RuntimeException ex) {
-                    Logger.getLogger(WindowManagerImpl.class.getName()).log(
-                            Level.WARNING, null, ex);
-                }
+                final Runnable toRun = arr.remove(0);
+                SwingUtilities.invokeLater(new Runnable() {
+                    public void run() {
+                        Logger perf = Logger.getLogger("org.netbeans.log.startup"); // NOI18N
+                        try {
+                            perf.log(Level.FINE, "start", "invokeWhenUIReady: " + toRun.getClass().getName()); // NOI18N
+                            toRun.run();
+                            perf.log(Level.FINE, "end", "invokeWhenUIReady: " + toRun.getClass().getName()); // NOI18N
+                        } catch (RuntimeException ex) {
+                            Logger.getLogger(WindowManagerImpl.class.getName()).log(
+                                    Level.WARNING, null, ex);
+                        }
+                        SwingUtilities.invokeLater(Exclusive.this);
+                    }
+                });
             }
         }
 
@@ -1626,6 +1591,39 @@ public final class WindowManagerImpl extends WindowManager implements Workspace
             }
         }
     }
+
+    @Override
+    public TopComponent[] getOpenedTopComponents(Mode mode) {
+        if( mode instanceof ModeImpl ) {
+            java.util.List<TopComponent> openedTcs = ((ModeImpl)mode).getOpenedTopComponents();
+            return openedTcs.toArray(new TopComponent[openedTcs.size()]);
+        }
+        return super.getOpenedTopComponents(mode);
+    }
+
+    public boolean isHeavyWeightShowing() {
+        TopComponent.Registry registry = TopComponent.getRegistry();
+        Set<TopComponent> opened = new HashSet<TopComponent>(registry.getOpened());
+        for( TopComponent tc : opened ) {
+            if( !tc.isShowing() )
+                continue;
+            if( isHeavyWeight( tc ) )
+                return true;
+        }
+        return false;
+    }
+
+    private boolean isHeavyWeight( java.awt.Component c ) {
+        if( null != c && !c.isLightweight() )
+            return true;
+        if( c instanceof Container ) {
+            for( java.awt.Component child : ((Container)c).getComponents() ) {
+                if( isHeavyWeight(child) )
+                    return true;
+            }
+        }
+        return false;
+    }
     
     /**
      * An empty TopComponent needed for deselectEditorTopComponents()
