@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -70,7 +70,6 @@ import org.openide.cookies.SaveCookie;
 import org.openide.filesystems.FileLock;
 import org.openide.filesystems.FileObject;
 import org.openide.filesystems.FileUtil;
-import org.openide.filesystems.Repository;
 import org.openide.loaders.DataFolder;
 import org.openide.loaders.DataObject;
 import org.openide.loaders.DataObjectNotFoundException;
@@ -246,7 +245,7 @@ public final class PersistenceManager implements PropertyChangeListener {
         try {
             if (rootModuleFolder == null) {
                 rootModuleFolder = FileUtil.createFolder(
-                    Repository.getDefault().getDefaultFileSystem().getRoot(), ROOT_MODULE_FOLDER
+                    FileUtil.getConfigRoot(), ROOT_MODULE_FOLDER
                 );
             }
             return rootModuleFolder;
@@ -262,7 +261,7 @@ public final class PersistenceManager implements PropertyChangeListener {
         try {
             if (rootLocalFolder == null) {
                 rootLocalFolder = FileUtil.createFolder(
-                    Repository.getDefault().getDefaultFileSystem().getRoot(), ROOT_LOCAL_FOLDER
+                    FileUtil.getConfigRoot(), ROOT_LOCAL_FOLDER
                 );
             }
             return rootLocalFolder;
@@ -534,6 +533,8 @@ public final class PersistenceManager implements PropertyChangeListener {
                         id2TopComponentMap.put(stringId, new TopComponentReference(tc,stringId));
                         if (persistenceType(tc) == TopComponent.PERSISTENCE_ONLY_OPENED) {
                             topComponentPersistentOnlyOpenedID.add(stringId);
+                        } else if (persistenceType(tc) == TopComponent.PERSISTENCE_NEVER) {
+                            topComponentNonPersistentID.add(stringId);
                         }
                         dataobjectToTopComponentMap.put(dob, stringId);
                     }
@@ -545,11 +546,11 @@ public final class PersistenceManager implements PropertyChangeListener {
                     String excAnnotation = NbBundle.getMessage(
                             PersistenceManager.class, "EXC_BrokenTCSetting", 
                             stringId);
-                    resultExc = new SafeException(new IOException(excAnnotation));
+                    //resultExc = new SafeException(new IOException(excAnnotation));
                     LOG.log(warningLevelForDeserTC(stringId),
                         "[PersistenceManager.getTopComponentForID]" // NOI18N
                         + " Problem when deserializing TopComponent for tcID:'" + stringId + "'. Reason: " // NOI18N
-                        + excAnnotation, resultExc);
+                        + excAnnotation/*, resultExc*/);
                 }
             }
             else {
@@ -641,6 +642,13 @@ public final class PersistenceManager implements PropertyChangeListener {
             return tc;
         }
     }
+
+    DataObject findTopComponentDataObject( String name ) throws IOException {
+        DataObject res = findTopComponentDataObject(getComponentsLocalFolder(), name);
+        if( null == res )
+            res = findTopComponentDataObject(getComponentsLocalFolder(), name);
+        return res;
+    }
     
     /** Try to find the data object representing a top component ref in some folder.
      * Only the TC name is known, but we can guess at some likely filenames before
