@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -58,6 +58,7 @@ import java.awt.Image;
 import java.awt.Rectangle;
 import java.awt.event.*;
 import java.awt.image.BufferedImage;
+import java.io.File;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.util.*;
@@ -72,6 +73,7 @@ import org.netbeans.core.windows.view.ui.toolbars.ToolbarConfiguration;
 import org.openide.LifecycleManager;
 import org.openide.awt.*;
 import org.openide.cookies.InstanceCookie;
+import org.openide.cookies.SaveCookie;
 import org.openide.filesystems.*;
 import org.openide.loaders.DataObject;
 import org.openide.util.*;
@@ -98,6 +100,10 @@ public final class MainWindow extends JFrame {
     
     /** Flag indicating main window is initialized. */ 
     private boolean inited;
+
+    private Lookup.Result <SaveCookie> saveResult;
+    private Lookup.Result <DataObject> dobResult;
+    private LookupListener saveListener;
     
 
     /** Constructs main window. */
@@ -231,7 +237,47 @@ public final class MainWindow extends JFrame {
         });
         //#38810 end
         setTitle(NbBundle.getMessage(MainWindow.class, "CTL_MainWindow_Title_No_Project", System.getProperty("netbeans.buildnumber")));
+        if (Utilities.getOperatingSystem() == Utilities.OS_MAC) {
+            //Show a "save dot" in the close button if a modified file is
+            //being edited
+            //Show the icon of the edited file in the window titlebar like
+            //other mac apps
+            saveResult = Utilities.actionsGlobalContext().lookupResult (SaveCookie.class);
+            dobResult = Utilities.actionsGlobalContext().lookupResult (DataObject.class);
+            if( null != saveResult && null != dobResult ) {
+                saveListener = new LookupListener() {
+                    public void resultChanged(LookupEvent ev) {
+                        if (ev.getSource() == saveResult) {
+                            boolean modified = saveResult.allItems().size() > 0;
+                            getRootPane().putClientProperty ("Window.documentModified", //NOI18N
+                                    modified ? Boolean.TRUE : Boolean.FALSE);
+                        } else if (ev.getSource() == dobResult) {
+                            int count = dobResult.allItems().size();
+                            switch (count) {
+                                case 1 :
+                                    DataObject dob = dobResult.allInstances().iterator().next();
+                                    FileObject file = dob.getPrimaryFile();
+                                    File f = FileUtil.toFile(file);
+                                    if (f != null) {
+                                        getRootPane().putClientProperty("Window.documentFile", f); //NOI18N
+                                        break;
+                                    }
+                                    //Fall through
+                                case 0 :
+                                    //Fall through
+                                default :
+                                    getRootPane().putClientProperty("Window.documentFile", null); //NOI18N
+                            }
+                        }
+                    }
+
+                };
+                saveResult.addLookupListener(saveListener);
+                dobResult.addLookupListener(saveListener);
+            }
+        }
     }
+
     
     private static void decoratePanel (JPanel panel, boolean safeAccess) {
         assert safeAccess || SwingUtilities.isEventDispatchThread () : "Must run in AWT queue.";
@@ -711,5 +757,19 @@ public final class MainWindow extends JFrame {
     public boolean isFullScreenMode() {
         return isFullScreenMode;
     }
+
+    private static class HeavyWeightPopupFactory extends PopupFactory {
+
+        @Override
+        public Popup getPopup(Component owner, Component contents, int x, int y) throws IllegalArgumentException {
+            return new HeavyWeightPopup(owner, contents, x, y);
+        }
+    }
+
+    private static class HeavyWeightPopup extends Popup {
+        public HeavyWeightPopup(Component owner, Component contents, int x, int y) {
+            super( owner, contents, x, y);
+        }
+    }
 }
 
