@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -300,6 +300,8 @@ public final class ToolbarConfiguration implements ToolbarPool.Configuration {
             return null;
         if( rows.size() > 0 && rows.get(rows.size()-1).isEmpty() )
             return null;
+        if( !toolbarPanel.isShowing() )
+            return null;
         int rowHeight = rows.get(0).getHeight();
         int bottom = toolbarPanel.getLocationOnScreen().y + toolbarPanel.getHeight();
         if( screenLocation.y >= bottom && screenLocation.y <= bottom + rowHeight ) {
@@ -435,13 +437,19 @@ public final class ToolbarConfiguration implements ToolbarPool.Configuration {
     private ToolbarConstraints getConstraints( String toolbarName ) {
         ToolbarConstraints tc = collectAllConstraints().get(toolbarName);
         if( null == tc ) {
-            tc = new ToolbarConstraints(toolbarName, ToolbarConstraints.Align.left, true, true);
+            boolean isQuickSearch = "QuickSearch".equals(toolbarName); //NOI18N
+            tc = new ToolbarConstraints(toolbarName,
+                    isQuickSearch ? ToolbarConstraints.Align.right : ToolbarConstraints.Align.left,
+                    true, true);
             ToolbarRow row = null; //TODO find / add row with the best available space
             if( rows.isEmpty() ) {
                 row = new ToolbarRow();
                 rows.add(row);
             } else {
-                row = rows.get(rows.size()-1);
+                if( isQuickSearch )
+                    row = rows.get(0);
+                else
+                    row = rows.get(rows.size()-1);
             }
             row.addConstraint(tc);
             SwingUtilities.invokeLater( new Runnable() {
