@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -146,7 +146,7 @@ public final class ModuleManager {
             // the following call will set it to the same value.
             classLoader.setSystemClassLoader(
                 moduleFactory.getClasspathDelegateClassLoader(this, 
-                    ClassLoader.getSystemClassLoader()));
+                    ModuleManager.class.getClassLoader()));
         }
     }
 
@@ -467,7 +467,18 @@ public final class ModuleManager {
         }
 
         protected @Override boolean shouldDelegateResource(String pkg, ClassLoader parent) {
-            if ((parent == null || parent instanceof MainImpl.BootClassLoader) && !installer.shouldDelegateClasspathResource(pkg)) {
+            ClassLoader trueParent = getParent();
+            boolean parentIsJRE;
+            if (trueParent != null && trueParent.getClass().getName().equals("com.sun.jnlp.JNLPClassLoader")) { // #177120 NOI18N
+                parentIsJRE = false;
+            } else if (parent == null) {
+                parentIsJRE = true;
+            } else if (parent instanceof MainImpl.BootClassLoader) {
+                parentIsJRE = true;
+            } else {
+                parentIsJRE = false;
+            }
+            if (parentIsJRE && !installer.shouldDelegateClasspathResource(pkg)) {
                 return false;
             }
             return super.shouldDelegateResource(pkg, parent);
