@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -61,6 +61,7 @@ import javax.swing.JDialog;
 import javax.swing.event.ChangeListener;
 import org.netbeans.TopSecurityManager;
 import org.netbeans.core.startup.MainLookup;
+import org.netbeans.core.startup.ModuleLifecycleManager;
 import org.netbeans.core.startup.ModuleSystem;
 import org.netbeans.core.startup.layers.SessionManager;
 import org.netbeans.core.ui.SwingBrowser;
@@ -82,7 +83,6 @@ import org.openide.util.RequestProcessor;
 import org.openide.util.Task;
 import org.openide.util.lookup.InstanceContent;
 import org.openide.util.lookup.ServiceProvider;
-import org.openide.windows.Mode;
 import org.openide.windows.TopComponent;
 import org.openide.windows.WindowManager;
 
@@ -234,6 +234,18 @@ public abstract class NbTopManager {
                 }
             });
         }
+
+        @Override
+        public void showURLExternal(final URL u) {
+            Mutex.EVENT.readAccess(new Runnable() {
+                public void run() {
+                    if (htmlViewer == null) {
+                        htmlViewer = new NbBrowser();
+                    }
+                    htmlViewer.showUrlExternal(u);
+                }
+            });
+        }
     }
 
 
@@ -389,7 +401,7 @@ public abstract class NbTopManager {
         for (int i = 0; i < modifs.length; i++) {
             try {
                 dobj = modifs[i];
-                SaveCookie sc = dobj.getCookie(SaveCookie.class);
+                SaveCookie sc = dobj.getLookup().lookup(SaveCookie.class);
 
                 if (sc != null) {
                     org.openide.awt.StatusDisplayer.getDefault().setStatusText(
@@ -560,6 +572,9 @@ public abstract class NbTopManager {
         public void exit() {
             NbTopManager.exit();
         }
+        public @Override void markForRestart() throws UnsupportedOperationException {
+            new ModuleLifecycleManager().markForRestart();
+        }
     }
 
     /** Get the module subsystem. */
@@ -583,6 +598,7 @@ public abstract class NbTopManager {
     public static class NbBrowser {
         
         private HtmlBrowserComponent brComp;
+        private HtmlBrowserComponent externalBrowser;
         private PreferenceChangeListener idePCL;
         private static Lookup.Result factoryResult;
         
@@ -611,6 +627,12 @@ public abstract class NbTopManager {
                 brComp = new HtmlBrowserComponent(browser, true, true);
                 brComp.putClientProperty("TabPolicy", "HideWhenAlone"); // NOI18N
             }
+            browser = IDESettings.getExternalWWWBrowser();
+            if (browser == null) {
+                //external browser is not available, fallback to swingbrowser
+                browser = new SwingBrowser();
+            }
+            externalBrowser = new HtmlBrowserComponent(browser, true, true);
             setListener();
         }
 
@@ -619,16 +641,9 @@ public abstract class NbTopManager {
          * the case of success returns the instance, null otherwise.         
          */
         private HtmlBrowserComponent findOpenedBrowserComponent() {
-            for (Iterator it = WindowManager.getDefault().getModes().iterator(); it.hasNext(); ) {
-                Mode m = (Mode) it.next();
-                if ("editor".equals(m.getName())) { // NOI18N
-                    TopComponent[] tcs = m.getTopComponents();
-                    for (int i = 0; i < tcs.length; i++) {
-                        if (tcs[i] instanceof HtmlBrowserComponent) {
-                            return (HtmlBrowserComponent) tcs[i];
-                        }
-                    }
-                    break;
+            for (TopComponent tc : TopComponent.getRegistry().getOpened() ) {
+                if (tc instanceof HtmlBrowserComponent) {
+                    return (HtmlBrowserComponent) tc;
                 }
             }
             return null;
@@ -638,9 +653,15 @@ public abstract class NbTopManager {
          * @param url URL to be shown
          */
         private void showUrl(URL url) {
-            brComp.open();
-            brComp.requestActive();
-            brComp.setURL(url);
+            brComp.setURLAndOpen(url);
+        }
+
+        /**
+         * Show URL in an external browser.
+         * @param url URL to show
+         */
+        private void showUrlExternal(URL url) {
+            externalBrowser.setURLAndOpen(url);
         }
 
         /**
