@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -147,13 +147,6 @@ implements Runnable, org.netbeans.core.startup.RunLevel {
         Splash.getInstance().increment(10);
         StartLog.logProgress ("Timer initialized"); // NOI18N
 
-        // -----------------------------------------------------------------------------------------------------
-        // 13. Initialize Shortcuts
-        ShortcutsFolder.initShortcuts();
-        Splash.getInstance().increment(1);
-        StartLog.logProgress ("Shortcuts initialized"); // NOI18N
-
-
     // -----------------------------------------------------------------------------------------------------
     // 14. Open main window
         StatusDisplayer.getDefault().setStatusText (NbBundle.getMessage (NonGui.class, "MSG_WindowShowInit"));
