@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -42,13 +42,14 @@
 package org.netbeans.core;
 
 import java.awt.BorderLayout;
-import java.awt.Component;
 import java.awt.Dialog;
 import java.awt.Dimension;
+import java.awt.EventQueue;
 import java.awt.Font;
 import java.awt.GraphicsEnvironment;
 import java.awt.Rectangle;
 import java.awt.Window;
+import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.io.PrintWriter;
 import java.io.StringWriter;
@@ -61,7 +62,6 @@ import java.util.logging.Handler;
 import java.util.logging.Level;
 import java.util.logging.LogRecord;
 import java.util.logging.Logger;
-import javax.swing.Icon;
 import javax.swing.ImageIcon;
 import javax.swing.JButton;
 import javax.swing.JDialog;
@@ -69,15 +69,18 @@ import javax.swing.JPanel;
 import javax.swing.JScrollPane;
 import javax.swing.JTextPane;
 import javax.swing.SwingUtilities;
+import javax.swing.Timer;
 import javax.swing.UIManager;
 import org.netbeans.core.startup.CLIOptions;
 import org.openide.DialogDescriptor;
 import org.openide.DialogDisplayer;
 import org.openide.NotifyDescriptor;
 import org.openide.awt.Mnemonics;
+import org.openide.awt.Notification;
+import org.openide.awt.NotificationDisplayer;
 import org.openide.util.Exceptions;
+import org.openide.util.ImageUtilities;
 import org.openide.util.NbBundle;
-import org.openide.util.Utilities;
 import org.openide.windows.WindowManager;
 
 /**
@@ -122,12 +125,12 @@ public final class NotifyExcPanel extends JPanel implements ActionListener {
     
     /** the last position of the exception dialog window */
     private static Rectangle lastBounds;
+    
+    private static int extraH = 0, extraW = 0;
 
     /** Constructor.
     */
     private NotifyExcPanel () {
-        setPreferredSize(new Dimension(SIZE_PREFERRED_WIDTH,SIZE_PREFERRED_HEIGHT));
-
         java.util.ResourceBundle bundle = org.openide.util.NbBundle.getBundle(NotifyExcPanel.class);
         next = new JButton ();
         Mnemonics.setLocalizedText(next, bundle.getString("CTL_NextException"));
@@ -140,7 +143,7 @@ public final class NotifyExcPanel extends JPanel implements ActionListener {
         details.setDefaultCapable (false);
 
         output = new JTextPane() {
-            public boolean getScrollableTracksViewportWidth() {
+            public @Override boolean getScrollableTracksViewportWidth() {
                 return false;
             }
         };
@@ -177,6 +180,8 @@ public final class NotifyExcPanel extends JPanel implements ActionListener {
         descriptor.setModal( isModalDialogPresent() 
                 && WindowManager.getDefault().getMainWindow().isVisible() );
         
+        setPreferredSize(new Dimension(SIZE_PREFERRED_WIDTH + extraW, SIZE_PREFERRED_HEIGHT + extraH));
+
         dialog = DialogDisplayer.getDefault().createDialog(descriptor);
         if( null != lastBounds )
             dialog.setBounds( lastBounds );
@@ -211,6 +216,9 @@ public final class NotifyExcPanel extends JPanel implements ActionListener {
                 try {
                     Object o = ((Callable<?>)h).call();
                     assert o instanceof JButton;
+                    JButton b = (JButton) o;
+                    extraH += b.getPreferredSize ().height;
+                    extraW += b.getPreferredSize ().width;
                     arr.add(o);
                 } catch (Exception ex) {
                     Exceptions.printStackTrace(ex);
@@ -218,7 +226,7 @@ public final class NotifyExcPanel extends JPanel implements ActionListener {
             }
         }
         
-        arr.add(DialogDescriptor.CLOSED_OPTION);
+        arr.add(NotifyDescriptor.CANCEL_OPTION);
         return arr.toArray();
     }
     
@@ -262,10 +270,21 @@ public final class NotifyExcPanel extends JPanel implements ActionListener {
         }
         
         // #50018 Don't try to show any notify dialog when reporting headless exception
-        if ("java.awt.HeadlessException".equals(t.getClassName()) && GraphicsEnvironment.isHeadless()) {
+        if ("java.awt.HeadlessException".equals(t.getClassName()) && GraphicsEnvironment.isHeadless()) { // NOI18N
             t.printStackTrace(System.err);
             return;
         }
+        //#133092: Error in JDK 5 during printing #6704417.
+        if ("java.lang.NullPointerException".equals(t.getClassName())) { // NOI18N
+            StringWriter sw = new StringWriter();
+            PrintWriter pw = new PrintWriter(sw);
+            t.printStackTrace(pw);
+            if (sw.toString().contains("sun.awt.windows.WComponentPeer.nativeHandleEvent") && // NOI18N
+                System.getProperty("java.version").startsWith("1.5")) { // NOI18N
+                 t.printStackTrace(System.err);
+                 return;
+            }
+        }
 
         SwingUtilities.invokeLater (new Runnable () {
             public void run() {
@@ -311,10 +330,11 @@ public final class NotifyExcPanel extends JPanel implements ActionListener {
                     INSTANCE.updateState(t);
                 } else {
                     // No assertions, use the flashing icon.
-                    if( null != flasher && null == INSTANCE ) {
+                    if( null == INSTANCE ) {
+                        ImageIcon img1 = ImageUtilities.loadImageIcon("org/netbeans/core/resources/exception.gif", true);
+                        String summary = getExceptionSummary(t);
+                        ExceptionFlasher flash = ExceptionFlasher.notify(summary, img1);
                         //exception window is not visible, start flashing the icon
-                        flasher.setToolTipText( getExceptionSummary( t ) );
-                        flasher.startFlashing();
                     } else {
                         //exception window is already visible (or the flashing icon is not available)
                         //so we'll only update the exception window
@@ -486,7 +506,8 @@ public final class NotifyExcPanel extends JPanel implements ActionListener {
     //
 
     public void actionPerformed(final java.awt.event.ActionEvent ev) {
-        if (ev.getSource () == next && exceptions.setNextElement() || ev.getSource () == previous && exceptions.setPreviousElement()) {
+        Object source = ev.getSource();
+        if (source == next && exceptions.setNextElement() || source == previous && exceptions.setPreviousElement()) {
             current = exceptions.get();
             LogRecord rec = new LogRecord(Level.CONFIG, "NotifyExcPanel: " + ev.getActionCommand());// NOI18N
             String message = current.getMessage();
@@ -504,7 +525,7 @@ public final class NotifyExcPanel extends JPanel implements ActionListener {
             return;
         }
 
-        if (ev.getSource () == details) {
+        if (source == details) {
             showDetails = !showDetails;
             lastBounds = null;
             try {
@@ -522,7 +543,11 @@ public final class NotifyExcPanel extends JPanel implements ActionListener {
         }
 
         // bugfix #40834, remove all exceptions to notify when close a dialog
-        if (ev.getSource () == DialogDescriptor.OK_OPTION || ev.getSource () == DialogDescriptor.CLOSED_OPTION) {
+        if (source == NotifyDescriptor.OK_OPTION || source == NotifyDescriptor.CLOSED_OPTION || source == NotifyDescriptor.CANCEL_OPTION) {
+            LogRecord rec = new LogRecord(Level.CONFIG, "NotifyExcPanel:  close");// NOI18N
+            rec.setParameters(null);
+            //log changes in NotifyPanel - dialog is closed - forget previous params
+            Logger.getLogger("org.netbeans.ui.NotifyExcPanel").log(rec);// NOI18N
             try {
                 exceptions.removeAll();
             //Fixed bug #9435, call of setVisible(false) replaced by call of dispose()
@@ -568,41 +593,37 @@ public final class NotifyExcPanel extends JPanel implements ActionListener {
         }
     }
     
-
-    /**
-     * The icon shown in the main status bar that is flashing when an exception
-     * is encountered.
-     */
-    static FlashingIcon flasher = null;
-    
-    /**
-     * Return an icon that is flashing when a new internal exception occurs. 
-     * Clicking the icon opens the regular exception dialog box. The icon
-     * disappears (is hidden) after a short period of time and the exception
-     * list is cleared.
-     *
-     * @return A flashing icon component or null if console logging is switched on.
-     */
-    public static Component getNotificationVisualizer() {
-        //do not create flashing icon if not allowed in system properties
-        if( null == flasher ) {
-            ImageIcon img1 = new ImageIcon( Utilities.loadImage("org/netbeans/core/resources/exception.gif", true) );
-            flasher = new ExceptionFlasher( img1 );
+    static class ExceptionFlasher implements ActionListener {
+        static ExceptionFlasher flash;
+        private static synchronized ExceptionFlasher notify(String summary, ImageIcon icon) {
+            if (flash != null) {
+                flash.timer.restart();
+            } else {
+                flash = new ExceptionFlasher();
+                flash.note = NotificationDisplayer.getDefault().notify(
+                    NbBundle.getMessage(NotifyExcPanel.class, "NTF_ExceptionalExceptionTitle"),
+                    icon, summary,
+                    flash, NotificationDisplayer.Priority.SILENT);
+            }
+            return flash;
         }
-        return flasher;
-    }
+        Notification note;
+        private final Timer timer;
 
-    private static class ExceptionFlasher extends FlashingIcon {
-        static final long serialVersionUID = 1L;
-        
-        public ExceptionFlasher( Icon img1 ) {
-            super( img1 );
+        public ExceptionFlasher() {
+            timer = new Timer(30000, this);
+            timer.setRepeats(false);
+            timer.start();
         }
 
-        /**
-         * User clicked the flashing icon, display the exception window.
-         */
-        protected void onMouseClick() {
+        public void actionPerformed(ActionEvent e) {
+            if (e.getSource() == timer) {
+                timeout();
+                return;
+            }
+            synchronized (ExceptionFlasher.class) {
+                flash = null;
+            }
             if (null != exceptions && exceptions.size() > 0) {
                 if (INSTANCE == null) {
                     INSTANCE = new NotifyExcPanel();
@@ -611,22 +632,20 @@ public final class NotifyExcPanel extends JPanel implements ActionListener {
             }
         }
         
-        /**
-         * The flashing icon disappeared (timed-out), clear the current
-         * exception list.
-         */
-        protected void timeout() {
-            SwingUtilities.invokeLater( new Runnable() {
-                public void run() {
-                    if( null != INSTANCE ) {
-                        return;
-                    }
-                    if( null != exceptions ) {
-                        exceptions.clear();
-                    }
-                    exceptions = null;
+        private void timeout() {
+            synchronized (ExceptionFlasher.class) {
+                assert EventQueue.isDispatchThread();
+                if( null != INSTANCE ) {
+                    return;
                 }
-            });
+                if( null != exceptions ) {
+                    exceptions.clear();
+                }
+                exceptions = null;
+                flash = null;
+                timer.stop();
+                note.clear();
+            }
         }
     }
 
