@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -48,6 +48,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.PrintStream;
 import java.net.URL;
+import java.net.URLConnection;
 import java.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -69,6 +70,7 @@ import java.util.logging.LogRecord;
 import java.util.logging.Logger;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
+import javax.swing.Action;
 import junit.framework.Test;
 import junit.framework.TestSuite;
 import org.netbeans.core.startup.layers.LayerCacheManager;
@@ -88,7 +90,7 @@ import org.openide.modules.Dependency;
 import org.openide.modules.ModuleInfo;
 import org.openide.util.Lookup;
 import org.openide.util.Mutex;
-import org.openide.util.Utilities;
+import org.openide.util.NbCollections;
 
 /** Checks consistency of System File System contents.
  */
@@ -108,8 +110,6 @@ public class ValidateLayerConsistencyTest extends NbTestCase {
         return 1000 * 60 * 10;
     }
     
-    
-    
     public @Override void setUp() throws Exception {
         clearWorkDir();
         Mutex.EVENT.readAccess(new Mutex.Action<Void>() {
@@ -139,7 +139,7 @@ public class ValidateLayerConsistencyTest extends NbTestCase {
         suite.addTest(NbModuleSuite.create(NbModuleSuite.createConfiguration(ValidateLayerConsistencyTest.class).
                 clusters("(?!ergonomics).*").enableClasspathModules(false).enableModules(".*").gui(false)));
         suite.addTest(NbModuleSuite.create(NbModuleSuite.createConfiguration(ValidateLayerConsistencyTest.class).
-                clusters("(platform|harness|ide|websvccommon|gsf|java|profiler|nb)[0-9.]*").enableClasspathModules(false).enableModules(".*").gui(false)));
+                clusters("(platform|harness|ide|websvccommon|java|profiler|nb)[0-9.]*").enableClasspathModules(false).enableModules(".*").gui(false)));
         suite.addTest(NbModuleSuite.create(NbModuleSuite.createConfiguration(ValidateLayerConsistencyTest.class).
                 clusters("(platform|ide)[0-9.]*").enableClasspathModules(false).enableModules(".*").gui(false)));
         return suite;
@@ -266,7 +266,7 @@ public class ValidateLayerConsistencyTest extends NbTestCase {
             
             try {
                 DataObject obj = DataObject.find (fo);
-                DataShadow ds = obj.getCookie(DataShadow.class);
+                DataShadow ds = obj.getLookup().lookup(DataShadow.class);
                 if (ds != null) {
                     Object o = ds.getOriginal();
                     if (o == null) {
@@ -340,32 +340,115 @@ public class ValidateLayerConsistencyTest extends NbTestCase {
         List<String> errors = new ArrayList<String>();
         
         Enumeration<? extends FileObject> files = FileUtil.getConfigRoot().getChildren(true);
-        FILE: while (files.hasMoreElements()) {
+        while (files.hasMoreElements()) {
             FileObject fo = files.nextElement();
             
-            if (skipFile(fo.getPath())) {
+            if (skipFile(fo)) {
                 continue;
             }
             
             try {
                 DataObject obj = DataObject.find (fo);
-                InstanceCookie ic = obj.getCookie(InstanceCookie.class);
+                InstanceCookie ic = obj.getLookup().lookup(InstanceCookie.class);
                 if (ic != null) {
-                    String iof = (String) fo.getAttribute("instanceOf");
-                    if (iof != null) {
-                        for (String clz : iof.split("[, ]+")) {
-                            try {
-                                Class<?> c = Lookup.getDefault().lookup(ClassLoader.class).loadClass(clz);
-                            } catch (ClassNotFoundException x) {
-                                // E.g. Services/Hidden/org-netbeans-lib-jsch-antlibrary.instance in ide cluster
-                                // cannot be loaded (and would just be ignored) if running without java cluster
-                                System.err.println("Warning: skipping " + fo.getPath() + " due to inaccessible interface " + clz);
-                                continue FILE;
-                            }
-                        }
-                    }
                     Object o = ic.instanceCreate ();
                 }
+            } catch (Exception ex) {
+                ByteArrayOutputStream baos = new ByteArrayOutputStream();
+                PrintStream ps = new PrintStream(baos);
+                ex.printStackTrace(ps);
+                ps.flush();
+                errors.add(
+                    "File " + fo.getPath() +
+                    "\nRead from: " + Arrays.toString((Object[])fo.getAttribute("layers")) +
+                    "\nthrew: " + baos);
+            }
+        }
+        
+        assertNoErrors("Some instances cannot be created", errors);
+    }
+
+    public void testActionInstancesOnlyInActionsFolder() {
+        List<String> errors = new ArrayList<String>();
+
+        Enumeration<? extends FileObject> files = FileUtil.getConfigRoot().getChildren(true);
+        FILE: while (files.hasMoreElements()) {
+            FileObject fo = files.nextElement();
+
+            if (skipFile(fo)) {
+                continue;
+            }
+
+            try {
+                DataObject obj = DataObject.find (fo);
+                InstanceCookie ic = obj.getLookup().lookup(InstanceCookie.class);
+                if (ic == null) {
+                    continue;
+                }
+                Object o;
+                try {
+                    o = ic.instanceCreate();
+                } catch (ClassNotFoundException ok) {
+                    // wrong instances are catched by another test
+                    continue;
+                }
+                if (!(o instanceof Action)) {
+                    continue;
+                }
+                if (fo.hasExt("xml")) {
+                    continue;
+                }
+                if (fo.getPath().startsWith("Actions/")) {
+                    continue;
+                }
+                if (fo.getPath().startsWith("Editors/")) {
+                    // editor is a bit different world
+                    continue;
+                }
+                if (fo.getPath().startsWith("Databases/Explorer/")) {
+                    // db explorer actions shall not influence start
+                    // => let them be for now.
+                    continue;
+                }
+                if (fo.getPath().startsWith("WelcomePage/")) {
+                    // welcome screen actions are not intended for end user
+                    continue;
+                }
+                if (fo.getPath().startsWith("Projects/org-netbeans-modules-mobility-project/Actions/")) {
+                    // I am not sure what mobility is doing, but
+                    // I guess I do not need to care
+                    continue;
+                }
+                if (fo.getPath().startsWith("NativeProjects/Actions/")) {
+                    // I should probably report a bug for NativeProjects
+                    continue;
+                }
+                if (fo.getPath().startsWith("contextmenu/uml/")) {
+                    // UML is not the most important thing to fix
+                    continue;
+                }
+                if (fo.getPath().equals("Menu/Help/org-netbeans-modules-j2ee-blueprints-ShowBluePrintsAction.instance")) {
+                    // action included in some binary blob
+                    continue;
+                }
+                if (Boolean.TRUE.equals(fo.getAttribute("misplaced.action.allowed"))) {
+                    // it seems necessary some actions to stay outside
+                    // of the Actions folder
+                    continue;
+                }
+                if (fo.hasExt("shadow")) {
+                    o = fo.getAttribute("originalFile");
+                    if (o instanceof String) {
+                        String origF = o.toString().replaceFirst("\\/*", "");
+                        if (origF.startsWith("Actions/")) {
+                            continue;
+                        }
+                        if (origF.startsWith("Editors/")) {
+                            continue;
+                        }
+                    }
+                }
+                errors.add("File " + fo.getPath() + " represents an action which is not in Actions/ subfolder. Provided by " + Arrays.toString((Object[])fo.getAttribute("layers")));
             } catch (Exception ex) {
                 ByteArrayOutputStream baos = new ByteArrayOutputStream();
                 PrintStream ps = new PrintStream(baos);
@@ -374,8 +457,8 @@ public class ValidateLayerConsistencyTest extends NbTestCase {
                 errors.add ("File " + fo.getPath() + " threw: " + baos);
             }
         }
-        
-        assertNoErrors("Some instances cannot be created", errors);
+
+        assertNoErrors(errors.size() + " actions is not registered properly", errors);
     }
     
     public void testIfOneFileIsDefinedTwiceByDifferentModulesTheyNeedToHaveMutualDependency() throws Exception {
@@ -414,6 +497,10 @@ public class ValidateLayerConsistencyTest extends NbTestCase {
         Map</* path */String,Map</* attr name */String,Map</* module name */String,/* attr value */Object>>> folderAttributes =
                 new TreeMap<String,Map<String,Map<String,Object>>>();
         StringBuffer sb = new StringBuffer();
+        Map<String,URL> hiddenFiles = new HashMap<String, URL>();
+        Set<String> allFiles = new HashSet<String>();
+        final String suffix = "_hidden";
+
         
         boolean atLeastOne = false;
         Enumeration<URL> en = l.getResources("META-INF/MANIFEST.MF");
@@ -437,24 +524,19 @@ public class ValidateLayerConsistencyTest extends NbTestCase {
             java.net.URLConnection connect = layerURL.openConnection ();
             connect.setDefaultUseCaches (false);
             FileSystem fs = new XMLFileSystem(layerURL);
-            
+
             Enumeration<? extends FileObject> all = fs.getRoot().getChildren(true);
             while (all.hasMoreElements ()) {
                 FileObject fo = all.nextElement ();
                 String path = fo.getPath();
-                Map<String,Object> attributes = getAttributes(fo, base);
 
-                /* XXX too many failures to enable yet:
-                // Check for misplaced localizing bundle attributes.
-                // Might be more natural to check in testAreAttributesFine
-                // but easier here because of the way we load one layer at a time.
-                for (Map.Entry<String,Object> entry : attributes.entrySet()) {
-                    if (entry.getKey().equals(SFS_LB) && entry.getValue() instanceof Exception) {
-                        sb.append("Module " + module + " defines an incorrect " + SFS_LB + " on " + path +
-                                ": " + fo.getAttribute(SFS_LB) + "\n");
-                    }
+                if (path.endsWith(suffix)) {
+                    hiddenFiles.put(path, layerURL);
+                } else {
+                    allFiles.add(path);
                 }
-                 */
+
+                Map<String,Object> attributes = getAttributes(fo, base);
 
                 if (fo.isFolder()) {
                     for (Map.Entry<String,Object> attr : attributes.entrySet()) {
@@ -565,7 +647,7 @@ public class ValidateLayerConsistencyTest extends NbTestCase {
             for (Map.Entry<String,Map<String,Object>> entry2 : entry1.getValue().entrySet()) {
                 if (new HashSet<Object>(entry2.getValue().values()).size() > 1) {
                     // XXX currently do not check if the modules are unrelated by dependency.
-                    sb.append("Some modules conflict on the definition of " + entry2.getKey() + " in " + entry1.getKey() + ": " + entry2.getValue() + "\n");
+                    sb.append("Some modules conflict on the definition of " + entry2.getKey() + " for " + entry1.getKey() + ": " + entry2.getValue() + "\n");
                 }
             }
         }
@@ -573,14 +655,58 @@ public class ValidateLayerConsistencyTest extends NbTestCase {
         if (sb.length () > 0) {
             fail ("Some modules override their files and do not depend on each other\n" + sb);
         }
+
+
+        for (Map.Entry<String, URL> e : hiddenFiles.entrySet()) {
+            String p = e.getKey().substring(0, e.getKey().length() - suffix.length());
+            if (allFiles.contains(p)) {
+                continue;
+            }
+            sb.append("file " + e.getKey() + " from " + e.getValue() + " does not hide any other file\n");
+        }
+
+        if (sb.length () > 0) {
+            fail ("There are some useless hidden files\n" + sb);
+        }
     }
     
+    public void testLocalizingBundles() throws Exception {
+        StringBuilder sb = new StringBuilder();
+        for (URL u : NbCollections.iterable(Lookup.getDefault().lookup(ClassLoader.class).getResources("META-INF/MANIFEST.MF"))) {
+            String layer;
+            InputStream is = u.openStream();
+            try {
+                layer = new Manifest(is).getMainAttributes().getValue("OpenIDE-Module-Layer");
+                if (layer == null) {
+                    continue;
+                }
+            } finally {
+                is.close();
+            }
+            URL base = new URL(u, "../");
+            URL layerURL = new URL(base, layer);
+            URLConnection connect = layerURL.openConnection();
+            connect.setDefaultUseCaches(false);
+            for (FileObject fo : NbCollections.iterable(new XMLFileSystem(layerURL).getRoot().getChildren(true))) {
+                Object v = getAttributes(fo, base).get(SFS_LB);
+                if (v instanceof Exception) {
+                    sb.append(layerURL + ": " + v + "\n");
+                }
+            }
+        }
+        if (sb.length() > 0) {
+            /* Too many failures to solve right now.
+            fail("Some localizing bundle declarations are wrong\n" + sb);
+             */
+            System.err.print(sb);
+        }
+    }
+
     public void testNoWarningsFromLayerParsing() throws Exception {
         ClassLoader l = Lookup.getDefault().lookup(ClassLoader.class);
         assertNotNull ("In the IDE mode, there always should be a classloader", l);
         
         List<URL> urls = new ArrayList<URL>();
-        boolean atLeastOne = false;
         Enumeration<URL> en = l.getResources("META-INF/MANIFEST.MF");
         while (en.hasMoreElements ()) {
             URL u = en.nextElement();
@@ -595,8 +721,6 @@ public class ValidateLayerConsistencyTest extends NbTestCase {
             if (module == null) continue;
             String layer = mf.getMainAttributes ().getValue ("OpenIDE-Module-Layer");
             if (layer == null) continue;
-            
-            atLeastOne = true;
             URL layerURL = new URL(u, "../" + layer);
             urls.add(layerURL);
         }
@@ -744,7 +868,9 @@ public class ValidateLayerConsistencyTest extends NbTestCase {
         return attrs;
     }
     
-    private boolean skipFile (String s) {
+    private boolean skipFile(FileObject fo) {
+        String s = fo.getPath();
+
         if (s.startsWith ("Templates/") && !s.startsWith ("Templates/Services")) {
             if (s.endsWith (".shadow") || s.endsWith (".java")) {
                 return true;
@@ -765,6 +891,58 @@ public class ValidateLayerConsistencyTest extends NbTestCase {
         if (s.startsWith ("EnvironmentProviders/ProfileTypes/Execution/nb-j2ee-deployment.instance")) return true;
         if (s.startsWith ("Shortcuts/D-BACK_QUOTE.shadow")) return true;
         
+        String iof = (String) fo.getAttribute("instanceOf");
+        if (iof != null) {
+            for (String clz : iof.split("[, ]+")) {
+                try {
+                    Class<?> c = Lookup.getDefault().lookup(ClassLoader.class).loadClass(clz);
+                } catch (ClassNotFoundException x) {
+                    // E.g. Services/Hidden/org-netbeans-lib-jsch-antlibrary.instance in ide cluster
+                    // cannot be loaded (and would just be ignored) if running without java cluster
+                    System.err.println("Warning: skipping " + fo.getPath() + " due to inaccessible interface " + clz);
+                    return true;
+                }
+            }
+        }
+
         return false;
     }
+
+    public void testKeymapOverrides() throws Exception { // #170677
+        List<String> warnings = new ArrayList<String>();
+        FileObject[] keymaps = FileUtil.getConfigFile("Keymaps").getChildren();
+        Map<String,Integer> definitionCountById = new HashMap<String,Integer>();
+        for (FileObject keymap : keymaps) {
+            for (FileObject shortcut : keymap.getChildren()) {
+                DataObject d = DataObject.find(shortcut);
+                if (d instanceof DataShadow) {
+                    String id = ((DataShadow) d).getOriginal().getPrimaryFile().getPath();
+                    Integer prior = definitionCountById.get(id);
+                    definitionCountById.put(id, prior == null ? 1 : prior + 1);
+                } else if (!d.getPrimaryFile().hasExt("shadow")) {
+                    warnings.add("Anomalous file " + d);
+                } // else #172453: BrokenDataShadow, OK
+            }
+        }
+        for (FileObject shortcut : FileUtil.getConfigFile("Shortcuts").getChildren()) {
+            DataObject d = DataObject.find(shortcut);
+            if (d instanceof DataShadow) {
+                String id = ((DataShadow) d).getOriginal().getPrimaryFile().getPath();
+                if (!org.openide.util.Utilities.isMac() && // Would fail on Mac due to applemenu module
+                        Integer.valueOf(keymaps.length).equals(definitionCountById.get(id)))
+                {
+                    String layers = Arrays.toString((URL[]) d.getPrimaryFile().getAttribute("layers"));
+                    warnings.add(d.getPrimaryFile().getPath() + " " + layers + " useless since " + id + " is bound (somehow) in all keymaps");
+                }
+            } else if (!d.getPrimaryFile().hasExt("shadow")) {
+                warnings.add("Anomalous file " + d);
+            }
+        }
+        // XXX consider also checking for bindings in Shortcuts/ which are overridden in all keymaps or at least NetBeans
+        // taking into consideration O- and D- virtual modifiers
+        // (this is likely to be more common, e.g. mysterious Shortcuts/D-A.shadow in uml.drawingarea)
+        // XXX check for shortcut conflict between Shortcuts and each keymap, e.g. Ctrl-R in Eclipse keymap
+        assertNoErrors("Some shortcuts were overridden by keymaps", warnings);
+    }
+
 }
