@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -204,7 +204,7 @@ public final class PasteAction extends CallbackSystemAction {
             new ActionPT(t, ev.getActionCommand());
         } else {
             Logger.getLogger(PasteAction.class.getName()).log(Level.WARNING, null,
-                              new IllegalStateException("No paste types available when performing paste action")); // NOI18N
+                              new IllegalStateException("No paste types available when performing paste action. ActionEvent: " + ev)); // NOI18N
         }
     }
 
