@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -47,17 +47,26 @@ import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.net.URL;
 import java.util.logging.Level;
-import java.util.logging.Logger;
+import java.util.prefs.BackingStoreException;
+import java.util.prefs.PreferenceChangeEvent;
+import java.util.prefs.PreferenceChangeListener;
+import java.util.prefs.Preferences;
 import javax.swing.AbstractAction;
 import javax.swing.Action;
 import javax.swing.Icon;
 import javax.swing.ImageIcon;
+import javax.swing.JMenuItem;
+import junit.framework.TestCase;
 import org.netbeans.junit.Log;
 import org.netbeans.junit.NbTestCase;
+import org.netbeans.junit.RandomlyFails;
 import org.openide.filesystems.FileObject;
 import org.openide.filesystems.FileUtil;
 import org.openide.util.ContextAwareAction;
+import org.openide.util.Exceptions;
 import org.openide.util.Lookup;
+import org.openide.util.NbPreferences;
+import org.openide.util.actions.Presenter;
 import org.openide.util.lookup.AbstractLookup;
 import org.openide.util.lookup.InstanceContent;
 
@@ -83,12 +92,12 @@ public class AlwaysEnabledActionTest extends NbTestCase implements PropertyChang
         myListenerCounter = 0;
         MyAction.last = null;
     }
-    
+
     @Override
     protected boolean runInEQ() {
         return true;
     }
-    
+
     public void testIconIsCorrect() throws Exception {
         myListenerCounter = 0;
         myIconResourceCounter = 0;
@@ -104,7 +113,7 @@ public class AlwaysEnabledActionTest extends NbTestCase implements PropertyChang
         //Object icon = a.getValue(a.ICON)
             
         assertEquals("Right localized name", "Icon &Name Action", name);
-        assertEquals("Mnemonic is N", Character.valueOf('N'), mnem);
+        assertEquals("Mnemonic is N", (int)'N', mnem);
         assertNotNull("small icon present", smallIcon);
 
         assertEquals("once icon called", 1, myIconResourceCounter);
@@ -201,7 +210,7 @@ public class AlwaysEnabledActionTest extends NbTestCase implements PropertyChang
         //Object icon = a.getValue(a.ICON)
 
         assertEquals("Right localized name", "Icon &Name Action", name);
-        assertEquals("Mnemonic is N", Character.valueOf('N'), mnem);
+        assertEquals("Mnemonic is N", (int) 'N', mnem);
         assertNotNull("small icon present", smallIcon);
 
         assertEquals("once icon called", 1, myIconResourceCounter);
@@ -252,7 +261,7 @@ public class AlwaysEnabledActionTest extends NbTestCase implements PropertyChang
         //Object icon = a.getValue(a.ICON)
 
         assertEquals("Right localized name", "Icon &Name Action", name);
-        assertEquals("Mnemonic is N", Character.valueOf('N'), mnem);
+        assertEquals("Mnemonic is N", (int) 'N', mnem);
         assertNotNull("small icon present", smallIcon);
 
         assertEquals("once icon called", 1, myIconResourceCounter);
@@ -333,6 +342,60 @@ public class AlwaysEnabledActionTest extends NbTestCase implements PropertyChang
         assertEquals("MyNamedAction", MyAction.last.getValue(Action.NAME));
     }
 
+    public void testPreferencesAction() throws Exception {
+//        checkPreferencesAction("testSystemPreferences.instance", Preferences.systemRoot());
+        checkPreferencesAction("testUserPreferences.instance", "user:", Preferences.userRoot());
+        checkPreferencesAction("testNbPreferences.instance", "", NbPreferences.root());
+        checkPreferencesAction("testCustomPreferences.instance", "user:", Preferences.userRoot()); // customPreferences() uses "myNode" subnode
+    }
+
+    private void checkPreferencesAction(String actionFileName, String preferencesNodePrefix, Preferences prefsRoot) throws Exception {
+        Action a = readAction(actionFileName);
+        Preferences prefsNode = prefsRoot.node("myNode");
+        checkPreferencesAction(a, prefsNode);
+        a = Actions.checkbox(preferencesNodePrefix + "/myNode", "myKey", null, null, false);
+        checkPreferencesAction(a, prefsNode);
+    }
+
+    private void checkPreferencesAction(Action a, Preferences prefsNode) throws Exception {
+        prefsNode.putBoolean("myKey", true);
+        prefsNode.sync();
+        class L implements PreferenceChangeListener {
+            boolean notified;
+
+            public synchronized void preferenceChange(PreferenceChangeEvent evt) {
+                notified = true;
+                notifyAll();
+            }
+
+            public synchronized void waitFor() throws Exception {
+                while (!notified) {
+                    wait();
+                }
+                notified = false;
+            }
+        }
+        L listener = new L();
+
+        // Verify value
+        assertTrue("Expected true as preference value", prefsNode.getBoolean("myKey", false));
+
+        TestCase.assertTrue("Expected to be instance of Presenter.Menu", a instanceof Presenter.Menu);
+        JMenuItem item = ((Presenter.Menu) a).getMenuPresenter();
+        TestCase.assertTrue("Expected to be selected", item.isSelected());
+        prefsNode.addPreferenceChangeListener(listener);
+        prefsNode.putBoolean("myKey", false);
+        prefsNode.sync();
+        listener.waitFor();
+        TestCase.assertFalse("Expected to not be selected", item.isSelected());
+        a.actionPerformed(null); // new ActionEvent(null, 0, ""));
+        listener.waitFor();
+        TestCase.assertTrue("Expected to be selected", item.isSelected());
+        prefsNode.putBoolean("myKey", false);
+        prefsNode.sync();
+        listener.waitFor();
+    }
+
     private static void assertPropertyPropagated(String propertyName, Object value, Action a, Action delegate) {
         assertEquals("Action's property \"" + propertyName + "\"", value, a.getValue(propertyName));
         assertEquals("Delegate's property \"" + propertyName + "\"", value, delegate.getValue(propertyName));
@@ -367,7 +430,7 @@ public class AlwaysEnabledActionTest extends NbTestCase implements PropertyChang
     private static int myIconResourceCounter;
     private static String myIconResource() {
         myIconResourceCounter++;
-        return "/org/openide/awt/TestIcon.png";
+        return "org/openide/awt/TestIcon.png";
     }
     
     
@@ -418,4 +481,35 @@ public class AlwaysEnabledActionTest extends NbTestCase implements PropertyChang
         }
     }
 
+    private static final class PreferencesAction extends AbstractAction {
+        static PreferencesAction last;
+        int performedCount;
+
+        PreferencesAction() {
+            last = this;
+        }
+
+        public void actionPerformed(ActionEvent e) {
+            performedCount++;
+        }
+    }
+    static Action preferencesAction() {
+        return new PreferencesAction();
+    }
+
+    private static final Preferences customPrefs;
+    
+    static {
+        customPrefs = Preferences.userRoot().node("/myNode");
+        try {
+            customPrefs.sync();
+        } catch (BackingStoreException ex) {
+            Exceptions.printStackTrace(ex);
+        }
+    }
+
+    public static Preferences customPreferences() {
+        return customPrefs;
+    }
+
 }
