@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -43,6 +43,7 @@ package org.openide;
 
 import java.awt.Component;
 import java.awt.Dimension;
+import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
 import java.lang.reflect.InvocationTargetException;
@@ -53,8 +54,6 @@ import javax.swing.JPanel;
 import javax.swing.JTextArea;
 import javax.swing.JTextField;
 import javax.swing.UIManager;
-import javax.swing.border.CompoundBorder;
-import javax.swing.border.EmptyBorder;
 import org.jdesktop.layout.GroupLayout;
 import org.openide.awt.Mnemonics;
 import org.openide.util.NbBundle;
@@ -106,6 +105,27 @@ public class NotifyDescriptor extends Object {
     /** Name of property for the OK button validation. */
     public static final String PROP_VALID = "valid"; // NOI18N
 
+    /** Name of property for the error message at the bottom of the wizard.
+     * To set such message use {@link #createNotificationLineSupport}
+     *
+     * @since 7.10
+     */
+    public static final String PROP_ERROR_NOTIFICATION = "errorNotification"; // NOI18N
+
+    /** Name of property for the error message at the bottom of the wizard.
+     * To set such message use {@link #createNotificationLineSupport}
+     *
+     * @since 7.10
+     */
+    public static final String PROP_WARNING_NOTIFICATION = "warningNotification"; // NOI18N
+
+    /** Name of property for the error message at the bottom of the wizard.
+     * To set such message use {@link #createNotificationLineSupport}
+     *
+     * @since 7.10
+     */
+    public static final String PROP_INFO_NOTIFICATION = "infoNotification"; // NOI18N
+
     //
     // Return values
     //
@@ -194,6 +214,11 @@ public class NotifyDescriptor extends Object {
     /** Is OK button valid (enabled). */
     private boolean valid = true;
 
+    private NotificationLineSupport notificationLineSupport = null;
+    private String infoMsg;
+    private String warnMsg;
+    private String errMsg;
+
     /** The object specifying the detail object. */
 
     //  private Object detail;
@@ -579,6 +604,95 @@ public class NotifyDescriptor extends Object {
         return title;
     }
 
+    /** Create {@link NotificationLineSupport} if you want to notify users
+     * using info/warning/error messages in designed line at the bottom
+     * of your dialog. These message will be labelled with appropriate icons.
+     * <br>
+     * Note: Call this method <b>before</b> you call {@link DialogDisplayer#createDialog}
+     *
+     * @return NotificationLineSupport
+     * @since 7.10
+     */
+    public final NotificationLineSupport createNotificationLineSupport() {
+        notificationLineSupport = new NotificationLineSupport (this);
+        return notificationLineSupport;
+    }
+
+    /** Returns NotificationLineSupport if it was created or <code>null</code> if doesn't.
+     * <br>
+     * Note: NotificationLineSupport will not be created by default, API client
+     * has to create this support purposely with the exception {@link WizardDescriptor}
+     * which has such capability longer.
+     *
+     * @see #createNotificationLineSupport() 
+     * @return NotificationLineSupport or null if was not created yet
+     * @since 7.10
+     */
+    public final NotificationLineSupport getNotificationLineSupport() {
+        return notificationLineSupport;
+    }
+
+    void setInformationMessage (String msg) {
+        if (notificationLineSupport == null) {
+            throw new IllegalStateException ("NotificationLineSupport wasn't created yet.");
+        }
+        infoMsg = msg;
+        warnMsg = null;
+        errMsg = null;
+        firePropertyChange (PROP_INFO_NOTIFICATION, null, msg);
+    }
+
+    void setWarningMessage (String msg) {
+        if (notificationLineSupport == null) {
+            throw new IllegalStateException ("NotificationLineSupport wasn't created yet.");
+        }
+        infoMsg = null;
+        warnMsg = msg;
+        errMsg = null;
+        firePropertyChange (PROP_WARNING_NOTIFICATION, null, msg);
+    }
+
+    void setErrorMessage (String msg) {
+        if (notificationLineSupport == null) {
+            throw new IllegalStateException ("NotificationLineSupport wasn't created yet.");
+        }
+        infoMsg = null;
+        warnMsg = null;
+        errMsg = msg;
+        firePropertyChange (PROP_ERROR_NOTIFICATION, null, msg);
+    }
+
+    String getInformationMessage () {
+        if (notificationLineSupport == null) {
+            throw new IllegalStateException ("NotificationLineSupport wasn't created yet.");
+        }
+        return infoMsg;
+    }
+
+    String getWarningMessage () {
+        if (notificationLineSupport == null) {
+            throw new IllegalStateException ("NotificationLineSupport wasn't created yet.");
+        }
+        return warnMsg;
+    }
+
+    String getErrorMessage () {
+        if (notificationLineSupport == null) {
+            throw new IllegalStateException ("NotificationLineSupport wasn't created yet.");
+        }
+        return errMsg;
+    }
+
+    void clearMessages () {
+        if (notificationLineSupport == null) {
+            throw new IllegalStateException ("NotificationLineSupport wasn't created yet.");
+        }
+        infoMsg = null;
+        warnMsg = null;
+        errMsg = null;
+        firePropertyChange (PROP_INFO_NOTIFICATION, null, null);
+    }
+
     /**
     * Define a detail message to be reported.  In the most common usage,
     * this message is just a <code>String</code>.  However, the type of this
@@ -903,16 +1017,15 @@ public class NotifyDescriptor extends Object {
         */
         protected Component createDesign(final String text) {
             JPanel panel = new JPanel();
+            panel.setOpaque (false);
 
             JLabel textLabel = new JLabel();
             Mnemonics.setLocalizedText(textLabel, text);
 
-            textLabel.setBorder(new EmptyBorder(0, 0, 0, 10));
             boolean longText = text.length () > 80;
             textField = new JTextField(25);
             textLabel.setLabelFor(textField);
             
-            textField.setBorder(new CompoundBorder(textField.getBorder(), new EmptyBorder(2, 0, 2, 0)));
             textField.requestFocus();
             
             GroupLayout layout = new GroupLayout(panel);
@@ -936,7 +1049,8 @@ public class NotifyDescriptor extends Object {
                         .addContainerGap()
                         .add(textLabel)
                         .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
-                        .add(textField))
+                        .add(textField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 207, Short.MAX_VALUE)
+                        .addContainerGap())
                 );
             }
             if (longText) {
@@ -951,14 +1065,14 @@ public class NotifyDescriptor extends Object {
                 );
             } else {
                 layout.setVerticalGroup(
-                    layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
-                    .add(layout.createSequentialGroup()
-                        .addContainerGap()
-                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
-                            .add(textLabel)
-                            .add(textField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
-                        .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
-                );
+                            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
+                            .add(layout.createSequentialGroup()
+                                .addContainerGap()
+                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
+                                    .add(textLabel)
+                                    .add(textField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
+                                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
+                        );
             }
 
             javax.swing.KeyStroke enter = javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_ENTER, 0);
