@@ -94,6 +94,7 @@ import org.openide.awt.MouseUtils;
 import org.openide.explorer.ExplorerManager;
 import org.openide.explorer.propertysheet.PropertyPanel;
 import org.openide.nodes.Node;
+import org.openide.util.Exceptions;
 import org.openide.util.Lookup;
 import org.openide.util.NbBundle;
 import org.openide.util.WeakListeners;
@@ -153,7 +154,6 @@ public class OutlineView extends JScrollPane {
 
     /** Listener on keystroke to invoke default action */
     private ActionListener defaultTreeActionListener;
-    private final String nodesColumnLabel;
 
     /** Creates a new instance of TableView */
     public OutlineView() {
@@ -165,7 +165,6 @@ public class OutlineView extends JScrollPane {
         treeModel = new NodeTreeModel();
         rowModel = new PropertiesRowModel();
         model = createOutlineModel(treeModel, rowModel, nodesColumnLabel);
-        this.nodesColumnLabel = nodesColumnLabel;
         outline = new OutlineViewOutline(model, rowModel);
         rowModel.setOutline(outline);
         outline.setRenderDataProvider(new NodeRenderDataProvider(outline));
@@ -473,6 +472,7 @@ public class OutlineView extends JScrollPane {
     @Override
     public void removeNotify () {
         super.removeNotify ();
+        outline.getSelectionModel().clearSelection();
         outline.getSelectionModel().removeListSelectionListener(managerListener);
         manager.removePropertyChangeListener (wlpc);
         manager.removeVetoableChangeListener (wlvc);
@@ -604,33 +604,6 @@ public class OutlineView extends JScrollPane {
             }
         }
 
-        @Override
-        public void mouseClicked (MouseEvent e) {
-            int selRow = outline.rowAtPoint (e.getPoint ());
-
-            if ((selRow != -1) && SwingUtilities.isLeftMouseButton (e) && MouseUtils.isDoubleClick (e)) {
-                // Default action.
-                if (outline.getSelectedColumn () == 0) {
-                    TreePath selPath = outline.getClosestPathForLocation (e.getX (), e.getY ());
-                    Node node = Visualizer.findNode (selPath.getLastPathComponent ());
-                    if (node.isLeaf ()) {
-                        Action a = TreeView.takeAction (node.getPreferredAction (), node);
-
-                        if (a != null) {
-                            if (a.isEnabled ()) {
-                                a.actionPerformed (new ActionEvent (node, ActionEvent.ACTION_PERFORMED, "")); // NOI18N
-                            } else {
-                                Logger.getLogger (OutlineView.class.getName ()).info ("Action " + a + " on node " + node + " is disabled");
-                            }
-
-                            e.consume ();
-                            return;
-                        }
-                    }
-                }
-            }
-            super.mouseClicked (e);
-        }
     }
 
     /**
@@ -828,13 +801,6 @@ public class OutlineView extends JScrollPane {
             if (manager == null) return; // the tree view has been removed before the event got delivered
             if (evt.getPropertyName().equals(ExplorerManager.PROP_ROOT_CONTEXT)) {
                 synchronizeRootContext();
-                if (nodesColumnLabel == null && model instanceof NodeOutlineModel) {
-                    String oldLabel = ((NodeOutlineModel) model).getColumnName(0);
-                    String newLabel = manager.getRootContext().getDisplayName();
-                    if (oldLabel != null && ! oldLabel.equals(newLabel)) {
-                        ((NodeOutlineModel) model).setNodesColumnLabel(newLabel);
-                    }
-                }
             }
             if (evt.getPropertyName().equals(ExplorerManager.PROP_SELECTED_NODES)) {
                 synchronizeSelectedNodes(true);
@@ -862,8 +828,6 @@ public class OutlineView extends JScrollPane {
                 }
             }
         }
-        public void run() {
-        }
     }
 
 
@@ -953,6 +917,27 @@ public class OutlineView extends JScrollPane {
                 }
             }
 
+            if (row != -1 && e instanceof MouseEvent &&
+                    SwingUtilities.isLeftMouseButton ((MouseEvent) e) &&
+                    ((MouseEvent) e).getClickCount() > 1) {
+                // Default action.
+                if (column == 0) {
+                    Node node = Visualizer.findNode (o);
+                    if (node != null) {
+                        if (node.isLeaf () && !node.canRename()) {
+                            Action a = TreeView.takeAction (node.getPreferredAction (), node);
+
+                            if (a != null) {
+                                if (a.isEnabled ()) {
+                                    a.actionPerformed (new ActionEvent (node, ActionEvent.ACTION_PERFORMED, "")); // NOI18N
+                                } else {
+                                    Logger.getLogger (OutlineView.class.getName ()).info ("Action " + a + " on node " + node + " is disabled");
+                                }
+                            }
+                        }
+                    }
+                }
+            }
             boolean res = super.editCellAt(row, column, e);
             if( !res && e instanceof MouseEvent ) {
                 //try invoking custom editor on disabled cell
