@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -55,11 +55,14 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.WeakHashMap;
+import java.util.logging.Level;
+import java.util.logging.LogRecord;
 import java.util.logging.Logger;
 import org.netbeans.modules.openide.filesystems.declmime.MIMEResolverImpl;
 import org.openide.util.Lookup;
 import org.openide.util.LookupEvent;
 import org.openide.util.LookupListener;
+import org.openide.util.NbBundle;
 import org.openide.util.Union2;
 
 /**
@@ -96,18 +99,13 @@ final class MIMESupport extends Object {
 
     /** Asks all registered subclasses of MIMEResolver to resolve FileObject passed as parameter.
      * @param fo is FileObject, whose MIME should be resolved
-     * @param def the default value to return or null
      * @param withinMIMETypes an array of MIME types which only should be considered
      * @return  MIME type or null if not resolved*/
-    static String findMIMEType(FileObject fo, String def, String... withinMIMETypes) {
+    static String findMIMEType(FileObject fo, String... withinMIMETypes) {
         if (!fo.isValid() || fo.isFolder()) {
             return null;
         }
 
-        if ((def != null) && !CachedFileObject.isAnyResolver()) {
-            return def;
-        }
-
         CachedFileObject cfo = null;
 
         try {
@@ -122,7 +120,7 @@ final class MIMESupport extends Object {
                 lastCfo = EMPTY;
             }
 
-            return cfo.getMIMEType(def, withinMIMETypes);
+            return cfo.getMIMEType(withinMIMETypes);
         } finally {
             synchronized (lock) {
                 lastFo = new WeakReference<FileObject>(fo);
@@ -170,7 +168,7 @@ final class MIMESupport extends Object {
                     creators = resolvers.second();
                     if (creators.contains (Thread.currentThread())) {
                         // prevent stack overflow
-                        ERR.fine("Stack Overflow prevention. Returning previousResolvers: " + previousResolvers);
+                        if (ERR.isLoggable(Level.FINE)) ERR.fine("Stack Overflow prevention. Returning previousResolvers: " + previousResolvers);
                         MIMEResolver[] toRet = previousResolvers;
                         if (toRet == null) {
                             toRet = new MIMEResolver[0];
@@ -212,7 +210,7 @@ final class MIMESupport extends Object {
                     previousResolvers = null;
                     ERR.fine("Resolvers assigned"); // NOI18N
                 } else {
-                    ERR.fine("Somebody else computes resolvers: " + resolvers); // NOI18N
+                    if (ERR.isLoggable(Level.FINE)) ERR.fine("Somebody else computes resolvers: " + resolvers); // NOI18N
                 }
 
 
@@ -241,9 +239,14 @@ final class MIMESupport extends Object {
         };
         private static final FileChangeListener weakDeclarativeFolderListener = FileUtil.weakFileChangeListener(declarativeFolderListener, null);
         private static final Map<FileObject,MIMEResolver> declarativeResolverCache = new WeakHashMap<FileObject,MIMEResolver>();
+        // holds reference to not loose FileChangeListener
+        private static FileObject declarativeFolder = null;
+
         private static synchronized List<MIMEResolver> declarativeResolvers() {
             List<MIMEResolver> declmimes = new ArrayList<MIMEResolver>();
-            FileObject declarativeFolder = Repository.getDefault().getDefaultFileSystem().findResource("Services/MIMEResolver"); // NOI18N
+            if (declarativeFolder == null) {
+                declarativeFolder = FileUtil.getConfigFile("Services/MIMEResolver"); // NOI18N
+            }
             if (declarativeFolder != null) {
                 for (FileObject f : Ordering.getOrder(Arrays.asList(declarativeFolder.getChildren()), true)) {
                     if (f.hasExt("xml")) { // NOI18N
@@ -275,15 +278,27 @@ final class MIMESupport extends Object {
 
         @Override
         public String getMIMEType() {
-            return getMIMEType(null);
+            return getMIMEType((String[]) null);
         }
 
-        public String getMIMEType(String def, String... withinMIMETypes) {
-            if (mimeType == null) {
-                mimeType = resolveMIME(def, withinMIMETypes);
+        public String getMIMEType(String... withinMIMETypes) {
+            String resolvedMimeType = mimeType;
+            if (resolvedMimeType == null) {
+                resolvedMimeType = resolveMIME(withinMIMETypes);
+                if (resolvedMimeType == null) {
+                    // fallback for xml files to be recognized e.g. in platform without any MIME resolver registered
+                    if (getExt().toLowerCase().equals("xml")) {  //NOI18N
+                        resolvedMimeType = "text/xml"; // NOI18N
+                    } else {
+                        // general fallback
+                        resolvedMimeType = "content/unknown"; // NOI18N
+                    }
+                } else if (withinMIMETypes.length == 0) {
+                    // cache resolved MIME type only for unrestricted query
+                    mimeType = resolvedMimeType;
+                }
             }
-
-            return mimeType;
+            return resolvedMimeType;
         }
 
         /** Decides whether given MIMEResolver is capable to resolve at least
@@ -320,7 +335,7 @@ final class MIMESupport extends Object {
             return false;
         }
         
-        private String resolveMIME(String def, String... withinMIMETypes) {
+        private String resolveMIME(String... withinMIMETypes) {
             String retVal = null;
             MIMEResolver[] local = getResolvers();
 
@@ -334,12 +349,6 @@ final class MIMESupport extends Object {
                         return retVal;
                     }
                 }
-
-                if (def != null) {
-                    return def;
-                }
-
-                return "content/unknown"; // NOI18N
             } finally {
                 if (fixIt != null) {
                     fixIt.internalClose();
@@ -347,6 +356,7 @@ final class MIMESupport extends Object {
 
                 fixIt = null;
             }
+            return retVal;
         }
 
         public java.util.Date lastModified() {
@@ -358,6 +368,12 @@ final class MIMESupport extends Object {
 
         public InputStream getInputStream() throws java.io.FileNotFoundException {
             if (fixIt == null) {
+                if (ERR.isLoggable(Level.FINE)) {
+                    LogRecord rec = new LogRecord(Level.FINE, "MSG_CACHED_INPUT_STREAM");
+                    rec.setParameters(new Object[] { this });
+                    rec.setResourceBundle(NbBundle.getBundle(MIMESupport.class));
+                    ERR.log(rec);
+                }
                 InputStream is = fileObj.getInputStream();
 
                 if (!(is instanceof BufferedInputStream)) {
@@ -662,21 +678,21 @@ final class MIMESupport extends Object {
 
        private boolean ensureBufferLength(int newLen) throws IOException {
            if (!eof && newLen > len) {
-                byte[] tmpBuffer = new byte[len + newLen];
+                byte[] tmpBuffer = new byte[newLen];
                 if (len > 0) {
                     System.arraycopy(buffer, 0, tmpBuffer, 0, len);
                 }
-                int readLen = inputStream.read(tmpBuffer, len, newLen);
+                int readLen = inputStream.read(tmpBuffer, len, newLen - len);
                 if ((readLen > 0)) {
                     buffer = tmpBuffer;
-                    len += readLen;                                    
+                    len += readLen;
                 } else {
                     eof = true;
                 }
            }
            return len >= newLen;
         }
-        
+
         @Override
         public int read(byte[] b, int off, int blen) throws IOException {
             ensureBufferLength(pos + blen);
@@ -688,7 +704,7 @@ final class MIMESupport extends Object {
             }
             return retval;
         }
-        
+
 
         public int read() throws IOException {
             int retval = -1;
@@ -699,7 +715,7 @@ final class MIMESupport extends Object {
             }
             return retval;
         }
-        
+
         void cacheToStart() {
             pos = 0;
             eof = false;
