@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -52,6 +52,7 @@ import java.util.List;
 import java.util.Set;
 import java.util.StringTokenizer;
 import org.openide.util.Enumerations;
+import org.openide.util.NbBundle;
 import org.openide.util.NbCollections;
 import org.openide.util.actions.SystemAction;
 
@@ -227,7 +228,7 @@ public class MultiFileSystem extends FileSystem {
     /** The name of the filesystem.
     */
     public String getDisplayName() {
-        return getString("CTL_MultiFileSystem");
+        return NbBundle.getMessage(MultiFileSystem.class, "CTL_MultiFileSystem");
     }
 
     /** Root of the filesystem.
@@ -441,7 +442,7 @@ public class MultiFileSystem extends FileSystem {
     protected FileSystem createWritableOn(String name)
     throws IOException {
         if (isReadOnly()) {
-            FSException.io("EXC_FSisRO", getDisplayName()); // NOI18N
+            throw new FSException(NbBundle.getMessage(MultiFileSystem.class, "EXC_FSisRO", getDisplayName()));
         }
 
         return systems[WRITE_SYSTEM_INDEX];
@@ -471,7 +472,7 @@ public class MultiFileSystem extends FileSystem {
     * @return set of filesystems
     * @exception IOException if the resource cannot be locked
     */
-    protected Set createLocksOn(String name) throws IOException {
+    protected Set<? extends FileSystem> createLocksOn(String name) throws IOException {
         FileSystem writable = createWritableOn(name);
 
         return Collections.singleton(writable);
