@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -637,7 +637,12 @@ public final class XMLFileSystem extends AbstractFileSystem {
                 names.add(name);
                 children.add(child);
             } else {
+                // already exists
                 retVal = children.get(idx);
+                Set<URL> mergedContext = new HashSet<URL>();
+                mergedContext.addAll(Arrays.asList(retVal.getUrlContext()));
+                mergedContext.addAll(Arrays.asList(child.getUrlContext()));
+                retVal.setUrlContext(mergedContext);
             }
 
             return retVal;
@@ -670,6 +675,13 @@ public final class XMLFileSystem extends AbstractFileSystem {
             return retVal;
         }
 
+        void setUrlContext(Collection<? extends URL> context) {
+            if (context != null) {
+                urlContext.clear();
+                urlContext.addAll(context);
+            }
+        }
+
         String getURI() {
             return uri;
         }
@@ -781,7 +793,7 @@ public final class XMLFileSystem extends AbstractFileSystem {
         }
 
         public void lock(String name) throws IOException {
-            FSException.io("EXC_CannotLock", name, fs.getDisplayName(), name); // NOI18N
+            throw new FSException(NbBundle.getMessage(XMLFileSystem.class, "EXC_CannotLock", name, fs.getDisplayName(), name));
         }
 
         public void unlock(String name) {
