@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -241,7 +241,7 @@ final class BrokenDataShadow extends MultiDataObject {
     * @param fo the primary file
     */
     private BrokenDataShadow (FileObject fo) throws DataObjectExistsException {
-        this (fo, (MultiFileLoader)DataLoaderPool.getShadowLoader ());
+        this(fo, DataLoaderPool.getShadowLoader());
     }
     
     /* Getter for delete action.
