@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -45,11 +45,14 @@ package org.openide.text;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.io.BufferedOutputStream;
+import java.io.CharConversionException;
 import java.io.FilterOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.io.InvalidObjectException;
 import java.io.ObjectInputStream;
+import java.io.ObjectInputValidation;
 import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 import java.io.Reader;
@@ -60,11 +63,13 @@ import java.util.Arrays;
 import java.util.Collections;
 import java.util.Date;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Map;
-import java.util.concurrent.atomic.AtomicReference;
+import java.util.Set;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 import javax.swing.text.BadLocationException;
+import javax.swing.text.Document;
 import javax.swing.text.EditorKit;
 import javax.swing.text.StyledDocument;
 import org.netbeans.api.queries.FileEncodingQuery;
@@ -72,13 +77,16 @@ import org.netbeans.modules.openide.loaders.DataObjectAccessor;
 import org.netbeans.modules.openide.loaders.UIException;
 import org.openide.DialogDisplayer;
 import org.openide.NotifyDescriptor;
+import org.openide.cookies.EditorCookie;
 import org.openide.cookies.OpenCookie;
+import org.openide.filesystems.FileAttributeEvent;
 import org.openide.filesystems.FileChangeAdapter;
 import org.openide.filesystems.FileEvent;
 import org.openide.filesystems.FileObject;
 import org.openide.filesystems.FileSystem.AtomicAction;
 import org.openide.filesystems.FileUtil;
 import org.openide.filesystems.FileLock;
+import org.openide.filesystems.FileRenameEvent;
 import org.openide.filesystems.FileSystem;
 import org.openide.loaders.DataFolder;
 import org.openide.loaders.DataObject;
@@ -90,10 +98,12 @@ import org.openide.nodes.NodeAdapter;
 import org.openide.nodes.NodeListener;
 import org.openide.util.Mutex;
 import org.openide.util.NbBundle;
+import org.openide.util.Parameters;
 import org.openide.util.WeakListeners;
 import org.openide.util.lookup.AbstractLookup;
 import org.openide.util.lookup.InstanceContent;
 import org.openide.windows.CloneableOpenSupport;
+import org.openide.xml.XMLUtil;
 
 /**
  * Support for associating an editor and a Swing {@link Document} to a data object.
@@ -107,7 +117,7 @@ public class DataEditorSupport extends CloneableEditorSupport {
     private final DataObject obj;
     /** listener to associated node's events */
     private NodeListener nodeL;
-    
+
     /** Editor support for a given data object. The file is taken from the
     * data object and is updated if the object moves or renames itself.
     * @param obj object to work with
@@ -184,6 +194,48 @@ public class DataEditorSupport extends CloneableEditorSupport {
             obj.getPrimaryFile().getNameExt()
         );
     }
+
+    static boolean TABNAMES_HTML = Boolean.parseBoolean(System.getProperty("nb.tabnames.html", "true")); // #47290
+
+    /**
+     * Marks up a tab name according to modified and read-only status.
+     * Done for subclasses automatically in {@link #messageName} and {@link #messageHtmlName}
+     * but useful for other editor-like windows.
+     * <p class="nonnormative">Behavior currently varies according to the system property {@code nb.tabnames.html}.</p>
+     * @param label incoming label (null not permitted, so take care with {@link Node#getHtmlDisplayName})
+     * @param html if true, {@code label} may include HTML markup (with or without initial {@code <html>}), and result may as well
+     * @param modified mark up the label as for a document which is modified in memory
+     * @param readOnly mark up the label as for a document based on a read-only file
+     * @return a possibly marked-up label
+     * @since org.openide.loaders 7.7
+     */
+    public static String annotateName(String label, boolean html, boolean modified, boolean readOnly) {
+        Parameters.notNull("original", label);
+        if (html && TABNAMES_HTML) {
+            if (label.startsWith("<html>")) {
+                label = label.substring(6);
+            }
+            if (modified) {
+                label = "<b>" + label + "</b>";
+            }
+            if (readOnly) {
+                label = "<i>" + label + "</i>";
+            }
+            return "<html>" + label;
+        } else {
+            if (html && !label.startsWith("<html>")) {
+                label = "<html>" + label;
+            }
+            int version = modified ? (readOnly ? 2 : 1) : (readOnly ? 0 : 3);
+            try {
+                return NbBundle.getMessage(DataObject.class, "LAB_EditorName", version, label);
+            } catch (IllegalArgumentException iae) {
+                String pattern = NbBundle.getMessage(DataObject.class, "LAB_EditorName");
+                ERR.log(Level.WARNING, "#166035: formatting failed. pattern=" + pattern + ", version=" + version + ", name=" + label, iae);  //NOI18N
+                return label;
+            }
+        }
+    }
     
     /** Constructs message that should be used to name the editor component.
     *
@@ -194,7 +246,7 @@ public class DataEditorSupport extends CloneableEditorSupport {
             return ""; // NOI18N
         }
 
-        return addFlagsToName(obj.getNodeDelegate().getDisplayName());
+        return annotateName(obj.getNodeDelegate().getDisplayName(), false, isModified(), !obj.getPrimaryFile().canWrite());
     }
     
     @Override
@@ -204,34 +256,17 @@ public class DataEditorSupport extends CloneableEditorSupport {
         }
 
         String name = obj.getNodeDelegate().getHtmlDisplayName();
-        if (name != null) {
-            if (!name.startsWith("<html>")) {
-                name = "<html>" + name;
-            }
-            name = addFlagsToName(name);
-        }
-        return name;
-    }
-        
-    /** Helper only. */
-    private String addFlagsToName(String name) {
-        int version = 3;
-        if (isModified ()) {
-            if (!obj.getPrimaryFile().canWrite()) {
-                version = 2;
-            } else {
-                version = 1;
-            }
-        } else {
-            if (!obj.getPrimaryFile().canWrite()) {
-                version = 0;
+        if (name == null) {
+            try {
+                name = XMLUtil.toElementContent(obj.getNodeDelegate().getDisplayName());
+            } catch (CharConversionException ex) {
+                return null;
             }
         }
 
-        return NbBundle.getMessage (DataObject.class, "LAB_EditorName",
-		new Integer (version), name );
+        return annotateName(name, true, isModified(), !obj.getPrimaryFile().canWrite());
     }
-    
+        
     @Override
     protected String documentID() {
         if (! obj.isValid()) {
@@ -240,6 +275,29 @@ public class DataEditorSupport extends CloneableEditorSupport {
         return obj.getPrimaryFile().getName();
     }
 
+    /**
+     * Constructs a tool tip possibly marked up with document modified and read-only status.
+     * Done for subclasses automatically in {@link #messageToolTip} but useful for other editor-like windows.
+     * <p class="nonnormative">Behavior currently varies according to the system property {@code nb.tabnames.html}.</p>
+     * @param file a file representing the tab
+     * @param modified mark up the tool tip as for a document which is modified in memory
+     * @param readOnly mark up the tool tip as for a document based on a read-only file
+     * @return a tool tip
+     * @since org.openide.loaders 7.7
+     */
+    public static String toolTip(FileObject file, boolean modified, boolean readOnly) {
+        String tip = FileUtil.getFileDisplayName(file);
+        if (TABNAMES_HTML) {
+            if (modified) {
+                tip += NbBundle.getMessage(DataObject.class, "TIP_editor_modified");
+            }
+            if (readOnly) {
+                tip += NbBundle.getMessage(DataObject.class, "TIP_editor_ro");
+            }
+        }
+        return tip;
+    }
+
     /** Text to use as tooltip for component.
     *
     * @return text to show to the user
@@ -247,7 +305,7 @@ public class DataEditorSupport extends CloneableEditorSupport {
     @Override
     protected String messageToolTip () {
         // update tooltip
-        return FileUtil.getFileDisplayName(obj.getPrimaryFile());
+        return toolTip(obj.getPrimaryFile(), isModified(), !obj.getPrimaryFile().canWrite());
     }
     
     /** Computes display name for a line based on the 
@@ -311,11 +369,11 @@ public class DataEditorSupport extends CloneableEditorSupport {
         StyledDocument doc = super.createStyledDocument (kit);
             
         // set document name property
-        doc.putProperty(javax.swing.text.Document.TitleProperty,
+        doc.putProperty(Document.TitleProperty,
             FileUtil.getFileDisplayName(obj.getPrimaryFile())
         );
         // set dataobject to stream desc property
-        doc.putProperty(javax.swing.text.Document.StreamDescriptionProperty,
+        doc.putProperty(Document.StreamDescriptionProperty,
             obj
         );
         
@@ -363,6 +421,35 @@ public class DataEditorSupport extends CloneableEditorSupport {
      * between saveFromKitToStream and saveDocument
      */
     private static Map<DataObject,Charset> charsets = Collections.synchronizedMap(new HashMap<DataObject,Charset>());
+    /** Holds counter of charsets cache. Cached value mustn't be removed until couter is zero. */
+    private static final Map<DataObject,Integer> cacheCounter = Collections.synchronizedMap(new HashMap<DataObject,Integer>());
+
+    private static int incrementCacheCounter(DataObject tmpObj) {
+        synchronized (cacheCounter) {
+            Integer count = cacheCounter.get(tmpObj);
+            if (count == null) {
+                count = 0;
+            }
+            count++;
+            cacheCounter.put(tmpObj, count);
+            return count;
+        }
+    }
+
+    private static synchronized int decrementCacheCounter(DataObject tmpObj) {
+        synchronized (cacheCounter) {
+            Integer count = cacheCounter.get(tmpObj);
+            assert count != null;
+            count--;
+            if (count == 0) {
+                cacheCounter.remove(tmpObj);
+            } else {
+                cacheCounter.put(tmpObj, count);
+            }
+            return count;
+        }
+    }
+
     /**
      * @inheritDoc
      */
@@ -395,13 +482,20 @@ public class DataEditorSupport extends CloneableEditorSupport {
 
     @Override
     public StyledDocument openDocument() throws IOException {
-        Charset c = FileEncodingQuery.getEncoding(this.getDataObject().getPrimaryFile());
         DataObject tmpObj = getDataObject();
+        Charset c = charsets.get(tmpObj);
+        if (c == null) {
+            c = FileEncodingQuery.getEncoding(tmpObj.getPrimaryFile());
+        }
         try {
             charsets.put(tmpObj, c);
+            incrementCacheCounter(tmpObj);
             return super.openDocument();
         } finally {
-            charsets.remove(tmpObj);
+            if (decrementCacheCounter(tmpObj) == 0) {
+                charsets.remove(tmpObj);
+            }
+            ERR.finest("openDocument - charset removed");
         }
     }
 
@@ -590,15 +684,23 @@ public class DataEditorSupport extends CloneableEditorSupport {
         * Not private for tests.
         */
         transient FileLock fileLock;
+        
         /** did we warned about the size of the file?
          */
-        private transient boolean warned;
+        private transient static Set<FileObject> warnedFiles = new HashSet<FileObject>();
+
+        /** Atomic action used to ignore fileChange event from FileObject.refresh */
+        private transient FileSystem.AtomicAction action = null;
+
+        /** Holds read-only state of associated file object. */
+        private transient boolean canWrite;
 
         /** Constructor.
         * @param obj this support should be associated with
         */
         public Env (DataObject obj) {
             super (obj);
+            canWrite = obj.getPrimaryFile().canWrite();
         }
         
         /** Getter for the file to work on.
@@ -628,7 +730,6 @@ public class DataEditorSupport extends CloneableEditorSupport {
         */
         protected final void changeFile () {
             FileObject newFile = getFile ();
-            
             if (newFile.equals (fileObject)) {
                 // the file has not been updated
                 return;
@@ -679,7 +780,7 @@ public class DataEditorSupport extends CloneableEditorSupport {
         */
         public InputStream inputStream() throws IOException {
             final FileObject fo = getFileImpl ();
-            if (!warned && fo.getSize () > 1024 * 1024) {
+            if (!warnedFiles.contains(fo) && fo.getSize () > 1024 * 1024) {
                 throw new ME (fo.getSize ());
             }
             InputStream is = getFileImpl ().getInputStream ();
@@ -712,7 +813,17 @@ public class DataEditorSupport extends CloneableEditorSupport {
         */
         public Date getTime() {
             // #32777 - refresh file object and return always the actual time
-            getFileImpl().refresh(false);
+            action = new FileSystem.AtomicAction() {
+                public void run() throws IOException {
+                    getFileImpl().refresh(false);
+                }
+            };
+            try {
+                getFileImpl().getFileSystem().runAtomicAction(action);
+            } catch (IOException ex) {
+                //Nothing to do here
+            }
+            
             return getFileImpl ().lastModified ();
         }
         
@@ -773,13 +884,38 @@ public class DataEditorSupport extends CloneableEditorSupport {
         * @param expected is the change expected
         * @param time of the change
         */
-        final void fileChanged (boolean expected, long time) {
-            ERR.fine("fileChanged: " + expected + " for " + fileObject); // NOI18N
-            if (expected) {
+        final void fileChanged (FileEvent fe) {
+            ERR.fine("fileChanged: " + fe.isExpected() + " for " + fileObject); // NOI18N
+            //#155680: We will ignore events generated from FileObject.refresh() in getTime().
+            if ((action != null) && fe.firedFrom(action)) {
+                return;
+            }
+            if (fe.isExpected()) {
                 // newValue = null means do not ask user whether to reload
                 firePropertyChange (PROP_TIME, null, null);
             } else {
-                firePropertyChange (PROP_TIME, null, new Date (time));
+                firePropertyChange (PROP_TIME, null, new Date (fe.getTime()));
+            }
+        }
+
+        /** Called from EnvListener if read-only state is externally changed (#129178).
+         * @param readOnly true if changed to read-only state, false if changed to read-write
+         */
+        private void readOnlyRefresh() {
+            boolean oldCanWrite = canWrite;
+            canWrite = getFileImpl().canWrite();
+            if (oldCanWrite != canWrite) {
+                if (!canWrite && isModified()) {
+                    // notify user if the object is modified and externally changed to read-only
+                    DialogDisplayer.getDefault().notify(
+                            new NotifyDescriptor.Message(
+                            NbBundle.getMessage(DataObject.class,
+                            "MSG_FileReadOnlyChanging",
+                            new Object[]{getFileImpl().getNameExt()}),
+                            NotifyDescriptor.WARNING_MESSAGE));
+                }
+                // event is consumed in CloneableEditorSupport
+                firePropertyChange("DataEditorSupport.read-only.changing", null, null);  //NOI18N
             }
         }
 
@@ -804,22 +940,36 @@ public class DataEditorSupport extends CloneableEditorSupport {
             firePropertyChange(Env.PROP_VALID, Boolean.TRUE, Boolean.FALSE);            
              */
         }
-        
-        @Override
-        public CloneableOpenSupport findCloneableOpenSupport() {
-            CloneableOpenSupport cos = super.findCloneableOpenSupport ();
-            if (cos instanceof DataEditorSupport) {
-                Object o = ((DataEditorSupport)cos).env;
-                if (o != this && o instanceof Env) {
-                   ((Env)o).warned = this.warned;
+
+        /**
+         * Called from the <code>EnvListener</code>.
+         */
+        final void updateDocumentProperty () {
+            //Update document TitleProperty
+            EditorCookie ec = getDataObject().getCookie(EditorCookie.class);
+            if (ec != null) {
+                StyledDocument doc = ec.getDocument();
+                if (doc != null) {
+                    doc.putProperty(Document.TitleProperty,
+                    FileUtil.getFileDisplayName(getDataObject().getPrimaryFile()));
                 }
             }
-            return cos;
+        }
+        
+        /** Called from the <code>EnvListener</code>.
+         */
+        final void fileRenamed () {
+            //#151787: Sync timestamp when svn client changes timestamp externally during rename.
+            firePropertyChange("expectedTime", null, getTime()); // NOI18N
         }
         
         private void readObject (ObjectInputStream ois) throws ClassNotFoundException, IOException {
             ois.defaultReadObject ();
-            warned = true;
+            ois.registerValidation(new ObjectInputValidation() {
+                public void validateObject() throws InvalidObjectException {
+                    warnedFiles.add(getFileImpl());
+                }
+            }, 0);
         }
         
         private class SaveAsCapableImpl implements SaveAsCapable {
@@ -853,9 +1003,9 @@ public class DataEditorSupport extends CloneableEditorSupport {
                 };
                 return NbBundle.getMessage(DataObject.class, "MSG_ObjectIsTooBig", arr);
             }
-
+            
             public void confirmed () {
-                warned = true;
+                warnedFiles.add(getFileImpl());
             }
         } // end of ME
     } // end of Env
@@ -879,6 +1029,9 @@ public class DataEditorSupport extends CloneableEditorSupport {
         public void fileDeleted(FileEvent fe) {
             Env myEnv = this.env.get();
             FileObject fo = fe.getFile();
+            if (myEnv != null) {
+                myEnv.updateDocumentProperty();
+            }
             if(myEnv == null || myEnv.getFileImpl() != fo) {
                 // the Env change its file and we are not used
                 // listener anymore => remove itself from the list of listeners
@@ -918,10 +1071,29 @@ public class DataEditorSupport extends CloneableEditorSupport {
                 myEnv.fileRemoved(true);
                 fe.getFile().addFileChangeListener(this);
             } else {
-                myEnv.fileChanged (fe.isExpected (), fe.getTime ());
+                myEnv.fileChanged (fe);
+            }
+        }
+        
+        @Override
+        public void fileRenamed(FileRenameEvent fe) {
+            Env myEnv = this.env.get();
+            if (myEnv != null) {
+                myEnv.updateDocumentProperty();
+                myEnv.fileRenamed();
+            }
+        }
+        
+        @Override
+        public void fileAttributeChanged(FileAttributeEvent fae) {
+            // wait only for event from org.netbeans.modules.masterfs.filebasedfs.fileobjects.FileObj.refreshImpl()
+            if ("DataEditorSupport.read-only.refresh".equals(fae.getName())) {  //NOI18N
+                Env myEnv = this.env.get();
+                if (myEnv != null) {
+                    myEnv.readOnlyRefresh();
+                }
             }
         }
-                
     }
     
     /** Listener on node representing associated data object, listens to the
@@ -1003,12 +1175,19 @@ public class DataEditorSupport extends CloneableEditorSupport {
                 throw e;
             }
             DataObject tmpObj = des.getDataObject();
-            Charset c = FileEncodingQuery.getEncoding(tmpObj.getPrimaryFile());
+            Charset c = charsets.get(tmpObj);
+            if (c == null) {
+                c = FileEncodingQuery.getEncoding(tmpObj.getPrimaryFile());
+            }
             try {
                 charsets.put(tmpObj, c);
+                incrementCacheCounter(tmpObj);
+                ERR.finest("SaveImpl - charset put");
                 des.superSaveDoc();
             } finally {
-                charsets.remove(tmpObj);
+                if (decrementCacheCounter(tmpObj) == 0) {
+                    charsets.remove(tmpObj);
+                }
             }
         }
 
