@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -68,7 +68,7 @@ public class CanYouCreateFolderLookupFromHandleFindTest extends NbTestCase {
     }
     
     public void testCreateAndImmediatelyQueryTheLookup() throws Exception {
-        MyLoader m = (MyLoader)MyLoader.getLoader(MyLoader.class);
+        MyLoader m = MyLoader.getLoader(MyLoader.class);
         m.button = FileUtil.createFolder(FileUtil.getConfigRoot(), "FolderLookup");
         DataObject instance = InstanceDataObject.create(DataFolder.findFolder(m.button), "SomeName", JButton.class);
         m.instanceFile = instance.getPrimaryFile();
