@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -83,6 +83,11 @@ public final class DefaultDataObjectHasOpenTest extends NbTestCase {
         super(name);
     }
 
+    @Override
+    protected int timeOut() {
+        return 10000;
+    }
+
     @Override
     protected void setUp() throws Exception {
         super.setUp();
@@ -154,7 +159,7 @@ public final class DefaultDataObjectHasOpenTest extends NbTestCase {
         doRegularCheck(c);
     }
 
-    private void doRegularCheck(EditorCookie c) throws Exception {
+    private void doRegularCheck(final EditorCookie c) throws Exception {
         assertEquals(
                 "Next questions results in the same cookie",
                 c,
@@ -179,7 +184,15 @@ public final class DefaultDataObjectHasOpenTest extends NbTestCase {
         OpenCookie open = obj.getCookie(OpenCookie.class);
         open.open();
 
-        Document d = c.getDocument();
+
+        Document d = null;
+        for (int i = 0; i < 10; i++) {
+            d = c.getDocument();
+            if (d != null) {
+                break;
+            }
+            Thread.sleep(1000);
+        }
         assertNotNull(d);
 
         d.insertString(0, "Kuk", null);
