@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -57,6 +57,7 @@ import java.util.Set;
 import java.util.TreeSet;
 import java.util.concurrent.Executor;
 import java.util.concurrent.atomic.AtomicReference;
+import java.util.logging.Level;
 import java.util.logging.Logger;
 import org.openide.util.Enumerations;
 import org.openide.util.Mutex;
@@ -1834,9 +1835,11 @@ public abstract class Children extends Object {
         }
 
         private Mutex callPMMutexMethod() {
+            Class<?> clazz = null;
+            Method method = null;
             try {
-                Class<?> clazz = Thread.currentThread().getContextClassLoader().loadClass("org.netbeans.api.project.ProjectManager"); // NOI18N
-                Method method = clazz.getMethod("mutex"); // NOI18N
+                clazz = Thread.currentThread().getContextClassLoader().loadClass("org.netbeans.api.project.ProjectManager"); // NOI18N
+                method = clazz.getMethod("mutex"); // NOI18N
                 return (Mutex) method.invoke(null);
             } catch (ClassNotFoundException e) {
                 return null;
@@ -1848,6 +1851,11 @@ public abstract class Children extends Object {
                 return null;
             } catch (NoSuchMethodException e) {
                 return null;
+            } catch (ClassCastException e) { // observed to occur in MemoryValidationTest
+                Class<?> type = method.getReturnType();
+                LOG.log(Level.WARNING, "Reopen #175325 and save complete log: type=" + type.getName() + " type.cl=" + type.getClassLoader() +
+                        " Mutex.cl=" + Mutex.class.getClassLoader() + " clazz.cl=" + clazz.getClassLoader(), e);
+                return null;
             }
         }
     }
