@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -109,17 +109,8 @@ final class ChildrenArray extends NodeAdapter {
         }
     }
 
-    /** Finalizes nodes by calling get on weak hash map,
-     * all references stored in the map, that are finalized
-     * will be cleared.
-     */
-    public synchronized void finalizeNodes() {
-        Map m = map;
-        if (m != null) {
-            // processes the queue of garbage
-            // collected keys
-            m.remove(null);
-        }
+    void remove(Info info) {
+        map.remove(info);
     }
 
     /** Initilized if has some nodes. */
