@@ -86,6 +86,9 @@ abstract class EntrySupport {
      * does not exists it should return null.*/
     public abstract Node getNodeAt(int index);
 
+    /**
+     * @return currently created nodes or null if no node is created
+     */
     public abstract Node[] testNodes();
 
     public abstract boolean isInitialized();
@@ -131,7 +134,8 @@ abstract class EntrySupport {
         }
 
         public boolean isInitialized() {
-            return inited;
+            ChildrenArray arr = array.get();
+            return inited && arr != null && arr.isInitialized();
         }
 
         @Override
@@ -169,7 +173,7 @@ abstract class EntrySupport {
                         // support was switched while we were waiting for access
                         return new Node[0];
                     }
-                    results[1] = inited;
+                    results[1] = isInitialized();
                     nodes = tmpArray.nodes();
                 } finally {
                     Children.PR.exitReadAccess();
@@ -360,10 +364,12 @@ abstract class EntrySupport {
         private void updateRemove(Node[] current, Set<Entry> toRemove) {
             List<Node> nodes = new LinkedList<Node>();
 
+            ChildrenArray cha = array.get();
             for (Entry en : toRemove) {
                 Info info = map.remove(en);
                 checkInfo(info, en, null, map);
                 nodes.addAll(info.nodes(true));
+                cha.remove(info);
             }
 
             // modify the current set of entries
@@ -686,15 +692,21 @@ abstract class EntrySupport {
                 n.fireSubNodesChange(true, arr, null);
             }
         }
-        //
-        // ChildrenArray operations call only under lock
-        //
-        /** @return either nodes associated with this children or null if
-         * they are not created
+
+        /**
+         * @return either nodes associated with this children or null if they are not created
          */
         public Node[] testNodes() {
             ChildrenArray arr = array.get();
-            return (arr == null) ? null : arr.nodes();
+            if (arr == null) {
+                return null;
+            }
+            try {
+                Children.PR.enterReadAccess();
+                return arr.nodes();
+            } finally {
+                Children.PR.exitReadAccess();
+            }
         }
 
         /** Obtains references to array holder. If it does not exist, it is created.
@@ -795,6 +807,7 @@ abstract class EntrySupport {
                     if (cannotWorkBetter != null) {
                         cannotWorkBetter[0] = true;
                     }
+                    arr.entrySupport = this;
                     return arr;
                 }
 
@@ -884,16 +897,6 @@ abstract class EntrySupport {
                 Children.PR.exitWriteAccess();
             }
         }
-        /** Forces finalization of nodes for given info.
-         * Called from finalizer of Info.
-         */
-        final void finalizeNodes() {
-            ChildrenArray arr = array.get();
-
-            if (arr != null) {
-                arr.finalizeNodes();
-            }
-        }
 
         /** Information about an entry. Contains number of nodes,
          * position in the array of nodes, etc.
@@ -907,13 +910,6 @@ abstract class EntrySupport {
                 this.entry = entry;
             }
 
-            /** Finalizes the content of ChildrenArray.
-             */
-            @Override
-            protected void finalize() {
-                finalizeNodes();
-            }
-
             public Collection<Node> nodes(boolean hasToExist) {
                 // forces creation of the array
                 assert !hasToExist || array.get() != null : "ChildrenArray is not initialized";
@@ -1181,7 +1177,6 @@ abstract class EntrySupport {
                         toReturn.add(node);
                     }
                     tmpNodes = toReturn.toArray(new Node[0]);
-                    nodesCreated = true;
                     if (invalidEntries == null) {
                         return tmpNodes;
                     }
@@ -1196,10 +1191,25 @@ abstract class EntrySupport {
             }
         }
 
-        boolean nodesCreated = false;
         @Override
         public Node[] testNodes() {
-            return nodesCreated ? getNodes(false) : null;
+            if (!inited) {
+                return null;
+            }
+            List<Node> created = new ArrayList<Node>();
+            try {
+                Children.PR.enterReadAccess();
+                for (Entry entry : visibleEntries) {
+                    EntryInfo info = entryToInfo.get(entry);
+                    Node node = info.currentNode();
+                    if (node != null) {
+                        created.add(node);
+                    }
+                }
+            } finally {
+                Children.PR.exitReadAccess();
+            }
+            return created.isEmpty() ? null : created.toArray(new Node[created.size()]);
         }
 
         @Override
