@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -41,21 +41,16 @@
 
 package org.openide.nodes;
 
-import com.sun.org.apache.bcel.internal.generic.ARRAYLENGTH;
-import junit.framework.*;
-import junit.textui.TestRunner;
-import java.util.*;
 import org.openide.cookies.EditCookie;
 import org.openide.cookies.OpenCookie;
 import org.openide.cookies.SaveCookie;
-import org.openide.nodes.*;
 
-import org.netbeans.junit.*;
 import javax.swing.event.ChangeListener;
+import org.netbeans.junit.NbTestCase;
+import org.openide.nodes.Node.Cookie;
 import org.openide.util.Lookup;
 import org.openide.util.LookupEvent;
 import org.openide.util.LookupListener;
-import org.openide.util.lookup.InstanceContent;
 
 /** Tests behaviour of CookieSet.
  *
@@ -66,6 +61,68 @@ public class CookieSetTest extends NbTestCase {
         super(name);
     }
 
+    public void testInconsistencyOfLookupAndGetCookie175750() {
+        CookieSet cs = CookieSet.createGeneric(null);
+
+
+        class Factory implements CookieSet.Factory {
+            public <T extends Cookie> T createCookie(Class<T> klass) {
+                if (klass == Cook.class) {
+                    return klass.cast(new Cook() { });
+                }
+                return null;
+            }
+        }
+        final Factory f = new Factory();
+        cs.add(Cook.class, f);
+
+        Cook one = cs.getCookie(Cook.class);
+        Cook two = cs.getCookie(Cook.class);
+        Cook three = cs.getLookup().lookup(Cook.class);
+
+        assertSame("One and two", one, two);
+        assertSame("3 and two", three, two);
+
+        cs.remove(new Class[] { Cook.class }, f);
+
+        assertNull("No cookie now ", cs.getCookie(Cook.class));
+        assertNull("No lkp now ", cs.getLookup().lookup(Cook.class));
+    }
+    static interface Cook extends Node.Cookie {
+    }
+    public void testInconsistencyWhenDoubleCookie175750() {
+        CookieSet cs = CookieSet.createGeneric(null);
+
+
+        class Factory implements CookieSet.Factory {
+            public <T extends Cookie> T createCookie(Class<T> klass) {
+                if (klass == Cook.class || klass == Hook.class) {
+                    return klass.cast(new Hook() { });
+                }
+                return null;
+            }
+        }
+        final Factory f = new Factory();
+        cs.add(new Class[] { Cook.class, Hook.class }, f);
+
+        Cook one = cs.getCookie(Hook.class);
+        Cook two = cs.getCookie(Cook.class);
+        Cook three = cs.getLookup().lookup(Hook.class);
+        Cook four = cs.getLookup().lookup(Cook.class);
+
+        assertSame("One and three", one, three);
+        assertSame("Two and 4", two, four);
+
+        cs.remove(new Class[] { Hook.class, Cook.class }, f);
+
+        assertNull("No cookie now ", cs.getCookie(Cook.class));
+        assertNull("No lkp now ", cs.getLookup().lookup(Cook.class));
+        assertNull("No h cookie now ", cs.getCookie(Hook.class));
+        assertNull("No h lkp now ", cs.getLookup().lookup(Hook.class));
+    }
+    static interface Hook extends Cook {
+    }
+
     
     public void testAddRemove () throws Exception {
         CookieSet set = new CookieSet ();
