@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -41,18 +41,27 @@
 
 package org.openide.text;
 
-import java.awt.*;
+import java.awt.BorderLayout;
+import java.awt.Component;
+import java.awt.Dimension;
+import java.awt.Insets;
+import java.awt.Rectangle;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.io.*;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.List;
 import java.util.concurrent.Callable;
 import java.util.logging.Level;
-import java.util.logging.LogRecord;
 import java.util.logging.Logger;
 import javax.swing.*;
 import javax.swing.border.Border;
 import javax.swing.border.EmptyBorder;
 import javax.swing.text.*;
+import org.netbeans.modules.openide.text.Installer;
+import org.openide.DialogDisplayer;
+import org.openide.NotifyDescriptor;
 import org.openide.awt.UndoRedo;
 import org.openide.cookies.EditorCookie;
 import org.openide.util.*;
@@ -79,14 +88,34 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
     /** Flag indicating it was initialized this <code>CloneableEditor</code> */
     private boolean initialized;
 
+    /** Flag indicating progress of DoInitialize tasks */
+    private boolean initVisualFinished;
+    
+    /** Flag indicating if AWT is waiting in getEditorPane on initVisual.
+     It is static for now when we have only one worker thread/RP */
+    private static boolean waitingOnInitVisual;
+
+    /** Flag indicating if modal dialog for handling UQE is displayed.
+     If it is yes we cannot handle call of getEditorPane from modal EQ because it results in deadlock. */
+    private static boolean isModalDialog;
+    
+    /** Flag to detect if document loading in initNonVisual was canceled by user */
+    private boolean isDocLoadingCanceled = false;
+
+    /** Flag to detect if component is opened or closed to control return value
+     * of getEditorPane. If component creation starts this flag is set to true and
+     * getEditorPane waits till initialization finishes */
+    private boolean isComponentOpened = false;
+    
     /** Position of cursor. Used to keep the value between deserialization
      * and initialization time. */
     private int cursorPosition = -1;
-
-    private StringBuffer sb = new StringBuffer(1024);
-
+    
     private final Object CLOSE_LAST_LOCK = new Object();
 
+    private static List<AWTQuery> tbdList = Collections.synchronizedList(new ArrayList<AWTQuery>());
+
+    private static List<AWTQuery> finishedList = Collections.synchronizedList(new ArrayList<AWTQuery>());
     // #20647. More important custom component.
 
     /** Custom editor component, which is used if specified by document
@@ -192,6 +221,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
 
         this.pane = tmp;
         this.initialized = true;
+        this.isComponentOpened = true;
         
         this.doInitialize = new DoInitialize(tmp);
     }
@@ -205,7 +235,35 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
         }
         return !Boolean.TRUE.equals(getClientProperty("oldInitialize")); // NOI18N
     }
+    
+    class AWTQuery implements Runnable {
+        private UserQuestionException ex;
+        private boolean isConfirmed ;
 
+        public AWTQuery (UserQuestionException ex) {
+            this.ex = ex;
+        }
+
+        public boolean confirmed () {
+            return isConfirmed;
+        }
+        
+        public void run() {
+            NotifyDescriptor nd = new NotifyDescriptor.Confirmation(
+                    ex.getLocalizedMessage(), NotifyDescriptor.YES_NO_OPTION
+                );
+            nd.setOptions(new Object[] { NotifyDescriptor.YES_OPTION, NotifyDescriptor.NO_OPTION });
+            isModalDialog = true;
+            Object res = DialogDisplayer.getDefault().notify(nd);
+            isModalDialog = false;
+            if (NotifyDescriptor.OK_OPTION.equals(res)) {
+                isConfirmed = true;
+            } else {
+                isConfirmed = false;
+            }
+        }
+    }
+    
     class DoInitialize implements Runnable, ActionListener {
         private final QuietEditorPane tmp;
         private Document doc;
@@ -221,6 +279,8 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
         /** Flag to avoid recursive call of initVisual. */
         private boolean isInInitVisual = false;
 
+        private boolean confirmed = false;
+
         public DoInitialize(QuietEditorPane tmp) {
             this.tmp = tmp;
             this.tmpComp = initLoading();
@@ -261,7 +321,10 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
             switch (phase++) {
             case 0:
                 synchronized (CLOSE_LAST_LOCK) {
-                    initNonVisual();
+                    phase = initNonVisual(phase);
+                    if (phase == Integer.MAX_VALUE) {
+                        break;
+                    }
                 }
                 if (newInitialize()) {
                     WindowManager.getDefault().invokeWhenUIReady(this);
@@ -306,7 +369,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
             }
         }
             
-        private void initNonVisual() {
+        private int initNonVisual (int phase) {
             if (LOG.isLoggable(Level.FINE)) {
                 LOG.log(Level.FINE,"DoInitialize.initNonVisual Enter"
                 + " Time:" + System.currentTimeMillis()
@@ -318,9 +381,150 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
             Task prepareTask = support.prepareDocument();
             assert prepareTask != null : "Failed to get prepareTask";
             prepareTask.waitFinished();
-            Throwable ex = support.getPrepareDocumentRuntimeException();
-            if (ex instanceof CloneableEditorSupport.DelegateIOExc) {
-                if ("org.openide.text.DataEditorSupport$Env$ME".equals(ex.getCause().getClass().getName())) {
+            final Throwable ex = support.getPrepareDocumentRuntimeException();
+            if (ex == null) {
+                isDocLoadingCanceled = false;
+            }
+            if (support.asynchronousOpen()) {
+                if (ex instanceof CloneableEditorSupport.DelegateIOExc) {
+                    if (ex.getCause() instanceof UserQuestionException) {
+                        class Query implements Runnable {
+                            
+                            private DoInitialize doInit;
+                            boolean inAWT;
+                            boolean finished;
+                            
+                            public Query (DoInitialize doInit) {
+                                this.doInit = doInit;
+                            }
+                            
+                            public void run() {
+                                synchronized (this) {
+                                    inAWT = true;
+                                    notifyAll();
+                                    if (finished) {
+                                        return;
+                                    }
+                                }
+                                try {
+                                    doQuestion();
+                                } finally {
+                                    synchronized (this) {
+                                        finished = true;
+                                        notifyAll();
+                                    }
+                                }
+                            }
+
+                            private void doQuestion() {
+                                UserQuestionException e = (UserQuestionException) ex.getCause();
+                                NotifyDescriptor nd = new NotifyDescriptor.Confirmation(
+                                        e.getLocalizedMessage(), NotifyDescriptor.YES_NO_OPTION
+                                    );
+                                nd.setOptions(new Object[] { NotifyDescriptor.YES_OPTION, NotifyDescriptor.NO_OPTION });
+                                isModalDialog = true;
+                                Object res = DialogDisplayer.getDefault().notify(nd);
+                                isModalDialog = false;
+                                if (NotifyDescriptor.OK_OPTION.equals(res)) {
+                                    doInit.confirmed = true;
+                                } else {
+                                    return;
+                                }
+                            }
+
+                            public synchronized boolean awaitAWT() throws InterruptedException {
+                                if (!inAWT) {
+                                    wait(10000);
+                                }
+                                return inAWT;
+                            }
+
+                            public synchronized void waitRest() throws InterruptedException {
+                                while (inAWT && !finished) {
+                                    wait();
+                                }
+                            }
+                        }
+                        //If AWT is blocked by call of CloneableEditor.getEditorPane ie. by waiting on initVisual
+                        //we cannot unblock it so we will confirm UQE for now.
+                        if (waitingOnInitVisual) {
+                            UserQuestionException e = (UserQuestionException) ex.getCause();
+                            AWTQuery query = new AWTQuery(e);
+                            tbdList.add(query);
+                            synchronized (this) {
+                                //Wake up AWT thread to process queries
+                                notifyAll();
+                                try {
+                                    for (;;) {
+                                        //Wait for processed query
+                                        if (!finishedList.isEmpty()) {
+                                            //Check if our query is already processed
+                                            if (finishedList.remove(query)) {
+                                                break;
+                                            }
+                                        }
+                                        wait();
+                                    }
+                                } catch (InterruptedException exc) {
+                                    Exceptions.printStackTrace(exc);
+                                }
+                            }
+                            if (query.isConfirmed) {
+                                confirmed = true;
+                                try {
+                                    e.confirmed();
+                                } catch (IOException ex1) {
+                                    Exceptions.printStackTrace(ex1);
+                                }
+                            }
+                        } else {
+                            Query query = new Query(this);
+                            try {
+                                SwingUtilities.invokeLater(query);
+                                if (query.awaitAWT()) {
+                                    query.waitRest();
+                                } else {
+                                    //#175956: If AWT is blocked so user could not answer UQE.
+                                    //In such case open document anyway.
+                                    confirmed = true;
+                                }
+                                synchronized (query) {
+                                    query.finished = true;
+                                }
+                            } catch (InterruptedException exc) {
+                                Exceptions.printStackTrace(exc);
+                            }
+                        }
+                        if (confirmed) {
+                            isDocLoadingCanceled = false;
+                            UserQuestionException e = (UserQuestionException) ex.getCause();
+                            try {
+                                e.confirmed();
+                            } catch (IOException ex1) {
+                                Exceptions.printStackTrace(ex1);
+                            }
+                            prepareTask = support.prepareDocument();
+                            assert prepareTask != null : "Failed to get prepareTask";
+                            prepareTask.waitFinished();
+                        } else {
+                            isDocLoadingCanceled = true;
+                            synchronized (this) {
+                                //Wake up AWT thread to cancel initVisual
+                                notifyAll();
+                            }
+                            //Cancel initialization sequence and close editor
+                            isComponentOpened = false;
+                            SwingUtilities.invokeLater(new Runnable () {
+                                public void run () {
+                                    CloneableEditor.this.close();
+                                }
+                            });
+                            return Integer.MAX_VALUE;
+                        }
+                    }
+                }
+            } else {
+                if (ex instanceof CloneableEditorSupport.DelegateIOExc) {
                     if (ex.getCause() instanceof UserQuestionException) {
                         UserQuestionException e = (UserQuestionException) ex.getCause();
                         try {
@@ -348,7 +552,6 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
             paneMap.put("delete", getAction(DefaultEditorKit.deleteNextCharAction)); // NOI18N
             paneMap.put(DefaultEditorKit.pasteAction, getAction(DefaultEditorKit.pasteAction));
 
-
             EditorKit k = support.cesKit();
             if (newInitialize() && k instanceof Callable) {
                 try {
@@ -359,18 +562,11 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
             }
             
             synchronized (this) {
-                try {
-                    doc = support.openDocument();
-                } catch (IOException exc) {
-                    LOG.log(Level.INFO,"Failed to open document",exc);
-                }
+                //Last fix of #138686 makes sure doc is not gc'ed till we keep reference to prepareDocument task
+                //so we can use support.getDocument instead of support.openDocument
+                doc = support.getDocument();
+                assert doc != null;
                 kit = k;
-                sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
-                + " -- CloneableEditor.DoInitialize.initNonVisual"
-                + " name:" + getName()
-                + " SET kit:" + kit
-                + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
-                + " Thread:[" + Thread.currentThread().getName() + "]");
                 initialized = true;
                 if (LOG.isLoggable(Level.FINE)) {
                     LOG.log(Level.FINE,"DoInitialize.initNonVisual doc and kit are set"
@@ -390,45 +586,20 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
                 }
                 notifyAll();
             }
+            return phase;
         }
         
         private void initCustomEditor() {
-            sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
-            + " -- CloneableEditor.DoInitialize.initCustomEditor"
-            + " name:" + getName()
-            + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
-            + " Thread:[" + Thread.currentThread().getName() + "]" + " ENTER");
             if (doc instanceof NbDocument.CustomEditor) {
                 NbDocument.CustomEditor ce = (NbDocument.CustomEditor) doc;
-                sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
-                + " -- CloneableEditor.DoInitialize.initCustomEditor"
-                + " tmp:" + tmp.getClass().getName() + "@" + Integer.toHexString(System.identityHashCode(tmp)));
-                sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
-                + " -- CloneableEditor.DoInitialize.initCustomEditor"
-                + " tmp.ui:" + tmp.getUI().getClass().getName());
-                try {
-                    customComponent = ce.createEditor(tmp);
-                } catch (NullPointerException ex) {
-                    Logger logger = Logger.getLogger("org.netbeans.ui.text");   // NOI18N
-                    LogRecord rec = new LogRecord(Level.INFO, "LOG_CLONEABLE_EDITOR");   // NOI18N
-                    rec.setLoggerName(logger.getName());
-                    String log;
-                    if (tmp instanceof QuietEditorPane) {
-                        log = ((QuietEditorPane) tmp).getLog();
-                    } else {
-                        log = "Pane is not QuietEditorPane";
-                    }
-                    rec.setParameters(new Object [] {"1.0", sb.toString(), log, support.getLog()});
-                    logger.log(rec);
-                    throw new IllegalStateException("Unexpected UI class:" + tmp.getUI().getClass().getName(), ex);
-                }
+                customComponent = ce.createEditor(tmp);
                 
                 if (customComponent == null) {
                     throw new IllegalStateException(
                         "Document:" + doc // NOI18N
-                         +" implementing NbDocument.CustomEditor may not" // NOI18N
-                         +" return null component"
-                    ); // NOI18N
+                         + " implementing NbDocument.CustomEditor may not" // NOI18N
+                         + " return null component" // NOI18N
+                    );
                 }
             }            
         }
@@ -453,11 +624,24 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
             Document d;
             synchronized (this) {
                 for (;;) {
+                    if (!tbdList.isEmpty()) {
+                        while (!tbdList.isEmpty()) {
+                            AWTQuery query = tbdList.remove(0);
+                            query.run();
+                            finishedList.add(query);
+                        }
+                        notifyAll();
+                    }
                     d = doc;
                     k = kit;
                     if (initialized) {
                         break;
                     }
+                    if (isDocLoadingCanceled) {
+                        //Reset pane here so getEditorPane will return null
+                        pane = null;
+                        return false;
+                    }
                     try {
                         if (LOG.isLoggable(Level.FINE)) {
                             LOG.log(Level.FINE,"DoInitialize.initDocument Starting wait"
@@ -467,8 +651,8 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
                             + " this:[" + Integer.toHexString(System.identityHashCode(this)) + "]"
                             + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
                             + " Name:" + CloneableEditor.this.getName()
-                            + " doc:" + doc
-                            + " kit:" + kit);
+                            + " doc:" + d
+                            + " kit:" + k);
                         }
                         wait();
                         if (LOG.isLoggable(Level.FINE)) {
@@ -478,37 +662,30 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
                             + " this:[" + Integer.toHexString(System.identityHashCode(this)) + "]"
                             + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
                             + " Name:" + CloneableEditor.this.getName()
-                            + " doc:" + doc
-                            + " kit:" + kit);
+                            + " doc:" + d
+                            + " kit:" + k);
                         }
                     } catch (InterruptedException ex) {
                         Exceptions.printStackTrace(ex);
                     }
                 }
             }
-            if (tmp.getDocument() == doc) {
+            if (tmp.getDocument() == d) {
                 return false;
             }
-            sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
-            + " -- CloneableEditor.DoInitialize.initDocument INVOKE tmp.setEditorKit(kit)"
-            + " kit:" + kit
-            + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
-            + " Thread:[" + Thread.currentThread().getName() + "]");
-            tmp.setEditorKit(kit);
+            tmp.setEditorKit(k);
             // #132669, do not fire prior setting the kit, which by itself sets a bogus document, etc.
             // if this is a problem please revert the change and initialize QuietEditorPane.working = FIRE
             // and reopen #132669
             tmp.setWorking(QuietEditorPane.FIRE);
-            tmp.setDocument(doc);
-            sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
-            + " -- CloneableEditor.DoInitialize.initDocument"
-            + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
-            + " RETURN TRUE"
-            + " Thread:[" + Thread.currentThread().getName() + "]");
+            tmp.setDocument(d);
             return true;
         }
         
         final void initVisual() {
+            if (isDocLoadingCanceled) {
+                return;
+            }
             //Do not allow recursive call
             if (isInInitVisual) {
                 return;
@@ -519,12 +696,6 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
                 isInInitVisual = false;
                 return;
             }
-            sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
-            + " -- CloneableEditor.DoInitialize.initVisual"
-            + " name:" + getName()
-            + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
-            + " ENTER"
-            + " Thread:[" + Thread.currentThread().getName() + "]");
             if (LOG.isLoggable(Level.FINE)) {
                 /*Exception ex = new Exception();
                 StringWriter sw = new StringWriter(500);
@@ -577,15 +748,20 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
             if (CloneableEditor.this.equals(getRegistry().getActivated())) {
                 requestFocusInWindow();
             }
-            //#162961: Force repaint of editor on Mac. Undocked editor stays empty otherwise sometimes.
-            if (org.openide.util.Utilities.isMac()) {
-                SwingUtilities.invokeLater(new Runnable () {
-                    public void run () {
-                        revalidate();
-                    }
-                });
-            }
+            //#162961, #167289: Force repaint of editor. Sometimes editor stays empty.
+            SwingUtilities.invokeLater(new Runnable () {
+                public void run () {
+                    revalidate();
+                }
+            });
             isInInitVisual = false;
+            initVisualFinished = true;
+
+            //#168415: Notify clients that pane creation is finished.
+            CloneableEditorSupport ces = cloneableEditorSupport();
+            if (ces != null) {
+                ces.firePropertyChange(EditorCookie.Observable.PROP_OPENED_PANES, null, null);
+            }
         }
         
         private void initRest() {
@@ -598,6 +774,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
             });
         }
     } // end of DoInitialize
+    
     @Override
     protected CloneableTopComponent createClonedObject() {
         return support.createCloneableTopComponent();
@@ -611,14 +788,19 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
     @Override
     protected void componentOpened() {
         super.componentOpened();
-
+        
         CloneableEditorSupport ces = cloneableEditorSupport();
-
+        
         if (ces != null) {
             ces.firePropertyChange(EditorCookie.Observable.PROP_OPENED_PANES, null, null);
+            Document d = ces.getDocument();
+            if (d != null) {
+                String mimeType = (String) d.getProperty("mimeType"); //NOI18N
+                Installer.add(mimeType);
+            }
         }
     }
-
+    
     /** Descendants overriding this method must either call
      * this implementation or fire the
      * {@link org.openide.cookies.EditorCookie.Observable#PROP_OPENED_PANES}
@@ -626,38 +808,33 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
      */
     @Override
     protected void componentClosed() {
-        SwingUtilities.invokeLater(
-            new Runnable() {
-                public void run() {
-                    sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
-                    + " -- CloneableEditor.componentClosed.run ENTER"
-                    + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
-                    + " Thread:[" + Thread.currentThread().getName() + "]");
-                    // #23486: pane could not be initialized yet.
-                    if (pane != null) {
-                        // #114608 - commenting out setting of the empty document
+        // #23486: pane could not be initialized yet.
+        if (pane != null) {
+            // #114608 - commenting out setting of the empty document
 //                        Document doc = support.createStyledDocument(pane.getEditorKit());
 //                        pane.setDocument(doc);
-                        
-                        // #138611 - this calls kit.deinstall, which is what our kits expect,
-                        // calling it with null does not impact performance, because the pane
-                        // will not create new document and typically nobody listens on "editorKit" prop change
-                        sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
-                        + " -- CloneableEditor.componentClosed.run INVOKE pane.setEditorKit(null)"
-                        + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
-                        + " Thread:[" + Thread.currentThread().getName() + "]");
-                        pane.setEditorKit(null);
-                    }
 
-                    removeAll();
-                    customComponent = null;
-                    customToolbar = null;
-                    pane = null;
-                    initialized = false;
-                }
+            // #138611 - this calls kit.deinstall, which is what our kits expect,
+            // calling it with null does not impact performance, because the pane
+            // will not create new document and typically nobody listens on "editorKit" prop change
+            pane.setEditorKit(null);
+        }
+
+        // #175576 - EditorRegistry listens on TopComponent.getRegistry() and iterates through
+        // children of closed TCs to registered JEditorPanes and close them as well. Calling
+        // removeAll() here directly means that ER will have no children to search through.
+        SwingUtilities.invokeLater(new Runnable() {
+            public void run() {
+                removeAll();
             }
-        );
-
+        });
+        customComponent = null;
+        customToolbar = null;
+        pane = null;
+        initialized = false;
+        initVisualFinished = false;
+        isComponentOpened = false;
+        
         super.componentClosed();
 
         CloneableEditorSupport ces = cloneableEditorSupport();
@@ -806,15 +983,22 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
 
         return null;
     }
-
-    /** Overrides superclass method. Remembers last selected component of
+    
+    /**
+     * Overrides superclass method. Remembers last selected component of
      * support belonging to this component.
-     * @see #componentDeactivated */
+     *
+     * Descendants overriding this method must call this implementation to set last
+     * selected pane otherwise <code>CloneableEditorSupport.getRecentPane</code> and
+     * <code>CloneableEditorSupport.getOpenedPanes</code> will be broken.
+     *
+     * @see #componentDeactivated
+     */
     @Override
     protected void componentActivated() {
         support.setLastSelected(this);
     }
-
+    
     /** Updates the name and tooltip of this <code>CloneableEditor</code>
      * {@link org.openide.windows.TopComponent TopCompoenent}
      * according to the support retrieved from {@link #cloneableEditorSupport}
@@ -933,6 +1117,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
         }
 
         updateName();
+        isComponentOpened = true;
     }
 
     /**
@@ -982,12 +1167,50 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
         return this;
     }
 
+    /**
+     * #168415: Returns true if creation of editor pane is finished. It is used
+     * to avoid blocking AWT thread by call of getEditorPane.
+     */
+    boolean isEditorPaneReady () {
+        assert SwingUtilities.isEventDispatchThread();
+        return initVisualFinished;
+    }
+
+    /** Used from test only. Can be called out of AWT */
+    boolean isEditorPaneReadyTest () {
+        return initVisualFinished;
+    }
+
+    /** Returns editor pane. Returns null if document loading was canceled by user
+     * through answering UserQuestionException or when CloneableEditor is closed.
+     *
+     * @return editor pane or null
+     */
     public JEditorPane getEditorPane() {
         assert SwingUtilities.isEventDispatchThread();
+        //User selected not to load document
+        if (isDocLoadingCanceled || !isComponentOpened) {
+            return null;
+        }
+        //#175528: This case should not happen as modal dialog handling UQE should
+        //not be displayed during IDE start ie. during component deserialization.
+        if (isModalDialog) {
+            LOG.log(Level.WARNING,"AWT is blocked by modal dialog. Return null from CloneableEditor.getEditorPane."
+            + " Please report this to IZ.");
+            LOG.log(Level.WARNING,"support:" + support.getClass().getName());
+            Exception ex = new Exception();
+            StringWriter sw = new StringWriter(500);
+            PrintWriter pw = new PrintWriter(sw);
+            ex.printStackTrace(pw);
+            LOG.log(Level.WARNING,sw.toString());
+            return null;
+        }
         initialize();
         DoInitialize d = doInitialize;
         if (d != null && !Thread.holdsLock(support.getLock())) {
+            waitingOnInitVisual = true;
             d.initVisual();
+            waitingOnInitVisual = false;
         }
         return pane;
     }
