@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -58,8 +58,6 @@ import java.awt.dnd.DropTargetDropEvent;
 import java.awt.dnd.DropTargetEvent;
 import java.awt.dnd.DropTargetListener;
 import java.awt.event.InputEvent;
-import java.io.PrintWriter;
-import java.io.StringWriter;
 import java.util.Arrays;
 import java.util.HashSet;
 import java.util.Set;
@@ -73,7 +71,6 @@ import javax.swing.JViewport;
 import javax.swing.SwingConstants;
 import javax.swing.SwingUtilities;
 import javax.swing.TransferHandler;
-import javax.swing.plaf.TextUI;
 import javax.swing.plaf.UIResource;
 import javax.swing.text.Caret;
 import javax.swing.text.Document;
@@ -99,9 +96,7 @@ final class QuietEditorPane extends JEditorPane {
             throw new AssertionError(e);
         }
     }
-
-    private StringBuffer sb;
-
+    
     final static int FIRE = 0x1;
     final static int PAINT = 0x2;
     final static int ALL = FIRE | PAINT;
@@ -127,30 +122,6 @@ final class QuietEditorPane extends JEditorPane {
         setFontHeightWidth(getFont());
     }
     
-    String getLog () {
-        if (sb != null) {
-            return sb.toString();
-        } else {
-            return "";
-        }
-    }
-    
-    @Override
-    public void setUI(TextUI ui) {
-        if (sb == null) {
-            sb = new StringBuffer(1024);
-        }
-        sb.append("\n++ [" + Integer.toHexString(System.identityHashCode(this)) + "]"
-        + " QuietEditorPane.setUI ui:" + ui.getClass().getName()
-        + " th:" + Thread.currentThread().getName());
-        Exception ex = new Exception();
-        StringWriter sw = new StringWriter(500);
-        PrintWriter pw = new PrintWriter(sw);
-        ex.printStackTrace(pw);
-        sb.append("\n" + sw.toString());
-        super.setUI(ui);
-    }
-    
     @Override
     public void setFont(Font font) {
         super.setFont(font);
@@ -276,6 +247,7 @@ final class QuietEditorPane extends JEditorPane {
         }
     }
 
+/* Commented out since virtual height is implemented in the editor (in GapDocumentView)
     @Override
     public Dimension getPreferredSize() {
         //Avoid always typing at the bottom of the screen by
@@ -292,6 +264,7 @@ final class QuietEditorPane extends JEditorPane {
         }
         return result;
     }
+*/
 
     /**
      * Delegating TransferHandler.
