@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -40,7 +40,6 @@
  */
 package org.openide.util.lookup;
 
-import org.openide.util.Enumerations;
 import org.openide.util.Lookup;
 import org.openide.util.lookup.AbstractLookup.Pair;
 import org.openide.util.lookup.AbstractLookup.ReferenceIterator;
@@ -594,7 +593,7 @@ implements Serializable, AbstractLookup.Storage<ArrayList<Class>> {
 
         if (n == null) {
             // not for us
-            return org.openide.util.Enumerations.empty();
+            return emptyEn();
         } else {
             return nodeToEnum(n);
         }
@@ -668,36 +667,15 @@ implements Serializable, AbstractLookup.Storage<ArrayList<Class>> {
             // create a simple enumeration because we do not have children
             Enumeration<Pair> e;
             if (n.items == null) {
-                e = Enumerations.empty();
+                e = emptyEn();
             } else {
                 e = Collections.enumeration(n.items);
             }
             return e;
         }
 
-        // we have found what we need
-        // now we have to just build the enumeration
-        class DeepAndItems implements Enumerations.Processor<Node,Enumeration<Pair>> {
-            public Enumeration<Pair> process(Node n2, Collection<Node> toAdd) {
-                if (n2.children != null) {
-                    toAdd.addAll(n2.children);
-                }
-
-                if ((n2.items == null) || n2.items.isEmpty()) {
-                    return Enumerations.empty();
-                } else {
-                    return Collections.enumeration(n2.items);
-                }
-            }
-        }
-
-        Enumeration<Enumeration<Pair>> en = Enumerations.queue(
-            // initial node is our current one
-            Enumerations.singleton(n), new DeepAndItems()
-        );
-        Enumeration<Pair> all = Enumerations.concat(en);
         // create enumeration of Items
-        return new NeedsSortEnum<Pair>(all);
+        return new NeedsSortEnum(n);
     }
 
     //
@@ -916,7 +894,7 @@ implements Serializable, AbstractLookup.Storage<ArrayList<Class>> {
                 // single item mode
                 interfaces.put(clazz, one);
 
-                return org.openide.util.Enumerations.singleton(one);
+                return singletonEn(one);
             } else {
                 if (items == null) {
                     items = new ArrayList(2);
@@ -931,7 +909,7 @@ implements Serializable, AbstractLookup.Storage<ArrayList<Class>> {
                 return Collections.enumeration((Collection) obj);
             } else {
                 // single item mode
-                return org.openide.util.Enumerations.singleton((Pair)obj);
+                return singletonEn((Pair)obj);
             }
         }
     }
@@ -1194,6 +1172,7 @@ implements Serializable, AbstractLookup.Storage<ArrayList<Class>> {
             return new R(this);
         }
 
+        @Override
         public String toString() {
             return "Node for " + get();
         }
@@ -1241,21 +1220,55 @@ implements Serializable, AbstractLookup.Storage<ArrayList<Class>> {
     }
      // end of R
 
+    static Enumeration<Object> arrayEn(Object[] object) {
+        return Collections.enumeration(Arrays.asList(object));
+    }
+    static <T> Enumeration<T> singletonEn(T object) {
+        return Collections.enumeration(Collections.singleton(object));
+    }
+    static <T> Enumeration<T> emptyEn() {
+        return Collections.enumeration(Collections.<T>emptyList());
+    }
+
     /** Just a marker class to be able to do instanceof and find out
      * that this enumeration is not sorted
      */
-    private static final class NeedsSortEnum<T> implements Enumeration<T> {
-        private Enumeration<T> en;
+    private static final class NeedsSortEnum extends LinkedList<Node>
+    implements Enumeration<Pair> {
+        private Enumeration<Pair> en;
 
-        public NeedsSortEnum(Enumeration<T> en) {
-            this.en = en;
+        public NeedsSortEnum(Node n) {
+            add(n);
+        }
+
+        private boolean ensureNext() {
+            for (;;) {
+                if (en != null && en.hasMoreElements()) {
+                    return true;
+                }
+                if (isEmpty()) {
+                    return false;
+                }
+
+                Node n2 = poll();
+                if (n2.children != null) {
+                    addAll(n2.children);
+                }
+
+                if (n2.items != null && !n2.items.isEmpty()) {
+                    en = Collections.enumeration(n2.items);
+                }
+            }
         }
 
         public boolean hasMoreElements() {
-            return en.hasMoreElements();
+            return ensureNext();
         }
 
-        public T nextElement() {
+        public Pair nextElement() {
+            if (!ensureNext()) {
+                throw new NoSuchElementException();
+            }
             return en.nextElement();
         }
     }
