@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -57,6 +57,7 @@ import org.openide.util.LookupListener;
 
 /** Runs all NbLookupTest tests on ProxyLookup and adds few additional.
  */
+@SuppressWarnings("unchecked") // XXX ought to be corrected, just a lot of them
 public class ProxyLookupTest extends AbstractLookupBaseHid
 implements AbstractLookupBaseHid.Impl {
     public ProxyLookupTest(java.lang.String testName) {
@@ -459,6 +460,7 @@ implements AbstractLookupBaseHid.Impl {
             }
 
             @Override
+            @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
             public boolean equals(Object obj) {
                 if (set != null) {
                     cnt[0]++;
