@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -69,8 +69,8 @@ import org.openide.util.lookup.ProxyLookup;
  * @author Radek Matous
  */
 public final class CategoryModel implements LookupListener {
+    private static final RequestProcessor RP = new RequestProcessor("org.netbeans.modules.options.CategoryModel");  //NOI18N
     private static Reference<CategoryModel> INSTANCE = new WeakReference<CategoryModel>(new CategoryModel());
-    private final RequestProcessor RP = new RequestProcessor();
     private static String currentCategoryID = null;
     private String highlitedCategoryID = null;
     private boolean categoriesValid = true;
@@ -314,7 +314,6 @@ public final class CategoryModel implements LookupListener {
         private OptionsCategory category;
         private OptionsPanelController controller;
         private boolean isUpdated;
-        private HelpCtx helpCtx;
         private JComponent component;
         private Lookup lookup;
         private final String id;
@@ -362,10 +361,6 @@ public final class CategoryModel implements LookupListener {
             return category.getCategoryName();
         }
 
-        public String getTitle() {
-            return category.getTitle();
-        }
-
         private synchronized OptionsPanelController create() {
             if (controller == null) {
                 controller = category.create();
