@@ -39,11 +39,9 @@
 package org.netbeans.api.options;
 
 import javax.swing.Icon;
-import javax.swing.ImageIcon;
 import org.netbeans.spi.options.OptionsCategory;
 import org.netbeans.spi.options.OptionsPanelController;
 import org.openide.util.ImageUtilities;
-import org.openide.util.Utilities;
 
 public final class MyAdvancedCategory extends OptionsCategory {
 
@@ -62,6 +60,7 @@ public final class MyAdvancedCategory extends OptionsCategory {
         return "My Advanced Category";
     }
 
+    @SuppressWarnings("deprecation")
     public OptionsPanelController create() {
         if(controller == null) {
             controller = OptionsPanelController.createAdvanced("MyAdvancedCategory");
